// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.annotatedlineeditor;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.source.Annotation;

/**
 * An <code>Annotation</code> which spans an entire line and holds an object associated
 * with the line in an <code>IEntityContainer</code>. 
 * 
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class LineAnnotation extends Annotation implements IEntityContainer {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(LineAnnotationModel.class);

	public static final String TYPE_GENERIC = Annotation.TYPE_UNKNOWN;
	
	private Object entity;
	private ILineDisplayStrategy lineDisplayStrategy;

	private boolean dirty = false;
	private boolean markedAsMerged;
	private boolean markedAsNew;
	private Object mergeTarget;

	
	public LineAnnotation(Object entity, ILineDisplayStrategy lineDisplayStrategy) {
		this.entity = entity;
		this.lineDisplayStrategy = lineDisplayStrategy;
		setType(TYPE_GENERIC);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.Annotation#getText()
	 */
	@Override
	public String getText() {
		return getEditableText();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.Annotation#setText(java.lang.String)
	 */
	@Override
	public void setText(String text) {
		lineDisplayStrategy.setText(text, entity);
		dirty  = true;		
		super.setText(text);
	}
	
	@Override
	public String toString() {
		// For debugging
		return getText();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getAttachedEntities()
	 */
	public Set getAttachedEntities() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getEntity()
	 */
	public Object getEntity() {
		return entity;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getEditableText()
	 */
	public String getEditableText() {
		return lineDisplayStrategy.getText(entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#getMergeTarget()
	 */
	public Object getMergeTarget() {
		return mergeTarget;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isMarkedAsDeleted()
	 */
	public boolean isMarkedAsDeleted() {
		return super.isMarkedDeleted();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isMarkedAsMerged()
	 */
	public boolean isMarkedAsMerged() {
		return markedAsMerged;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isMarkedAsNew()
	 */
	public boolean isMarkedAsNew() {
		return markedAsNew;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#markAsDeleted()
	 */
	public void markAsDeleted() {
		super.markDeleted(true);
		if (!isMarkedAsMerged()) {
			lineDisplayStrategy.setStatusMessage(entity + " deleted.", entity);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#markAsMerged(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public void markAsMerged(Object mergeTarget) {
		this.mergeTarget = mergeTarget;
		markedAsMerged = true;
		lineDisplayStrategy.setStatusMessage("'" + entity + "' merged into '" + mergeTarget + "'.", entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#markAsNew()
	 */
	public void markAsNew(boolean isNew) {
		markedAsNew = isNew;
		lineDisplayStrategy.setStatusMessage("New entity created.", entity);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#setDirty(boolean)
	 */
	public void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityContainer#isDirty()
	 */
	public boolean isDirty() {
		return dirty;
	}
	
	/**
	 * @return
	 */
	@Deprecated
	public Object getPropertySource() {
		return lineDisplayStrategy.getPropertySource(entity);
	}

	/**
	 * @return
	 */
	public String getHoverText() {
		return lineDisplayStrategy.getHoverText(entity);
	}
}