// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 * @created 17.08.2009
 * @version 1.0
 */
public class BulkEditorMarkerPreferenceComposite extends Composite {
	private static final Logger logger = Logger.getLogger(BulkEditorMarkerPreferenceComposite.class);
	
	private AbstractBulkEditorInput editorInput;
	
	Map<Button, MarkerType> markerTypeButtons = new HashMap<Button, MarkerType>();
		
	/**
	 * @param parent
	 * @param style
	 */
	public BulkEditorMarkerPreferenceComposite(AbstractBulkEditorInput editorInput, Composite parent, int style) {
		super(parent, style);
		this.editorInput = editorInput;
		createControl();
	}

	/**
	 * @param markerType 
	 * @return 
	 * 
	 */
	private boolean getEditMarkerPreference(MarkerType markerType) {
		return PreferencesUtil.getEditMarkerTypePreference(editorInput, markerType);
	}

	private void setEditMarkerPreference(MarkerType markerType, boolean edit) {
		PreferencesUtil.setEditMarkerTypePreference(editorInput, markerType, edit);
	}
		
	protected void createControl() {
		
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 12;
		setLayout(gridLayout);
		
		for (final MarkerType markerType : VocabularyStore.getNonTechnicalMarkerTypes()) {
			if (editorInput.isMarkerTypeEditingEnabled(markerType)) {
				final Button button = new Button(this, SWT.CHECK);
				button.setText("Edit " + markerType.getLabel() + " markers");
				button.addSelectionListener(new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						setEditMarkerPreference(markerType, button.getSelection());
					}
				});
				button.setSelection(getEditMarkerPreference(markerType));
				
				markerTypeButtons.put(button, markerType);
			}
		}
	}
}