// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.reference.Article;
import eu.etaxonomy.cdm.model.reference.Book;
import eu.etaxonomy.cdm.model.reference.BookSection;
import eu.etaxonomy.cdm.model.reference.CdDvd;
import eu.etaxonomy.cdm.model.reference.Database;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.InProceedings;
import eu.etaxonomy.cdm.model.reference.Journal;
import eu.etaxonomy.cdm.model.reference.Patent;
import eu.etaxonomy.cdm.model.reference.PersonalCommunication;
import eu.etaxonomy.cdm.model.reference.PrintSeries;
import eu.etaxonomy.cdm.model.reference.Proceedings;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.cdm.model.reference.Report;
import eu.etaxonomy.cdm.model.reference.Thesis;
import eu.etaxonomy.cdm.model.reference.WebPage;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;

/**
 * @author p.ciardelli
 * @created 07.07.2009
 * @version 1.0
 */
public class ReferenceCreator implements IEntityCreator<ReferenceBase> {
	private static final Logger logger = Logger
			.getLogger(ReferenceCreator.class);

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.IEntityCreator#createEntity(java.lang.String)
	 */
	public ReferenceBase createEntity(String text) {
		return createEntity(Generic.class, text);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator#createEntity(java.lang.Class, java.lang.String)
	 */
	public ReferenceBase createEntity(Object key, String text) {
		ReferenceBase reference = null;
		
		// TODO replace w more generic method when cdmlib matures accordingly
		if  (ReferenceType.Article.getMessage().equals(key)) {
			reference = Article.NewInstance();
		}
		if  (ReferenceType.Book.getMessage().equals(key)) {
			reference = Book.NewInstance();
		}
		if  (ReferenceType.BookSection.getMessage().equals(key)) {
			reference = BookSection.NewInstance();
		}
		if  (ReferenceType.CdDvd.getMessage().equals(key)) {
			reference = CdDvd.NewInstance();
		}
		if  (ReferenceType.Database.getMessage().equals(key)) {
			reference = Database.NewInstance();
		}
		if  (ReferenceType.Generic.getMessage().equals(key)) {
			reference = Generic.NewInstance();
		}
		if  (ReferenceType.InProceedings.getMessage().equals(key)) {
			reference = InProceedings.NewInstance();
		}
		if  (ReferenceType.Journal.getMessage().equals(key)) {
			reference = Journal.NewInstance();
		}
		if  (ReferenceType.Map.getMessage().equals(key)) {
			reference = eu.etaxonomy.cdm.model.reference.Map.NewInstance();
		}
		if  (ReferenceType.Patent.getMessage().equals(key)) {
			reference = Patent.NewInstance();
		}
		if  (ReferenceType.PersonalCommunication.getMessage().equals(key)) {
			reference = PersonalCommunication.NewInstance();
		}
		if  (ReferenceType.PrintSeries.getMessage().equals(key)) {
			reference = PrintSeries.NewInstance();
		}
		if  (ReferenceType.Proceedings.getMessage().equals(key)) {
			reference = Proceedings.NewInstance();
		}
		if  (ReferenceType.Report.getMessage().equals(key)) {
			reference = Report.NewInstance();
		}
		if  (ReferenceType.Thesis.getMessage().equals(key)) {
			reference = Thesis.NewInstance();
		}
		if  (ReferenceType.WebPage.getMessage().equals(key)) {
			reference = WebPage.NewInstance();
		}
		if (reference == null) {
			logger.warn("Reference type " + key + " not found. Creating reference with default type.");
			reference = Generic.NewInstance();
		}
		reference.setTitleCache(text);
		return reference;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator#getClassLabelPairs()
	 */
	public Map<Object, String> getKeyLabelPairs() {
		Map<Object, String> result = new HashMap<Object, String>();
		for (ReferenceType type : ReferenceType.values()) {
			result.put(type.getMessage(), type.getMessage());
		}
		return result;
	}
}
