// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.IParameterValues;
import org.eclipse.ui.IEditorInput;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.AgentEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.NameEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.OccurrenceEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.ReferenceEditorInput;

/**
 * @author p.ciardelli
 * @created 20.08.2009
 * @version 1.0
 */
public class BulkEditorInputTypeValues implements IParameterValues {
	private static final Logger logger = Logger
			.getLogger(BulkEditorInputTypeValues.class);

	public enum BulkEditorInputType{
		REFERENCE("Reference", ReferenceEditorInput.ID),
		NAME("Name", NameEditorInput.ID),
		AGENT("Authors and Author Teams", AgentEditorInput.ID),
		OCCURRENCE("Specimen or Observations", OccurrenceEditorInput.ID);
		
		public String id;
		public String label;
		
		BulkEditorInputType(String label, String id){
			this.id = id;
			this.label = label;
		}
		
		public static BulkEditorInputType getById(String id){
			
			for(BulkEditorInputType type : values()){
				if(id.equals(type.id)){
					return type;
				}
			}
			
			return null;
		}
		
		public static BulkEditorInputType getByType(Class clazz){
			if (ReferenceBase.class.isAssignableFrom(clazz)) {
				return REFERENCE;
			}
			if (TaxonNameBase.class.isAssignableFrom(clazz)) {
				return NAME;
			}
			if (AgentBase.class.isAssignableFrom(clazz)) {
				return AGENT;
			}
			if (SpecimenOrObservationBase.class.isAssignableFrom(clazz)){
				return OCCURRENCE;
			}
			return null;
		}
		
		public static BulkEditorInputType getByInput(IEditorInput input) {
			if (input instanceof ReferenceEditorInput) {
				return REFERENCE;
			}
			if (input instanceof NameEditorInput) {
				return NAME;
			}
			if (input instanceof AgentEditorInput) {
				return AGENT;
			}
			if (input instanceof OccurrenceEditorInput) {
				return OCCURRENCE;
			}
			return null;
		}
		
		public static AbstractBulkEditorInput getInput(BulkEditorInputType inputType){
			switch (inputType) {
			case REFERENCE:
				return new ReferenceEditorInput();
			case NAME:
				return new NameEditorInput();
			case AGENT:
				return new AgentEditorInput();
			case OCCURRENCE:
				return new OccurrenceEditorInput();
			default:
				return null;
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IParameterValues#getParameterValues()
	 */
	public Map getParameterValues() {
		final Map values = new HashMap();
		
		// TODO user role determines which editor inputs are returned
		for (BulkEditorInputType inputType : BulkEditorInputType.values()){
			// do not add the occurrence bulk editor
			if(! inputType.label.equals(BulkEditorInputType.OCCURRENCE.label))
				values.put(inputType.label, inputType.id);
		}		
		
		return values;
	}
}
