// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 * @created 20.10.2009
 * @version 1.0
 */
public class SetMarkerContributionItems extends CompoundContributionItem {
	private static final Logger logger = Logger
			.getLogger(SetMarkerContributionItems.class);

	/* (non-Javadoc)
	 * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
	 */
	@Override
	protected IContributionItem[] getContributionItems() {
			
		List<IContributionItem> contributions = new ArrayList<IContributionItem>();
		
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (editor.getEditorInput() instanceof AbstractBulkEditorInput) {
			AbstractBulkEditorInput input = (AbstractBulkEditorInput) editor.getEditorInput();		
			for (final MarkerType markerType : VocabularyStore.getNonTechnicalMarkerTypes()) {
				if (input.isMarkerTypeEditingEnabled(markerType) && PreferencesUtil.getEditMarkerTypePreference(input, markerType)) {
					contributions.add(new ContributionItem() {
						/* (non-Javadoc)
						 * @see org.eclipse.jface.action.ContributionItem#fill(org.eclipse.swt.widgets.Menu, int)
						 */
						@Override
						public void fill(Menu menu, int index) {
							MenuItem subMenuItem = new MenuItem(menu, SWT.CASCADE, index);
							subMenuItem.setText("Set Flag - \"" + markerType.getLabel() + "\"");
							Menu subMenu = new Menu(menu.getShell(), SWT.DROP_DOWN);
							subMenuItem.setMenu(subMenu);
							
							MenuItem trueItem = new MenuItem(subMenu, SWT.PUSH);
							trueItem.setText("true");
							trueItem.addSelectionListener(new SelectionAdapter() {
								public void widgetSelected(SelectionEvent e) {
									doSetFlagCommand(markerType, true);
								}	
							});
							
							MenuItem falseItem = new MenuItem(subMenu, SWT.PUSH);
							falseItem.setText("false");
							falseItem.addSelectionListener(new SelectionAdapter() {
								public void widgetSelected(SelectionEvent e) {
									doSetFlagCommand(markerType, false);
								}						
							});
						}
					});
				}
			}
		}
		return (IContributionItem[]) contributions.toArray(new IContributionItem[contributions.size()]);
	}

	private void doSetFlagCommand(MarkerType markerType, boolean flag) {
		try {
			IHandlerService handlerService = 
				(IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
			Event event = new Event();
			event.data = markerType;
			if (flag) {
				handlerService.executeCommand("taxeditor-bulkeditor.command.setMarkerFlagTrue", event);
			} else {
				handlerService.executeCommand("taxeditor-bulkeditor.command.setMarkerFlagFalse", event);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}					
}
