// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IdentifiableEntityComparator;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorQuery;
import eu.etaxonomy.taxeditor.bulkeditor.command.BulkEditorInputTypeValues.BulkEditorInputType;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractBulkEditorInput<T extends IdentifiableEntity> extends PropertyChangeSupport implements IEditorInput, IEntityPersistenceService {
	private static final Logger logger = Logger
			.getLogger(AbstractBulkEditorInput.class);
	/**
	 * 
	 */
	private static final long serialVersionUID = 416414530232743735L;
	private String mode;
	private UUID entityUuid;

	/**
	 * @param sourceBean
	 */
	public AbstractBulkEditorInput(Object sourceBean) {
		super(sourceBean);
	}
	
	public static final String QUERY_CHANGED = "query_changed";
	
	static public AbstractBulkEditorInput NewInstance(BulkEditorInputType inputType) {
		return BulkEditorInputType.getInput(inputType);
	}
	
	public static AbstractBulkEditorInput NewInstance(IdentifiableEntity entity) {
		
		
		BulkEditorInputType inputType = BulkEditorInputType.getByType(entity.getClass());
		
		AbstractBulkEditorInput editorInput = NewInstance(inputType);
		
		editorInput.setEntityUuid(entity.getUuid());
		
		return editorInput;
	}
	
	/**
	 * Queries the service for entities that match the criterias defined in the given 
	 * search configurator
	 * 
	 * @param configurator
	 * @return
	 */
	public List<T> listEntities(){
		if(getEntityUuid() != null){
			List<T> entityList = new ArrayList<T>();
			T entity = loadEntity(getEntityUuid());
			entityList.add(entity);
			return entityList;
		}
		
		if(query != null){
			
			String queryString = query.getSearchString();
			IIdentifiableEntityServiceConfigurator configurator = query.getSearchConfigurator();
			Comparator queryComparator = (query.getComparator() != null) ? query.getComparator() : new IdentifiableEntityComparator();
			
			List<T> entityList = listEntities(configurator);
			
			Collections.sort(entityList, queryComparator);			
			
			// reset query
			setQuery(null);
			
			return entityList;
		}
		return null;
	}
	
	protected abstract List<T> listEntities(IIdentifiableEntityServiceConfigurator configurator);
	
	protected abstract T loadEntity(UUID entityUuid);

	private void setEntityUuid(UUID entityUuid){
		this.entityUuid = entityUuid;
	}
	
	public UUID getEntityUuid() {
		return entityUuid;
	}

	private IBulkEditorQuery query;
			
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	public boolean exists() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	public ImageDescriptor getImageDescriptor() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	public IPersistableElement getPersistable() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		return null;
	}

	/**
	 * @param bulkEditorQuery
	 */
	public void setQuery(IBulkEditorQuery bulkEditorQuery) {
				
		this.query = bulkEditorQuery;
		
		if (query != null) {
			firePropertyChange(QUERY_CHANGED, null, null);
		}
	}
	
	public IBulkEditorQuery getQuery() {
		return query;
	}
	
	public boolean isMergingEnabled() {
		return false;
	}

	/**
	 * @param markerType
	 * @return
	 */
	public boolean isMarkerTypeEditingEnabled(MarkerType markerType) {
		return false;
	}
	
	public boolean isSingleEntityMode(){
		return getEntityUuid() != null;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#merge(eu.etaxonomy.cdm.model.common.IdentifiableEntity, eu.etaxonomy.cdm.model.common.IdentifiableEntity)
	 */
	public boolean merge(Object entity, Object mergeTarget) {
		logger.info("Merge "  + entity + " into " + mergeTarget);
		if (entity instanceof ReferenceBase) {
			try {
				CdmStore.getCommonService().merge((ReferenceBase) mergeTarget, (ReferenceBase) entity, null);
			} catch (MergeException e) {
				logger.error("Problems merging objects", e);
			}
		}
		return true;
	};
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#create(eu.etaxonomy.cdm.model.common.IdentifiableEntity)
	 */
	public boolean create(Object entity) {
		return save(entity);		
	};
}