// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.taxeditor.bulkeditor.command.BulkEditorInputTypeValues.BulkEditorInputType;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AgentEditorInput extends AbstractBulkEditorInput<AgentBase> {

	public static final String ID = "bulkeditor.input.author";
	
	private static AgentEditorInput instance;

	/**
	 * @param sourceBean
	 */
	public AgentEditorInput() {
		super(new Object());
	}

	/**
	 * @return the iD
	 */
	public static String getID() {
		return ID;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		return BulkEditorInputType.AGENT.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return getName();
	}

	/**
	 * @return
	 */
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new AgentEditorInput();
		}
		return instance;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMergingEnabled()
	 */
	@Override
	public boolean isMergingEnabled() {
		return false;
	}

	@Override
	public List listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.findTeamOrPersons(configurator);
	}

	@Override
	public AgentBase loadEntity(UUID uuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{}); 
		return CdmStore.getAgentService().load(uuid, propertyPaths);
	}

	public boolean delete(Object entity) {
		return CdmStore.getAgentService().delete((AgentBase) entity) != null;			
	}

	public boolean save(Object entity) {
		return CdmStore.getAgentService().saveOrUpdate((AgentBase) entity) != null;
	}
}