// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.bulkeditor.command.BulkEditorInputTypeValues.BulkEditorInputType;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 20.08.2009
 * @version 1.0
 */
public class NameEditorInput extends AbstractBulkEditorInput<TaxonNameBase> {
	private static final Logger logger = Logger
			.getLogger(NameEditorInput.class);
	
	private static NameEditorInput instance;

	/**
	 * @param sourceBean
	 */
	public NameEditorInput() {
		super(new Object());
	}

	public static final String ID = "bulkeditor.input.name";
	
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new NameEditorInput();
		}
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		return BulkEditorInputType.NAME.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return getName();
	}

	/**
	 * @return
	 */
	public static Object getID() {
		return ID;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMergingEnabled()
	 */
	@Override
	public boolean isMergingEnabled() {
		return false;
	}

	@Override
	public List listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.findNames(configurator);
	}

	@Override
	public TaxonNameBase loadEntity(UUID uuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{}); 
		return CdmStore.getNameService().load(uuid, propertyPaths);
	}

	public boolean delete(Object entity) {
		return CdmStore.getNameService().delete((TaxonNameBase) entity) != null;
	}

	public boolean save(Object entity) {
		return CdmStore.getNameService().saveOrUpdate((TaxonNameBase) entity) != null;
	}

}
