// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.bulkeditor.command.BulkEditorInputTypeValues.BulkEditorInputType;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jun 16, 2010
 * @version 1.0
 */
public class OccurrenceEditorInput extends AbstractBulkEditorInput<SpecimenOrObservationBase> {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1014860873939989973L;

	private static final Logger logger = Logger
			.getLogger(OccurrenceEditorInput.class);

	public static final String ID = "bulkeditor.input.occurrence";

	private static OccurrenceEditorInput instance;
	
	public OccurrenceEditorInput() {
		super(new Object());
	}
	
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new OccurrenceEditorInput();
		}
		return instance;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#listEntities(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
	 */
	@Override
	protected List listEntities(
			IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.findOccurrences(configurator);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput#loadEntity(java.util.UUID)
	 */
	@Override
	protected SpecimenOrObservationBase loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{}); 
		return CdmStore.getOccurrenceService().load(entityUuid, propertyPaths);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		return BulkEditorInputType.OCCURRENCE.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return getName();
	}
	
	public static Object getId(){
		return ID;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#delete(java.lang.Object)
	 */
	public boolean delete(Object entity) {
		return CdmStore.getOccurrenceService().delete((SpecimenOrObservationBase) entity) != null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#save(java.lang.Object)
	 */
	public boolean save(Object entity) {
		return CdmStore.getOccurrenceService().save((SpecimenOrObservationBase) entity) != null;
	}
}
