// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.bulkeditor.command.BulkEditorInputTypeValues.BulkEditorInputType;
import eu.etaxonomy.taxeditor.store.CdmStore;


/**
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class ReferenceEditorInput extends AbstractBulkEditorInput<ReferenceBase> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3806243517765126749L;

	public static final String ID = "bulkeditor.input.reference";
	
	private static ReferenceEditorInput instance;

	/**
	 * @param sourceBean
	 */
	public ReferenceEditorInput() {
		super(new Object());
	}

	/**
	 * @return the iD
	 */
	public static String getID() {
		return ID;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		return BulkEditorInputType.REFERENCE.label;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return getName();
	}

	/**
	 * @return
	 */
	public static AbstractBulkEditorInput getInstance() {
		if (instance == null) {
			instance = new ReferenceEditorInput();
		}
		return instance;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMergingEnabled()
	 */
	@Override
	public boolean isMergingEnabled() {
		return true;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput#isMarkerTypeEditingEnabled(eu.etaxonomy.cdm.model.common.MarkerType)
	 */
	@Override
	public boolean isMarkerTypeEditingEnabled(MarkerType markerType) {
		if (MarkerType.PUBLISH().equals(markerType)) {
			return true;
		}
		return super.isMarkerTypeEditingEnabled(markerType);
	}

	// Entity persistence
	
	@Override
	public List listEntities(IIdentifiableEntityServiceConfigurator configurator) {
		return CdmStore.findReferences(configurator);
	}

	@Override
	public ReferenceBase loadEntity(UUID entityUuid) {
		List<String> propertyPaths = Arrays.asList(new String[]{"*", "inReference.*", "authorTeam.*"}); 
		return CdmStore.getReferenceService().load(entityUuid, propertyPaths);
	}

	public boolean delete(Object entity) {
		return CdmStore.getReferenceService().delete((ReferenceBase) entity) != null;
	}

	public boolean save(Object entity) {
		return CdmStore.getReferenceService().saveOrUpdate((ReferenceBase) entity) != null;
	}
}