// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.referencingobjects;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.model.DescriptionHelper;
import eu.etaxonomy.taxeditor.model.LineSelection;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 08.07.2009
 * @version 1.0
 */
public class ReferencingObjectsView extends ViewPart implements ISelectionListener{
	private static final Logger logger = Logger
			.getLogger(ReferencingObjectsView.class);

	public static final String ID = "eu.etaxonomy.taxeditor.bulkeditor.referencingobjectsview";
	
	private TableViewer referencingObjectsViewer;

	private ISelectionService selectionService;

	private CdmBase referencedObject;

	private ArrayList<CdmBase> referencingObjects;

	
	public ReferencingObjectsView() {}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		selectionService = getSite().getWorkbenchWindow().getSelectionService();
		selectionService.addSelectionListener(this);
		
		createReferencingObjectsViewer(parent);
	}

    /**
     * @param parent
     */
    private void createReferencingObjectsViewer(Composite parent) {
        referencingObjectsViewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
									| SWT.V_SCROLL | SWT.FULL_SELECTION);
        createColumns(referencingObjectsViewer);
        referencingObjectsViewer.setContentProvider(new ReferencingObjectsContentProvider());
        referencingObjectsViewer.setLabelProvider(new ReferencingObjectsLabelProvider());
        referencingObjectsViewer.setInput(new ArrayList<CdmBase>());

    }
	
	// This will create the columns for the table
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = {"Class", "Description" };
		int[] bounds = { 100, 200 };

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		logger.warn("set focus");
		referencingObjectsViewer.getControl().setFocus();
	}

	/**
	 * 
	 */
	public void updateReferencingObjects(final IdentifiableEntity entity) {
//        IEditorPart editor = getSite().getPage().getActiveEditor();
//        ISelection selection = editor.getEditorSite().getSelectionProvider()
//                                     .getSelection();
//
//        if ((selection != null) && selection instanceof LineSelection) {
//        	final Object entity = ((IStructuredSelection) selection).getFirstElement();
		
		Job job = new Job("Update Referencing Objects") {
			
			@Override
			protected IStatus run(IProgressMonitor monitor) {
				monitor.beginTask("Calculating referencing objects", 10);
				ConversationHolder conversation = CdmStore.createConversation();
				conversation.bind();
				monitor.worked(3);
				doUpdateReferencingObjects(entity);
				monitor.done();
				conversation.close();
				return Status.OK_STATUS;
			}
		};
	
    	IRunnableWithProgress runnable = new IRunnableWithProgress() {
			
			public void run(IProgressMonitor monitor) throws InvocationTargetException,
					InterruptedException {
				monitor.beginTask("Calculating referencing objects", 10);
				ConversationHolder conversation = CdmStore.createConversation();
				conversation.bind();
				monitor.worked(3);
				doUpdateReferencingObjects(entity);
				monitor.done();
				conversation.close();
			}
		};
        	
//			BulkEditorUtil.busyCursorWhile(runnable);
			
//			BulkEditorUtil.runAsynchroneously(runnable);
			
		job.schedule();
			
		updateView();
	}

	private void doUpdateReferencingObjects(Object entity) {
        try {
        	if(entity instanceof CdmBase){
	        	referencedObject = (CdmBase) entity;
	        	Set<CdmBase> setOfReferencingObjects = CdmStore.getCommonService().getReferencingObjects(referencedObject);
	        	
	        	referencingObjects = new ArrayList<CdmBase>(setOfReferencingObjects);
	        	
	    		Collections.sort(referencingObjects, new ReferencingClassComparator());
        	}
        } catch (Exception e) {
            logger.error("Error retrieving referencing objects", e);
        }
	}
	
	class ReferencingClassComparator implements Comparator<CdmBase> {

		/* (non-Javadoc)
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
		public int compare(CdmBase o1, CdmBase o2) {
			String string1 = o1.getClass().getSimpleName();
			String string2 = o2.getClass().getSimpleName();
			int classCompare = string1.compareToIgnoreCase(string2);
			if (classCompare == 0) {
				string1 = DescriptionHelper.getObjectDescription(o1);
				string2 = DescriptionHelper.getObjectDescription(o2);
				return string1.compareToIgnoreCase(string2);
			} else {
				return classCompare;
			}
		}
	}
	
	/**
	 * 
	 */
	private void updateView() {
		referencingObjectsViewer.setInput(referencingObjects);
		setContentDescription("'" + referencedObject + "' is referenced by:");
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(! (part instanceof BulkEditor)){
			return;
		}
		
		if(! (selection instanceof LineSelection)){
			return;
		}
		LineSelection lineSelection = (LineSelection) selection;
		
		if(lineSelection.size() != 1){
			return;
		}
			
		if(! (lineSelection.getFirstElement() instanceof IdentifiableEntity)){
			return;
		}
		
		updateReferencingObjects((IdentifiableEntity) lineSelection.getFirstElement());
	}
	
	@Override
	public void dispose() {
		selectionService.removePostSelectionListener(this);
		super.dispose();
	}
	
}
