// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.editor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Sep 10, 2009
 * @version 1.0
 */
public class FilteredAgentSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<AgentBase> {
	private static final Logger logger = Logger
			.getLogger(FilteredAgentSelectionDialog.class);
	
	public static AgentBase selectAgent(Shell shell, ConversationHolder conversation, AgentBase entity) {
		FilteredAgentSelectionDialog dialog = new FilteredAgentSelectionDialog(shell, conversation,
				"Choose Agent", false, FilteredAgentSelectionDialog.class.getCanonicalName(), entity);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	* @param shell
	* @param title
	 * @param agent 
	*/
	protected FilteredAgentSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, String settings, AgentBase agent) {
		super(shell, conversation, title, multi, settings, agent);
	}
	
	
	/**
	 * @author p.ciardelli
	 * @created 18.09.2009
	 * @version 1.0
	 */
	public class DetailsLabelProvider extends LabelProvider {
		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
		 */
		public String getText(Object element) {
			AgentBase agent = getCdmObjectByUuid(((UuidAndTitleCache<AgentBase>) element).getUuid());
			if (agent instanceof INomenclaturalAuthor) {
				return "Nomenclatural title: '" + ((INomenclaturalAuthor) agent).getNomenclaturalTitle() + "'"; 
			} else {
				return "'" + agent.getTitleCache() + "' is not a nomenclatural author."; 
			}
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObect(java.util.UUID)
	 */
	@Override
	protected AgentBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getAgentService().load(cdmUuid);
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard() {
		return null;
	}

	@Override
	protected String getNewWizardLinkText() {
		return null;
	}
	
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		// nothing at the moment
		return null;
	}

	@Override
	protected void initModel() {
		model = CdmStore.getAgentService().getUuidAndTitleCache();
	}


}