// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.editor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.editor.newWizard.NewNonViralNameWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public class FilteredNameSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<TaxonNameBase> {
	private static final Logger logger = Logger
			.getLogger(FilteredNameSelectionDialog.class);

	/**
	 * Creates a filtered selection dialog to select a name.
	 * 
	 * @param shell
	 * 				The shell for displaying this widget
	 * @param name
	 * 				A name that should be selected when the dialog opens
	 * @return
	 * 				A name object upon selection
	 */
	public static TaxonNameBase selectName(Shell shell, ConversationHolder conversation, TaxonNameBase name) {
		FilteredNameSelectionDialog dialog = new FilteredNameSelectionDialog(shell, conversation,
				"Choose a name", false, name);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	* @param shell
	* @param title
	 * @param name 
	*/
	protected FilteredNameSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, TaxonNameBase name) {
		super(shell, conversation, title, multi, FilteredNameSelectionDialog.class.getCanonicalName(), name);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected TaxonNameBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getNameService().load(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		model = CdmStore.getNameService().getUuidAndTitleCache();		
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard() {
		return new NewNonViralNameWizard();
	}

	@Override
	protected String getNewWizardLinkText() {
		return "Click <A>here</A> to create a new name.";
	}
}
