// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.strategy.cache.reference.ReferenceBaseDefaultCacheStrategy;
import eu.etaxonomy.taxeditor.editor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.editor.newWizard.NewReferenceWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 04.06.2009
 * @version 1.0
 */
public class FilteredReferenceSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<ReferenceBase> {
	private static final Logger logger = Logger
			.getLogger(FilteredReferenceSelectionDialog.class);
	
	/**
	* @param reference 
	 * @param taxon
	* @return
	*/
	public static ReferenceBase selectReference(Shell shell, ConversationHolder conversation, ReferenceBase reference) {
		FilteredReferenceSelectionDialog dialog = new FilteredReferenceSelectionDialog(shell, conversation,
				"Choose a reference", false, (ReferenceBase) reference);
		return getSelectionFromDialog(dialog);
	}

	
	/**
	* @param shell
	* @param title
	 * @param reference 
	*/
	protected FilteredReferenceSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, ReferenceBase reference) {
		super(shell, conversation, title, multi, FilteredReferenceSelectionDialog.class.getCanonicalName(), null);
	}
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected ReferenceBase getPersistentObject(UUID cdmUuid) {
		return CdmStore.getReferenceService().load(cdmUuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {		
		model = CdmStore.getReferenceService().getUuidAndTitleCache();
	}
	

	@Override
	protected String getTitle(ReferenceBase cdmObject) {
		if(cdmObject.getAuthorTeam() == null){
			return super.getTitle(cdmObject);
		}else{
			return ReferenceBaseDefaultCacheStrategy.putAuthorToEndOfString(cdmObject.getTitleCache(), cdmObject.getAuthorTeam().getTitleCache());
		}
	}
	
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard() {
		return new NewReferenceWizard();
	}

	@Override
	protected String getNewWizardLinkText() {
		return "Click <A>here</A> to create a new reference.";
	}
	
}