/**
 * 
 */
package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.editor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 *
 */
public class FilteredTaxonNodeSelectionDialog extends AbstractFilteredCdmResourceSelectionDialog<TaxonNode> implements SelectionListener{	
	
	private static final Logger logger = Logger
			.getLogger(FilteredTaxonNodeSelectionDialog.class);

	
	/**
	 * @param taxon
	 * @return
	 */
	public static TaxonNode selectTaxonNode(Shell shell, String title, List<UUID> excludeTaxa, TaxonNode node) {
		FilteredTaxonNodeSelectionDialog dialog = new FilteredTaxonNodeSelectionDialog(shell, 
				null,
				title,
				excludeTaxa,
				false,
				node);
		return getSelectionFromDialog(dialog);
	}

	private Combo classificationSelectionCombo;

	private List<TaxonomicTree> classifications;
	
	private TaxonomicTree selectedClassification;
	

	/**
	 * 
	 * 
	 * @param shell
	 * @param title
	 */
	protected FilteredTaxonNodeSelectionDialog(Shell shell, ConversationHolder conversation, String title, List<UUID> excludeTaxa, boolean multi, TaxonNode node) {
		super(shell, conversation, title, multi, FilteredTaxonNodeSelectionDialog.class.getCanonicalName(), node);
		
		ILabelProvider labelProvider = new FilteredCdmResourceLabelProvider();

		setListLabelProvider(labelProvider);
		setDetailsLabelProvider(labelProvider);
		
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		return createTreeSelectionCombo(parent);
	}
	
	/*
	 * currently disabled tree selection composite
	 */
	private Control createTreeSelectionCombo(Composite parent){
//		classifications = CdmStore.getTaxonTreeService().list(null, null, null, null, null);
		
		Composite taxonomicTreeSelection = new Composite(parent, SWT.NULL);
		taxonomicTreeSelection.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		GridLayout layout = new GridLayout();
		taxonomicTreeSelection.setLayout(layout);
		
		Label label = new Label(taxonomicTreeSelection, SWT.NULL);
		// TODO not working is not really true but leave it there to remind everyone that this is under construction
		label.setText("Select Taxonomic Tree (not completely working at the moment)");
		classificationSelectionCombo = new Combo(taxonomicTreeSelection, SWT.BORDER | SWT.READ_ONLY);
		classificationSelectionCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));
		
		for(TaxonomicTree tree : classifications){
			classificationSelectionCombo.add(tree.getName().getText(), classifications.indexOf(tree));
		}
		
		classificationSelectionCombo.select(classifications.indexOf(selectedClassification));
		
		// TODO remember last selection
		
		classificationSelectionCombo.addSelectionListener(this);
		
		return taxonomicTreeSelection;
	}

	@Override
	protected String getTitle(TaxonNode taxonNode) {
		return taxonNode.getTaxon().getTitleCache();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected TaxonNode getPersistentObject(UUID uuid) {
		return CdmStore.getTaxonTreeService().getTaxonNodeByUuid(uuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		// default to first tree 
		// TODO this will be problematic and can only be seen as workaround 
		
		
		if(classifications == null){
			classifications = CdmStore.getTaxonTreeService().list(null, null, null, null, null);
			selectedClassification = classifications.iterator().next();
		}
		
		model = CdmStore.getTaxonTreeService().getTaxonNodeUuidAndTitleCacheOfAcceptedTaxaByTaxonomicTree(selectedClassification);
	}

	@Override
	protected AbstractNewEntityWizard getNewEntityWizard() {
		return null;
	}

	@Override
	protected String getNewWizardLinkText() {
		return null;
	}
	
	public void widgetSelected(SelectionEvent e) {
		selectedClassification = classifications.get(classificationSelectionCombo.getSelectionIndex());
		refresh();
	}

	public void widgetDefaultSelected(SelectionEvent e) {}


}