// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jun 18, 2010
 * @version 1.0
 */
public class CdmDataTransfer extends ByteArrayTransfer {
	private static final Logger logger = Logger
			.getLogger(CdmDataTransfer.class);

	private static int SEPARATOR = -10;
	private static int EOF = -20;
	
	private static final String TYPE_NAME = "cdmdata-transfer-format";
	private static final int TYPE_ID = registerType (TYPE_NAME);		
	private static final CdmDataTransfer instance = new CdmDataTransfer();
	
	public static synchronized CdmDataTransfer getInstance() {
		return instance;
	}

	private ICdmBase cdmBase;
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.dnd.Transfer#getTypeIds()
	 */
	@Override
	protected int[] getTypeIds() {
		return new int[] { TYPE_ID }; 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.dnd.Transfer#getTypeNames()
	 */
	@Override
	protected String[] getTypeNames() {
		return new String[]{ TYPE_NAME };
	}

	@Override
	protected void javaToNative(Object object, TransferData transferData) {
		byte[] bytes = toByteArray((ICdmBase[]) object);
		if (bytes != null)
			super.javaToNative(bytes, transferData);
	}



	@Override
	protected Object nativeToJava(TransferData transferData) {
		byte[] bytes = (byte[]) super.nativeToJava(transferData);
		return fromByteArray(bytes);
	}


	private byte[] toByteArray(ICdmBase[] cdmBaseObjects) {
		/**
		 * Transfer data is an array of gadgets. Serialized version is: (int)
		 * number of gadgets (Gadget) gadget 1 (Gadget) gadget 2 ... repeat for
		 * each subsequent gadget see writeGadget for the (Gadget) format.
		 */
		ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
		DataOutputStream out = new DataOutputStream(byteOut);

		byte[] bytes = null;

		try {
			
			for(ICdmBase cdmBaseObject : cdmBaseObjects){
				writeCdmBaseObject(cdmBaseObject, out);
			}
			out.close();
			bytes = byteOut.toByteArray();
		} catch (IOException e) {
			// when in doubt send nothing
		}
		return bytes;
	}
	
	
	private ICdmBase[] fromByteArray(byte[] bytes) {
		DataInputStream in = new DataInputStream(new ByteArrayInputStream(bytes));
		
		try {
			List<ICdmBase> cdmBaseObjects = new ArrayList<ICdmBase>();
                      
			try{
				while(true){
					cdmBaseObjects.add(readCdmBaseObject(in));
				}
			}catch(EOFException e){
				return cdmBaseObjects.toArray(new ICdmBase[]{});
			}
		} catch (IOException e) {
			return null;
		}
	}
	
	private ICdmBase readCdmBaseObject(DataInputStream in) throws IOException{
		String className = in.readUTF();
		in.readInt();
		String idString = in.readUTF();
		in.readInt();
		
		if(className.equals(Synonym.class.getName()) || className.equals(Taxon.class.getName())){
			ICdmBase object = CdmStore.getTaxonService().load(UUID.fromString(idString));
			
			if(object == null){
				return cdmBase;
			}
			
			return object;
		}
		
		return null;
	}
	
	private void writeCdmBaseObject(ICdmBase cdmBase, DataOutputStream out) throws IOException{
		/**
		 * CdmBase object serialization:
		 * 
		 */
		out.writeUTF(cdmBase.getClass().getName());
		out.writeInt(SEPARATOR);
		out.writeUTF(cdmBase.getUuid().toString());
		out.writeInt(EOF);
		
		if(cdmBase.getId() == 0){
			// fallback for unsaved instances
			this.cdmBase = cdmBase;
		}
	}

}
