/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.IPostSelectionProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;

/**
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 16.05.2008
 * @version 1.0
 */
public class SimpleSelectionProvider implements IPostSelectionProvider {

	private ListenerList selectionChangedListeners = new ListenerList();
	private ListenerList postSelectionChangedListeners = new ListenerList();
	private ISelection selection;
	private Job job;
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void addSelectionChangedListener(
			ISelectionChangedListener listener) {
		selectionChangedListeners.add(listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	public ISelection getSelection() {
		if (selection != null)
			return selection;
		// TODO we have to return an empty selection to avoid NPE
		// I don't really understand why this was not the case before
		return new StructuredSelection();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void removeSelectionChangedListener(
			ISelectionChangedListener listener) {
		selectionChangedListeners.remove(listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
	 */
	public void setSelection(ISelection selection) {
		this.selection = selection;
		
		// cancel previous selection setting
		if(job != null){
			job.cancel();
			job = null;
		}
		
		if(job == null){
			final SelectionChangedEvent selectionChangedEvent = new SelectionChangedEvent(this, selection);
			final Display display = Display.getCurrent();
			job = new Job("Setting Selection"){
	
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					monitor.beginTask("Setting Selection", 10);
					
					if(!monitor.isCanceled()){
						display.asyncExec(new Runnable() {
					 
							public void run() {
								firePostSelectionChanged(selectionChangedEvent);
							}
						});
					}
					
					return Status.OK_STATUS;
				}
				
			};
			
			job.setPriority(Job.DECORATE);
			job.schedule();
		}
		
	}

	private void firePostSelectionChanged(final SelectionChangedEvent event) {
		
			
//		for (Object listener : selectionChangedListeners.getListeners()) {
//			 if (listener instanceof ISelectionChangedListener) {
//				 ((ISelectionChangedListener)listener).selectionChanged(event);
//			 }
//		}
//		for (Object listener : postSelectionChangedListeners.getListeners()) {
//			 if (listener instanceof ISelectionChangedListener) {
//				 ((ISelectionChangedListener)listener).selectionChanged(event);
//			 }
//		}
		Object[] listeners = postSelectionChangedListeners.getListeners();
		for (int i = 0; i < listeners.length; ++i) {
			final ISelectionChangedListener l = (ISelectionChangedListener) listeners[i];
			SafeRunnable.run(new SafeRunnable() {
				public void run() {
					l.selectionChanged(event);
				}
			});
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IPostSelectionProvider#addPostSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void addPostSelectionChangedListener(
			ISelectionChangedListener listener) {
		postSelectionChangedListeners.add(listener);		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IPostSelectionProvider#removePostSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	public void removePostSelectionChangedListener(
			ISelectionChangedListener listener) {
		postSelectionChangedListeners.remove(listener);
	}

}
