/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author p.ciardelli
 * @created 26.01.2009
 * @version 1.0
 */
public class ConceptContainer extends AbstractGroupedContainer<Taxon> {
	
	private TaxonRelationshipType relationshipType;
	
		
	private ConceptContainer(TaxonNameEditor editor, AbstractGroup group,
			Taxon relatedTaxon, TaxonRelationshipType relationshipType, 
			boolean acceptedTaxonIsFromTaxon) {
		super(editor, group, relatedTaxon);

		// FIXME the acceptedTaxonIsFromTaxon is never used and 
		// I can't remember what is was used for in the first place
		
		this.relationshipType = relationshipType;
		showRelationshipType();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	@Override
	protected void initializeComposite() {
		
		setIsDraggable(true);
		setIcon(CONCEPT_ICON);
		setFont(getViewerFont());
		setIndent(CONCEPT_INDENT);
		
		showSec();
				
		initTextViewer();
	}
	
	private void showRelationshipType() {
		setNonEditableInfo(relationshipType.getLabel(), true);
	}

	private void showSec() {
		if (getTaxonBase() == null) {
			return;
		}
		
		if (getTaxonBase().getSec() == null) {
			setNonEditableInfo("sec. ???", false);
		} else {
			setNonEditableInfo("sec. " + getTaxonBase().getSec().getTitleCache(), false);
		}
	}

	public static ConceptContainer getNewInstance(TaxonNameEditor editor, AbstractGroup group, TaxonRelationship relationship){

		TaxonRelationshipType relationshipType = relationship.getType();
		
		if (relationship.getToTaxon().equals(editor.getTaxon())) {
			return new ConceptContainer (editor, group, 
					relationship.getFromTaxon(), relationshipType, false);
		} else {
			return new ConceptContainer (editor, group, 
					relationship.getToTaxon(), relationshipType, true);
		} 
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#getViewerFont()
	 */
	@Override
	protected Font getViewerFont() {
		return EditorUtil.getFont(Resources.CONCEPT_FONT);
	}

	public Taxon getRelatedTaxon() {
		return getTaxonBase();
	}
	
	protected void calculateErrors() {
		super.calculateErrors();
		
		textViewer.setShowSecError(getTaxonBase());
	}
	
	@Override
	protected void updateNonEditableInfo() {
		showSec();
		showRelationshipType();
	}
}






