package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.resources.IMarker;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationPresentation;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;

import eu.etaxonomy.taxeditor.model.ImageResources;

public class EditorAnnotation extends Annotation implements IAnnotationPresentation {
	private static final Logger logger = Logger
			.getLogger(EditorAnnotation.class);
    private IMarker marker;
    private String text;
    private int line;
    private Position position;

    // error identifiers, images and colors
    public static String ERROR_TYPE = "editor.error.type";
    public static Image ERROR_IMAGE = ImageResources.getImage(ImageResources.ERROR_ANNOTATION_ICON);
    public static final RGB ERROR_RGB = new RGB(255, 0, 0);    
    
    public static String WARNING_TYPE = "editor.warning.type";
    public static Image WARNING_IMAGE = ImageResources.getImage(ImageResources.WARNING_ANNOTATION_ICON);
    public static final RGB WARNING_RGB = new RGB(244, 200, 45); 
    
    public EditorAnnotation(IMarker marker) {
        this.marker = marker;
    }

    public EditorAnnotation(int line, String text) {
    	this(ERROR_TYPE, line, text);
    }

    public EditorAnnotation(String type, int line, String text) {
    	super(type, false, text);
        this.marker = null;
        this.line = line;
        this.text = text;
    }
    
    public IMarker getMarker() {
        return marker;
    }

    public int getLine() {
        return line;
    }

    public String getText() {
    	if (ERROR_TYPE.equals(getType())) {
    		return "Error: " + text;
    	}
    	if (WARNING_TYPE.equals(getType())) {
    		return "Warning: " + text;
    	}
    	return super.getText();
    }

	/**
	 * @return
	 */
	private Image getImage() {
    	if (ERROR_TYPE.equals(getType())) {
    		return ERROR_IMAGE;
    	}
    	if (WARNING_TYPE.equals(getType())) {
    		return WARNING_IMAGE;
    	}
		logger.warn("No image for type " + getType());
		return null;
	}
    
    public int getLayer() {
        return 3;
    }


    public Position getPosition() {
        return position;
    }

    public void setPosition(Position position) {
        this.position = position;
    }

	
	public void paint(GC gc, Canvas canvas, Rectangle bounds) {
		Point canvasSize= canvas.getSize();

		int x= 0;
		int y= bounds.y;
		int w= canvasSize.x;
		int h= bounds.height;

		if (y + h > canvasSize.y)
			h= canvasSize.y - y;

		if (y < 0) {
			h= h + y;
			y= 0;
		}

		if (h <= 0)
			return;

		Image image = getImage();
		
		if (image == null) {
			return;
		}
		
		Rectangle r = image.getBounds();
		
		int destX = x + w - r.width;
		int destY = y + h - r.height;
		
		gc.drawImage(image, 0, 0, r.width, r.height, destX, destY, r.width, r.height);
	}
}