/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;

/**
 * Checks whether the ScrolledForm a TextViewer is on needs to be
 *  redrawn because of a new line break or the removal of an existing
 *  one ...
 *  <p>
 *  ... in theory, anyway. Until I figure out how to detect a word wrap,
 *   the ScrolledForm is redrawn with every modify event. 
 * 
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
public class LineWrapSupport {
	private static final Logger logger = Logger
			.getLogger(LineWrapSupport.class);
	
	ModifyListener listener = new LineWrapListener();
	int lineCount;

	private StyledText textWidget;
	private ScrolledForm scrolledForm;

//	private TextViewer viewer;
	
	/**
	 * @return the viewer
	 */
//	public TextViewer getViewer() {
//		return viewer;
//	}
	
	public LineWrapSupport(TextViewer viewer, IManagedForm form) {
		
		if (viewer == null) {
			throw new IllegalArgumentException(
					"The provided TextViewer object is null.");
		}
		
//		this.viewer = viewer;
		
		textWidget = viewer.getTextWidget();
		textWidget.addModifyListener(listener);
				
		this.lineCount = textWidget.getLinePixel(textWidget.getLineCount());
		this.scrolledForm = form.getForm();
		
		logger.trace(this.getClass().getSimpleName() + " created");
	}
	
	/**
	 * Redraws the scrolledForm if a line wrap is detected. 
	 */
	private void checkLineCount() {

		boolean lineWrapDetected = true;
		if (lineWrapDetected ) {
			scrolledForm.getBody().layout();
		}
	}
	
	/**
	 * Calls <code>checkLineCount()</code> on a modify event.
	 * 
	 * @author p.ciardelli
	 * @created 21.05.2008
	 * @version 1.0
	 */
	class LineWrapListener implements ModifyListener {

		public void modifyText(ModifyEvent e) {
			checkLineCount();
		}
	}
}