/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class MisapplicationContainer extends AbstractGroupedContainer<Taxon> {
	
	public MisapplicationContainer(TaxonNameEditor editor, AbstractGroup group, Taxon misappliedName) {
		super(editor, group, misappliedName);
	}
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	@Override
	protected void initializeComposite() {
		setIsDraggable(true);
		setIcon(MISAPPLIEDNAME_ICON);
		setFont(getViewerFont());
		setIndent(MISAPPLIEDNAME_INDENT);
		
		showSec();		
		
		initTextViewer();
	}

	private void showSec() {
		if (getMisappliedName() == null) {
			return;
		}
		
		if (getMisappliedName().getSec() == null) {
			setNonEditableInfo("sec. ???", false);
		} else {
			String title = getMisappliedName().getSec().getTitleCache();
			setNonEditableInfo("sec. " + title, false);
		}
	}
	
	public Taxon getMisappliedName() {
		return getTaxonBase();
	}

	@Override
	protected Font getViewerFont() {
		return EditorUtil.getFont(Resources.MISAPPLIEDNAME_FONT);
	}
	
	protected void calculateErrors() {
		super.calculateErrors();
		
		textViewer.setShowSecError(getMisappliedName());
	}
	

	@Override
	protected void updateNonEditableInfo() {
		showSec();
	}

}