/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.IDropTargetable;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.ChangeConceptRelationshipTypeOperation;
import eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation;
import eu.etaxonomy.taxeditor.operations.ChangeHomotypicGroupOperation;
import eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation;

/**
 * @author n.hoffmann
 *
 */
public class NameEditorDropTargetListener extends DropTargetAdapter {
		
	private static final Logger logger = Logger.getLogger(NameEditorDropTargetListener.class);
	
	private TaxonNameEditor editor;

	private IDropTargetable target;

	public NameEditorDropTargetListener(IDropTargetable target){
		this.target = target;
		editor = target.getNameEditor();
	}
	
	public void dragEnter(DropTargetEvent dropTargetEvent) {
		target.dragEntered();
		
//		if(! EditorUtil.getActiveMultiPageTaxonEditor().equals(editor.getMultiPageTaxonEditor())){
//			editor.getMultiPageTaxonEditor().setFocus();
//		}
	}
	
	public void dragLeave(DropTargetEvent dropTargetEvent) {
		target.dragLeft();
	}
	
	public void drop(DropTargetEvent event) {
		
		TaxonBase taxonBase = getDroppedTaxonBase(event);
		
		if (taxonBase != null) {
			// Execute operations at end of drag event
			AbstractPostOperation operation = createOperation(taxonBase);
			
			
			// Execute operation if it has been initiated
			if (operation == null) {
				logger.warn("User unsuccessfully tried to drop: " + taxonBase.getClass());
			} else {
				EditorUtil.executeOperation(operation);
			}
		}
	}
	
	private AbstractPostOperation createOperation(TaxonBase taxonBase) {
		
		
		if(taxonBase instanceof Synonym){
			Synonym synonym = (Synonym) taxonBase;
			
			if(target instanceof HomotypicalSynonymGroup){
				HomotypicalGroup homotypicalGroup = ((HomotypicalSynonymGroup) target).getGroup();
				
				return new ChangeHomotypicGroupOperation("Change Homotypical Group", editor.getUndoContext(), 
						editor.getTaxon(), synonym, homotypicalGroup, target);
			}
			else if(target instanceof MisappliedGroup){
				return new ChangeSynonymToMisapplicationOperation("Change Synonym to Misapplication", editor.getUndoContext(), 
						editor.getTaxon(), synonym, target);
			}
			else if(target instanceof TaxonNameEditor){
				return new ChangeHomotypicGroupOperation("Change Homotypical Group", editor.getUndoContext(), 
						editor.getTaxon(), synonym, null, target);
			}
			
		}else if(taxonBase instanceof Taxon){
			Taxon concept = (Taxon) taxonBase;
			
			if(target instanceof HomotypicalSynonymGroup){
				HomotypicalGroup homotypicalGroup = ((HomotypicalSynonymGroup) target).getGroup();
				
				return new ChangeConceptToSynonymOperation("Change to Synonym", editor.getUndoContext(), 
						editor.getTaxon(), concept, homotypicalGroup, target);
			}
			else if(target instanceof MisappliedGroup ){
				return new ChangeConceptRelationshipTypeOperation("Change", editor.getUndoContext(), 
						editor.getTaxon(), concept, TaxonRelationshipType.MISAPPLIED_NAME_FOR(), target);
			}
		}

		
		return null;
	}

	@Override
	public void dropAccept(DropTargetEvent event) {
		// pass
	}

	private HomotypicalGroup getHomotypicalGroup() {
		if(target instanceof HomotypicalSynonymGroup){
			return ((HomotypicalSynonymGroup) target).getGroup();
		}
		return null;
	}	
	
	private TaxonBase getDroppedTaxonBase(DropTargetEvent dropTargetEvent){
		if(dropTargetEvent.data instanceof ICdmBase[]){
		
			ICdmBase[] cdmBaseObjects = (ICdmBase[]) dropTargetEvent.data;
			
			if(cdmBaseObjects[0] instanceof TaxonBase){
				return (TaxonBase) cdmBaseObjects[0];
			}
		}
		return null;
	}
}
