/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import org.apache.log4j.Logger;
import org.eclipse.swt.graphics.Font;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author p.ciardelli
 * @created 13.01.2009
 * @version 1.0
 */
public class SynonymContainer extends AbstractGroupedContainer<Synonym> {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger
			.getLogger(SynonymContainer.class);
	
	public SynonymContainer(TaxonNameEditor editor, AbstractGroup group, Synonym synonym) {
		super(editor, group, synonym);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.name.NameComposite#initializeComposite()
	 */
	@Override
	protected void initializeComposite() {
		setIsDraggable(true);
		setFont(getViewerFont());
		setIndent(SYNONYM_INDENT);
		
		TaxonNameBase name = getSynonym().getName();
		
		if(name == null){
			logger.error("Synonym with a null name detected. This should never happen.");
		}
		
		if (isHomotypic()) {
			if (name.isGroupsBasionym()) {
				setIcon(HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HOMOTYPIC_SYNONYM_ICON);
			}	
		} else {
			if (name.isGroupsBasionym()) {
				setIcon(HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON);
			} else {
				setIcon(HETEROTYPIC_SYNONYM_ICON);
			}
		}
		
		initTextViewer();
	}
	
	private boolean isHomotypic() {
		HomotypicalGroup group = getSynonym().getHomotypicGroup();
		return group.equals(editor.getTaxon().getHomotypicGroup()); 
	}
	
	public Synonym getSynonym() {
		return getTaxonBase();
	}
		

	protected Font getViewerFont() {
		return EditorUtil.getFont(Resources.SYNONYM_FONT);
	}

	@Override
	protected void updateNonEditableInfo() {
		// not needed
	}
}