/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.name.handler;

import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * 
 * @author n.hoffmann
 * @created 17.04.2009
 * @version 1.0
 */
public abstract class AbstractDynamicConceptRelationMenu extends ContributionItem {
	public static Logger logger = Logger.getLogger(AbstractDynamicConceptRelationMenu.class);

	private static List<TaxonRelationshipType> excludeRelationshipTypes = Arrays.asList(new TaxonRelationshipType[]{
			TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN(),
			TaxonRelationshipType.MISAPPLIED_NAME_FOR(),
			TaxonRelationshipType.ALL_RELATIONSHIPS()
	});
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.action.ContributionItem#fill(org.eclipse.swt.widgets.Menu, int)
	 */
	@Override
	public void fill(Menu menu, int index){
		final IHandlerService handlerService = (IHandlerService) TaxeditorEditorPlugin.getDefault().getWorkbench().getService(IHandlerService.class);
	
		List<TaxonRelationshipType> relationshipTypes = PreferencesUtil.getPreferredTaxonRelationshipTypes();
		
		relationshipTypes.removeAll(excludeRelationshipTypes);
		
		for(final TaxonRelationshipType type : relationshipTypes){
			MenuItem menuItem = new MenuItem(menu, -1);
			menuItem.setText(type.getLabel());
			menuItem.setData(type);
			menuItem.addSelectionListener(new SelectionListener(){

				public void widgetDefaultSelected(SelectionEvent e) {}

				public void widgetSelected(SelectionEvent ev) {
					Event event = new Event();
					event.data = type;
					try {
						handlerService.executeCommand(getCommandName(), event);
					} catch (Exception e) {
						logger.error("Error executing command", e);
					}
				}				
			});
		}
	}
	
	public abstract String getCommandName();
}
