/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.name.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredTaxonNodeSelectionDialog;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.operations.ChangeAcceptedTaxonToSynonymOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * 
 * @author n.hoffmann
 * @created Jan 4, 2010
 * @version 1.0
 */
public class ChangeAcceptedTaxonToSynonymHandler extends AbstractHandler
		implements IHandler, IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(ChangeAcceptedTaxonToSynonymHandler.class);
	private ChangeAcceptedTaxonToSynonymOperation operation;

	private UUID newAcceptedTaxonNodeUuid;
	private TaxonNameEditor editor;
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		editor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(
				Page.NAME);
		TaxonNode oldAcceptedTaxonNode = ((TaxonEditorInput) editor.getEditorInput()).getTaxonNode();
		
		// Choose new accepted taxon	
		List<UUID> excludeTaxa = new ArrayList<UUID>();
		excludeTaxa.add(oldAcceptedTaxonNode.getUuid());
		TaxonNode newAcceptedTaxonNode = FilteredTaxonNodeSelectionDialog.selectTaxonNode(HandlerUtil.getActiveShell(event), "Choose the accepted taxon", excludeTaxa, null);

		if (newAcceptedTaxonNode == null) {
			return null;
		}
		
		newAcceptedTaxonNodeUuid = newAcceptedTaxonNode.getUuid();
				
		try {
			operation = new ChangeAcceptedTaxonToSynonymOperation(event.getCommand().getName(), 
									editor.getUndoContext(), oldAcceptedTaxonNode, newAcceptedTaxonNode, this, editor);
			EditorUtil.executeOperation(operation);
			
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		
		return null;
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		Display.getDefault().asyncExec(new Runnable(){

			public void run() {
				EditorUtil.close(editor.getMultiPageTaxonEditor());
				
				try {
					MultiPageTaxonEditor possibleOpenEditor = (MultiPageTaxonEditor) EditorUtil.findEditorByTaxonNodeUuid(newAcceptedTaxonNodeUuid);
					if(possibleOpenEditor != null){
						EditorUtil.close(possibleOpenEditor);
					}
					EditorUtil.openTaxonNode(newAcceptedTaxonNodeUuid);
				} catch (PartInitException e) {
					logger.error(e);
					throw new RuntimeException(e);
				}
			}
			
		});
		
		
		return true;
	}

	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}	
}