// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.swt.widgets.Event;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.ChangeConceptRelationshipTypeOperation;
import eu.etaxonomy.taxeditor.operations.ChangeSynonymToConceptOperation;

/**
 * @author n.hoffmann
 * @created 22.04.2009
 * @version 1.0
 */
public class ChangeToConceptRelationHandler extends AbstractHandler implements
		IHandler {
	private static final Logger logger = Logger
			.getLogger(ChangeToConceptRelationHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TaxonNameEditor editor = (TaxonNameEditor) EditorUtil.getActiveEditorPage(
				Page.NAME);

		TaxonBase taxonBase = (TaxonBase) EditorUtil.getSelection(event).getFirstElement();
		
		TaxonRelationshipType type = (TaxonRelationshipType) ((Event)event.getTrigger()).data;
		
		AbstractPostOperation operation;
		try {
			if(taxonBase instanceof Synonym){
				Synonym synonym = (Synonym) taxonBase;
				operation = new ChangeSynonymToConceptOperation(event.getCommand().getName(), 
						editor.getUndoContext(), editor.getTaxon(), synonym, type, editor);
				EditorUtil.executeOperation(operation);
			}
			if(taxonBase instanceof Taxon){
				Taxon misapplication = (Taxon) taxonBase;
				operation = new ChangeConceptRelationshipTypeOperation(event.getCommand().getName(), 
						editor.getUndoContext(), editor.getTaxon(), misapplication, type, editor);
				EditorUtil.executeOperation(operation);
			}
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		
		return null;
	}
}
