// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.newWizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public abstract class AbstractNewEntityWizard<T extends IdentifiableEntity> extends Wizard implements
		INewWizard, IConversationEnabled {
	private static final Logger logger = Logger
			.getLogger(AbstractNewEntityWizard.class);

	private ConversationHolder conversation;
	
	protected CdmFormFactory formFactory;

	protected T entity;
	
	public AbstractNewEntityWizard(){
		setWindowTitle("New Entity");
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		saveEntity();
		
		conversation.commit();
		conversation.close();
		return true;
	}
	
	public T getEntity() {
		return entity;
	}
	
	/**
	 * Adds the entity to the current persistence context
	 */
	protected abstract void saveEntity();

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		formFactory = new CdmFormFactory(Display.getCurrent(), null);
	 	entity = createNewEntity();
	 	conversation = CdmStore.createConversation();
	}

	protected abstract T createNewEntity();
	
	public ConversationHolder getConversationHolder() {
		return conversation;
	}
	
	public void update(CdmDataChangeMap changeEvents) {}
}
