// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.RootElement;

/**
 * @author n.hoffmann
 * @created Feb 9, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDataViewer extends Viewer implements IPropertyChangeListener, ISelectionProvider, IConversationEnabled{
	private static final Logger logger = Logger
			.getLogger(AbstractCdmDataViewer.class);
	
	protected ManagedForm managedForm;

	protected CdmFormFactory formFactory;

	protected ScrolledForm scrolledForm;
	
	private Composite body;
	
	protected RootElement rootElement;

	private Object input;

	private AbstractCdmViewPart viewPart;

	
	/**
	 * 
	 */
	public AbstractCdmDataViewer(Composite parent, AbstractCdmViewPart viewPart) {
		this.viewPart = viewPart;
		
		managedForm = new ManagedForm(parent){
			
			/* (non-Javadoc)
			 * @see org.eclipse.ui.forms.ManagedForm#dirtyStateChanged()
			 */
			@Override
			public void dirtyStateChanged() {
				markViewPartDirty();
			}
		};
		
		createFormFactory();
		
		formFactory.addPropertyChangeListener(this);
		
		scrolledForm = managedForm.getForm();
		
		body = scrolledForm.getBody();
		
		body.setLayout(CdmFormFactory.LAYOUT());
		
		rootElement = new RootElement(formFactory, body);
	}
	
	protected void createFormFactory() {
		if(formFactory != null){
			formFactory.dispose();
			formFactory = null;
		}
		formFactory = new CdmFormFactory(Display.getCurrent(), this);
	}

	private void markViewPartDirty(){
		viewPart.dirtyStateChanged();
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getControl()
	 */
	@Override
	public Control getControl() {
		for(Control child : body.getChildren()){
			return child;
		}
		
		return body;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setInput(java.lang.Object)
	 */
	@Override
	public void setInput(Object input) {
		this.input = input;	
		// reset selection
		setSelection(new StructuredSelection(input));
		refresh();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getInput()
	 */
	@Override
	public Object getInput() {
		return input;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	@Override
	public void refresh() {
		showParts();
		
		managedForm.setInput(input);
		
		managedForm.refresh();
		
		managedForm.reflow(true);
	}
	
	public void layout(){
		body.layout();
	}
	
	protected abstract void showParts();
	
	
	protected void destroySections() {	
		for (IFormPart formPart : managedForm.getParts()){
			removePart((CdmSectionPart) formPart);
		}	
		
		managedForm.setInput(null);
				
		formFactory.destroyElement(rootElement);
		
		createFormFactory();
		
		rootElement = new RootElement(formFactory, body);
		
		for(Control control : body.getChildren()){
			control.dispose();
			control = null;
		}
	}
	
	
	public void setFocus(){
		// we have to set focus to a control of this viewer
		// otherwise, after opening a dialog from the details, the focus will not be
		// given back to the details view but to the editor
		for(Control child : body.getChildren()){
			child.setFocus();
			break;
		}
	}
	
	public void reflow(){
		managedForm.reflow(true);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		logger.warn("Property changed: " + event);
		
	}
	
	protected void removePart(CdmSectionPart sectionPart){
		managedForm.removePart(sectionPart);
		formFactory.removePropertyChangeListener(sectionPart);
	}
	
	protected void addPart(AbstractFormSection section){
		CdmSectionPart sectionPart = new CdmSectionPart(section);
		managedForm.addPart(sectionPart);
		formFactory.addPropertyChangeListener(sectionPart);
	}
	
	public ConversationHolder getConversationHolder() {
		return viewPart.getConversationHolder();
	}
	
	public void update(CdmDataChangeMap changeEvents) {}
}
