// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * @author n.hoffmann
 * @created Jun 15, 2010
 * @version 1.0
 */
public abstract class AbstractCdmViewPart extends ViewPart implements ISelectionListener, IPostOperationEnabled, IConversationEnabled{
	private static final Logger logger = Logger
			.getLogger(AbstractCdmViewPart.class);
	
	protected ISelectionService selectionService;

	protected IWorkbenchPart part;
	
    private PageBook pageBook;

	private Label emptySelectionLabel;

	private Composite viewerComposite;
	
	/**
	 * 
	 */
	@Override
	public void createPartControl(Composite parent) {
		selectionService = getSite().getWorkbenchWindow().getSelectionService();
		selectionService.addSelectionListener(this);
		
		pageBook = new PageBook(parent, SWT.NULL);
		createViewerComposite(pageBook);
		createViewer(viewerComposite);
		
//		viewer.getControl().addFocusListener(new FocusAdapter() {
//			@Override
//			public void focusGained(FocusEvent e) {
//				setFocus();
//			}
//		});
		
		// Page 2: Nothing selected
        emptySelectionLabel = new Label(pageBook, SWT.TOP + SWT.LEFT + SWT.WRAP);
        
        showEmptyPage();
        		
		// show viewer if there is an open editor
		if(getEditor() != null){
			showViewer(getEditor(), new StructuredSelection(getInitialInput()));
		}
	}
	
	protected abstract Object getInitialInput();

	private void createViewerComposite(Composite parent) {
		
		viewerComposite = new SashForm(parent, SWT.HORIZONTAL);
		
		
	}
    
	/**
	 * 
	 */
	public void showEmptyPage(){
		pageBook.showPage(emptySelectionLabel);
	}
	
    public void showViewer(IWorkbenchPart part, ISelection selection){

    	
    	this.part = part;		
				
		IStructuredSelection structuredSelection = (IStructuredSelection) selection;
		
		getViewer().setInput(structuredSelection.getFirstElement());
		
		showViewer();
    }
    
    public abstract Viewer getViewer();

	public void showViewer(){
		pageBook.showPage(viewerComposite);
    }
  
	
	@Override
	public void setFocus() {
		if(getConversationHolder() != null){
			getConversationHolder().bind();
		}
		getViewer().getControl().setFocus();	
	}
	
	public abstract void createViewer(Composite parent);
	
	/**
	 * @return the currently active editor
	 */
	public IEditorPart getEditor(){
		return EditorUtil.getActiveEditor();
	}
	
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		dirtyStateChanged();
		return true;
	}
	
	public void dirtyStateChanged(){
		if(part instanceof AbstractCdmViewPart){
			((AbstractCdmViewPart) part).dirtyStateChanged();
		}else{
			if(getEditor() != null && getEditor() instanceof MultiPageTaxonEditor){
				((MultiPageTaxonEditor)getEditor()).editorDirtyStateChanged();
			}
		}
	}
	
	public ConversationHolder getConversationHolder() {
		if(part != null)
			return ((IConversationEnabled) part).getConversationHolder();
		
		return null;
	}
	
	public void update(CdmDataChangeMap changeEvents) {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void dispose() {
		selectionService.removeSelectionListener(this);
		super.dispose();
	}
}
