// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.forms.SectionPart;

import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created Feb 8, 2010
 * @version 1.0
 */
public class CdmSectionPart<T extends VersionableEntity> extends SectionPart implements IPropertyChangeListener{
	
	private static final Logger logger = Logger
				.getLogger(CdmSectionPart.class);

	private AbstractFormSection<T> formSection;
	
	public CdmSectionPart(AbstractFormSection<T> section) {
		super(section);
		formSection = section;
	}
	
	@Override
	public boolean setFormInput(Object input) {
		if(input instanceof AbstractGroupedContainer){
			input = ((AbstractGroupedContainer) input).getData();
		}
		if(input instanceof VersionableEntity){
			logger.trace("Setting input for section: " + formSection.getClass().getSimpleName());
			if((input instanceof TaxonBase) && (formSection instanceof ITaxonDetailSection)){
				((ITaxonDetailSection) formSection).setTaxon((TaxonBase) input);
			}else{
				formSection.setEntity((T) input);
			}
		}
		return super.setFormInput(input);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		if (event != null){
			
			Object eventSource = event.getSource();
			
			if(eventSource instanceof ICdmFormElement){
				if(formSection.containsFormElement((ICdmFormElement)eventSource)){
					markDirty();
				}
			}	
		}
	}
	
	
	
}
