/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 *
 */
public class DescriptiveLabelProvider extends ColumnLabelProvider implements IStyledLabelProvider  {
	private static final Logger logger = Logger
			.getLogger(DescriptiveLabelProvider.class);

	public String getText(Object element) {
		return getLabel(element);
	}	

	public static String getLabel(Object element){
		if (element instanceof DescriptionBase) {
			return getDescriptionText((DescriptionBase) element);
		}
		else if (element instanceof CommonTaxonName) {
			return getCommonNameText((CommonTaxonName) element);			
		}
		else if (element instanceof Distribution) {
			return getDistributionText((Distribution) element);			
		}		
		else if (element instanceof TaxonInteraction) {
			return getTaxonInteractionText((TaxonInteraction) element);
		}
		else if (element instanceof IndividualsAssociation) {
			return getIndividualsAssociationText((IndividualsAssociation) element);
		}
		else if (element instanceof TextData) {
			return getElementText((TextData) element);
		}
		else{
			return element.toString();
		}
	}
	
	/**
	 * @param element
	 * @return
	 */
	public static String getDistributionText(Distribution element) {
		
		String text = "EMPTY";
		
		Distribution distribution = (Distribution) element;
		
		NamedArea area = distribution.getArea();
		if(area != null){
			PresenceAbsenceTermBase<?> status = distribution.getStatus();
			
			text = area.getLabel();
			if (text == null || "".equals(text.trim())) {
				text = area.getRepresentation(Language.DEFAULT()).getAbbreviatedLabel();
				if (text == null || "".equals(text.trim())) {
					text = area.getRepresentation(Language.DEFAULT()).getText();
					if (text == null || "".equals(text.trim())) {
						text = "EMPTY";
					}
				}
			}
			
			if (status != null) {
				text += ", " + status.getLabel();
			}else{
				text += ", NO STATUS";
			}
		}
		return text;
	}

	/**
	 * @param entity
	 * @return
	 */
	public static  String getImageText(Media media) {
		 LanguageString title = media.getTitle(CdmStore.getDefaultLanguage());
		if (title != null) {
			return title.getText();
		}
		return "No title.";
	}

	
	
	/**
	 * @param element
	 * @return
	 */
	public static  String getElementText(TextData element) {	
		String text = element.getText(CdmStore.getDefaultLanguage());		
		return text == null ? "No text provided" : text;
	}
	
	/**
	 * @param element
	 * @return
	 */
	public static  String getTaxonInteractionText(TaxonInteraction element) {
		String text = "";
		Taxon taxon2 = ((TaxonInteraction) element).getTaxon2();
		if(taxon2 != null && taxon2.getName() != null){
			text = taxon2.getName().getTitleCache();
		}else{
			text = "No taxon chosen";
		}
		
		return text;
	}

	/**
	 * @param entity
	 * @return
	 */
	public static  String getCommonNameText(CommonTaxonName commonName) {
		String name = commonName.getName();
		if (name == null || name.length() == 0) {
			name = "No name provided";
		}
		Language language = commonName.getLanguage();
		if (language == null) {
			name += " (No language provided)";
		} else {
			name += " (" + language.getDescription() + ")";
		}
		return name;
	}

	/**
	 * @param entity
	 * @return
	 */
	public static  String getDescriptionText(DescriptionBase element) {
		String text = element.getTitleCache();
		if (text == null || text.length() == 0) {
			text = "No label provided";
		}
		return "Description: " + text;
	}
	
	/**
	 * 
	 * @param element
	 * @return
	 */
	public static  String getIndividualsAssociationText(IndividualsAssociation element) {
		SpecimenOrObservationBase derivedUnit = element.getAssociatedSpecimenOrObservation();
		if(derivedUnit != null){
			return derivedUnit.getTitleCache();
		}
		return "No text provided";
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider#getStyledText(java.lang.Object)
	 */
	public StyledString getStyledText(Object element) {
		return new StyledString(getText(element), StyledString.QUALIFIER_STYLER);
	}
}
