// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.DeleteDescriptionElementOperation;
import eu.etaxonomy.taxeditor.operations.DeleteMediaOperation;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * @author n.hoffmann
 * @created Jun 22, 2010
 * @version 1.0
 */
public class DeleteDescriptionHandler extends AbstractHandler {
	private static final Logger logger = Logger
			.getLogger(DeleteDescriptionHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getActiveMenuSelection(event);

		IWorkbenchPart part = HandlerUtil.getActivePart(event);
		IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part : null;
		
		try {
			String label = event.getCommand().getName();
		
			IUndoContext undoContext = EditorUtil.getUndoContext();
			
			for(Object object : selection.toArray()){
				
				AbstractPostOperation operation = null;
				
				if(object instanceof TaxonDescription){
					operation = new DeleteTaxonDescriptionOperation(label, undoContext, (TaxonDescription) object, postOperationEnabled);
				}
				else if(object instanceof DescriptionElementBase){
					operation = new DeleteDescriptionElementOperation(label, undoContext, (DescriptionElementBase) object, postOperationEnabled);
				}
				else if(object instanceof Media){
					TreeSelection treeSelection = (TreeSelection) HandlerUtil.getActiveMenuSelection(event);
					
					TreePath[] path = treeSelection.getPathsFor(object);
					
					TaxonDescription imageGallery = (TaxonDescription) path[0].getFirstSegment();
										
					operation = new DeleteMediaOperation(label, undoContext, imageGallery, (Media) object, postOperationEnabled);
				}
				else{
					throw new IllegalArgumentException("Selection os not valid for this delete handler");
				}
				
				EditorUtil.executeOperation(operation);
			}
		} catch (NotDefinedException e) {
			logger.warn("Command name not set");
		}
		
		
		return null;
	}
}
