/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.IStructuredSelection;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.editor.view.descriptive.TaxonDescriptionFeature;

/**
 * Property tester used by the description tree menu. 
 * 
 * @author p.ciardelli
 * @author n.hoffmann
 */
public class DescriptionsMenuPropertyTester extends PropertyTester {

	private static final Logger logger = Logger
			.getLogger(DescriptionsMenuPropertyTester.class);
	
	private static final String MEDIA = "isMedia";
	private static final String FEATURE = "isFeature";
	private static final String DESCRIPTION = "isDescription";
	private static final String DESCRIPTION_ELEMENT = "isDescriptionElement";
	private static final String DELETABLE = "isDeletable";
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.expressions.IPropertyTester#test(java.lang.Object, java.lang.String, java.lang.Object[], java.lang.Object)
	 */
	public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {

		Object[] selectedElements = ((IStructuredSelection) receiver).toArray();
		
		if(selectedElements.length == 0){
			// nothing selected so all tests should fail
			return false;
		}
		
		if(MEDIA.equals(property)){
			return isMedia(selectedElements);
		}
		else if(FEATURE.equals(property)){
			return isFeature(selectedElements);
		}
		else if(DESCRIPTION.equals(property)){
			return isDescription(selectedElements);
		}
		else if(DESCRIPTION_ELEMENT.equals(property)){
			return isDescriptionElement(selectedElements);
		}
		else if(DELETABLE.equals(property)){
			return isDeletable(selectedElements);
		}
		else{
			return false;
		}
	}
	
	private boolean isFeature(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof TaxonDescriptionFeature)){
				logger.trace("Is not a feature");
				return false;
			}
		}
		return true;
	}

	private boolean isDeletable(Object[] selectedElements) {
		boolean result = false;
		
		for (Object object : selectedElements) {
			
			if((object instanceof DescriptionBase)
				|| (object instanceof DescriptionElementBase)
				|| (object instanceof Media)){
				result = true;
			}else{
				logger.trace("Is not deletable");
				return false;
			}
			
		}
		return result;
	}

	private boolean isDescriptionElement(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionElementBase)){
				logger.trace("Is not a description element");
				return false;
			}
		}
		return true;
	}

	private boolean isDescription(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof DescriptionBase)){
				logger.trace("Is not a description");
				return false;
			}
		}
		return true;
	}

	private boolean isMedia(Object[] selectedElements) {
		for (Object object : selectedElements){
			if(!(object instanceof Media)){
				logger.trace("Is not media");
				return false;
			}
		}
		return true;
	}
}
