// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.detail;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.editor.view.descriptive.TaxonDescriptionFeature;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;

/**
 * 
 * @author n.hoffmann
 * @created Jun 10, 2010
 * @version 1.0
 */
public class DetailsViewPart extends AbstractCdmViewPart implements IPartContentHasSupplementalData{
	private static final Logger logger = Logger
			.getLogger(DetailsViewPart.class);
	
	public static String ID = "eu.etaxonomy.taxeditor.editor.forms.detailsView";

	private DetailsViewer detailsViewer;

	@Override
	public void createViewer(Composite parent) {
		selectionService.addPostSelectionListener(this);
		
		detailsViewer = new DetailsViewer(parent, this);
		getSite().setSelectionProvider(detailsViewer);
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(EditorUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}
		
		if(part == this){
			return;
		}
		
		if(!(selection instanceof IStructuredSelection)){
			return;
		}
		
		if((part instanceof EditorPart) || (part instanceof AbstractCdmViewPart)) {
			if(((IStructuredSelection) selection).size() != 1){
				showEmptyPage();
				return;
			}
			
			// do not show details for features
			if(((IStructuredSelection) selection).getFirstElement() instanceof TaxonDescriptionFeature){
				showEmptyPage();
				return;
			}
			
			showViewer(part, selection);
		}else{
			showEmptyPage();		
		}	
	}

	public DetailsViewer getDetailsViewer() {
		return (DetailsViewer) getViewer();
	}
	
	@Override
	public void dirtyStateChanged() {
		super.dirtyStateChanged();
		if(part instanceof MultiPageTaxonEditor){
			MultiPageTaxonEditor editor = (MultiPageTaxonEditor) part;
			if(editor != null){
				TaxonNameEditor page = (TaxonNameEditor) editor.getPage(Page.NAME);
				page.getSelectedObject().refresh();
			}	
		}
	}
	
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		
		
		detailsViewer.setInput(objectAffectedByOperation);
		
		return super.postOperation(objectAffectedByOperation);
	}
	
	@Override
	public Viewer getViewer() {
		return detailsViewer;
	}

	@Override
	protected Object getInitialInput() {
		return ((IStructuredSelection) selectionService.getSelection()).getFirstElement();
	}
	
	@Override
	public void dispose() {
		super.dispose();
		selectionService.removePostSelectionListener(this);
	}

	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}
