// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;

/**
 * @author n.hoffmann
 * @created Jun 15, 2010
 * @version 1.0
 */
public class MediaContentProvider implements ITreeContentProvider{
	private static final Logger logger = Logger
			.getLogger(MediaContentProvider.class);

	private static final Object[] NO_CHILDREN = new Object[0];
	
	public Object[] getChildren(Object parentElement) {
				
		if (parentElement instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) parentElement).getTaxon();
			HashSet<DescriptionBase> imageGalleries = new HashSet<DescriptionBase>();
			for(DescriptionBase description : taxon.getDescriptions()){
				if(description.isImageGallery()){
					imageGalleries.add(description);
				}
			}			
			return imageGalleries.toArray(); 
		}
		else if (parentElement instanceof DescriptionBase) {
			if (((DescriptionBase) parentElement).isImageGallery()) {
				return getImages((DescriptionBase) parentElement).toArray();
			} 
		}
		else if (parentElement instanceof DerivedUnitBase){
			try {
				DerivedUnitFacade facade = DerivedUnitFacade.NewInstance((DerivedUnitBase) parentElement);
				
//				return new DescriptionBase[]{facade.getgetDerivedUnitMediaGallery(), facade.getFieldObjectMediaGallery()};
				
			} catch (DerivedUnitFacadeNotSupportedException e) {
				logger.error("DerivedUnitFacadeNotSupportedException when trying to instantiate DerivedUnitFacade", e);
			}
		}
		
		return NO_CHILDREN;
	}

	public Object getParent(Object element) {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean hasChildren(Object element) {
		return (getChildren(element).length > 0);
	}

	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	public void dispose() {}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
	
	private List<Media> getImages(DescriptionBase description){
		Assert.isTrue(description.isImageGallery(), "Description should have the imageGallery flag set.");
		
		Set<DescriptionElementBase> elements = description.getElements();
		if (elements != null) {
			// by definition, image galleries have only one description element
			if(elements.size() > 1){
				logger.error("There should be one and only one description element to hold the images. Found: " + elements.size() + " InDescription ID: " + description.getUuid());
			}
						
			DescriptionElementBase element = elements.iterator().next();
			
			return element.getMedia();
		}
		return null;
	}
}
