// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.media;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;

/**
 * @author n.hoffmann
 * @created Jun 15, 2010
 * @version 1.0
 */
public class MediaViewPart extends AbstractCdmViewPart implements IPartContentHasDetails, IPartContentHasSupplementalData {
	private static final Logger logger = Logger.getLogger(MediaViewPart.class);

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.media";

	private TreeViewer mediaViewer;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createViewer(Composite parent) {
		
		mediaViewer = new TreeViewer(new Tree(parent, SWT.NULL));
		
		mediaViewer.setContentProvider(new MediaContentProvider());
		mediaViewer.setLabelProvider(new MediaLabelProvider());
		mediaViewer.setAutoExpandLevel(2);
		
		// Propagate selection from viewer
		getSite().setSelectionProvider(mediaViewer);
		
		// Add context menu to tree
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, mediaViewer);
		
		Control control = mediaViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);	
		
		// set initial input
		if(getEditor() != null){
			mediaViewer.setInput(getEditor().getEditorInput());
		}
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(EditorUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}
		
		if(part instanceof MultiPageTaxonEditor){
			IEditorInput input = ((IEditorPart) part).getEditorInput();
			showViewer(part, new StructuredSelection(input));
		}

		if(part instanceof BulkEditor){
			showViewer(part, selectionService.getSelection());
		}
	}

	@Override
	public void dirtyStateChanged() {
		mediaViewer.refresh();
		super.dirtyStateChanged();
	}

	@Override
	public Viewer getViewer() {
		return mediaViewer;
	}

	@Override
	protected Object getInitialInput() {
		return getEditor().getEditorInput();
	}

	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}
