/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.view.media.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.CreateTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * @author p.ciardelli
 *
 */
public class AddImageGalleryHandler extends AbstractHandler {
	private static final Logger logger = Logger.getLogger(AddImageGalleryHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		IWorkbenchPart part = HandlerUtil.getActivePart(event);
		IPostOperationEnabled postOperationEnabled = (part instanceof IPostOperationEnabled) ? (IPostOperationEnabled) part : null;
		
		
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		if (editor instanceof FormEditor) {
			editor = ((FormEditor) editor).getActiveEditor();
		}
		IEditorInput input = editor.getEditorInput();
		if (input instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) input).getTaxon();	
			AbstractPostOperation operation;
			try {
				// TODO use undo context specific to editor
				operation = new CreateTaxonDescriptionOperation(event.getCommand().getName(), 
						EditorUtil.getUndoContext(), taxon, postOperationEnabled, true);
				EditorUtil.executeOperation(operation);
			} catch (NotDefinedException e) {
				logger.warn("Command name not set");
			}
		} 
		return null;
	}

}
