// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.supplementaldata;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.editor.view.descriptive.TaxonDescriptionFeature;


/**
 * @author n.hoffmann
 * @created Jun 10, 2010
 * @version 1.0
 */
public class SupplementalDataViewPart extends AbstractCdmViewPart {
	private static final Logger logger = Logger
			.getLogger(SupplementalDataViewPart.class);

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.supplementalData";

	private SupplementalDataViewer supplementalDataViewer;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createViewer(Composite parent) {
		selectionService.addPostSelectionListener(this);
		supplementalDataViewer = new SupplementalDataViewer(parent, this);
	}


	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(EditorUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}
		
		if((part instanceof EditorPart) || (part instanceof AbstractCdmViewPart)) {
			if(((IStructuredSelection) selection).size() != 1){
				showEmptyPage();
				return;
			}
			
			
			if(((IStructuredSelection) selection).getFirstElement() instanceof TaxonDescriptionFeature){
				showEmptyPage();
				return;
			}
			
			showViewer(part, selection);
		}else{
			showEmptyPage();		
		}
	}
	
	@Override
	public void dispose() {
		super.dispose();
		selectionService.removePostSelectionListener(this);
	}


	@Override
	public Viewer getViewer() {
		return supplementalDataViewer;
	}


	@Override
	protected Object getInitialInput() {
		return ((IStructuredSelection) selectionService.getSelection()).getFirstElement();
	}


	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
	
	
}
