// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.supplementaldata;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.model.media.IdentifiableMediaEntity;
import eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmDataViewer;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.editor.view.detail.DetailsViewer;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.RootElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.EntityDetailType;
import eu.etaxonomy.taxeditor.section.supplemental.HeadlineSection;
import eu.etaxonomy.taxeditor.section.supplemental.VersionSection;

/**
 * @author n.hoffmann
 * @created Feb 12, 2010
 * @version 1.0
 */
public class SupplementalDataViewer extends AbstractCdmDataViewer implements ISelectionChangedListener {
	private static final Logger logger = Logger
			.getLogger(SupplementalDataViewer.class);

	private enum VIEW_PART{
		VersionableEntity, 
		AnnotatableEntity, 
		IdentifiableEntity, 
		IdentifiableMediaEntity,
		EMPTY;
	}
	
	private VIEW_PART currentViewPart;
	
	private DetailsViewer detailsViewer;

	
	/**
	 * 
	 */
	public SupplementalDataViewer(Composite parent, AbstractCdmViewPart viewPart) {
		super(parent, viewPart);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getSelection()
	 */
	@Override
	public ISelection getSelection() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setSelection(org.eclipse.jface.viewers.ISelection, boolean)
	 */
	@Override
	public void setSelection(ISelection selection, boolean reveal) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.view.AbstractDetailsViewer#showParts(eu.etaxonomy.cdm.model.common.AnnotatableEntity)
	 */
	@Override
	protected void showParts() {
		
		Object input;
		
		if(getInput() instanceof AbstractGroupedContainer){
			input = ((AbstractGroupedContainer) getInput()).getData();
		}else{
			input = getInput();
		}
		
		if(input instanceof IdentifiableMediaEntity){
			if(currentViewPart != VIEW_PART.IdentifiableMediaEntity){
				currentViewPart = VIEW_PART.IdentifiableMediaEntity;
				createIdentifiableMediaSections(rootElement);
			}
		}else if(input instanceof IdentifiableEntity){
			if(currentViewPart != VIEW_PART.IdentifiableEntity){
				currentViewPart = VIEW_PART.IdentifiableEntity;
				createIdentifiableSections(rootElement);
			}
		}else if(input instanceof AnnotatableEntity){
			if(currentViewPart != VIEW_PART.AnnotatableEntity){
				currentViewPart = VIEW_PART.AnnotatableEntity;
				createAnnotationSections(rootElement);
			}
		}else if(input instanceof VersionableEntity){
			if(currentViewPart != VIEW_PART.VersionableEntity){
				currentViewPart = VIEW_PART.VersionableEntity;
				createHeadlineSection(rootElement);
			}
		}else{
			currentViewPart = VIEW_PART.EMPTY;
		}
		
		layout();
	}
	
	private void createHeadlineSection(RootElement parent){
		destroySections();
		HeadlineSection headlineSection = formFactory.createHeadlineSection(parent);
		addPart(headlineSection);
		
		if(currentViewPart == VIEW_PART.VersionableEntity){
			createVersionSection(parent);
		}
	}
	
	private void createAnnotationSections(RootElement parent){
		createHeadlineSection(parent);
		AbstractFormSection annotationSection = 
			formFactory.createEntityDetailSection(EntityDetailType.ANNOTATION, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection markerSection = 
			formFactory.createEntityDetailSection(EntityDetailType.MARKER, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		addPart(annotationSection);
		addPart(markerSection);
		
		if(currentViewPart == VIEW_PART.AnnotatableEntity){
			createVersionSection(parent);
		}
	}

	private void createIdentifiableSections(RootElement parent){
		createAnnotationSections(parent);		
		AbstractFormSection creditSection = formFactory.createEntityDetailSection(EntityDetailType.CREDIT, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection extensionSection = formFactory.createEntityDetailSection(EntityDetailType.EXTENSION, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection rightsSection = formFactory.createEntityDetailSection(EntityDetailType.RIGHTS, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection sourceSection = formFactory.createEntityDetailSection(EntityDetailType.SOURCE, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		addPart(creditSection);
		addPart(extensionSection);
		addPart(rightsSection);
		addPart(sourceSection);
		
		if(currentViewPart == VIEW_PART.IdentifiableEntity){
			createVersionSection(parent);
		}
	}
	
	private void createIdentifiableMediaSections(RootElement parent){
		createIdentifiableSections(parent);
		AbstractFormSection mediaSection = formFactory.createEntityDetailSection(EntityDetailType.MEDIA, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		addPart(mediaSection);
		
		if(currentViewPart == VIEW_PART.IdentifiableMediaEntity){
			createVersionSection(parent);
		}
	}
	
	private void createVersionSection(RootElement parent){		
		VersionSection versionSection = formFactory.createVersionSection(parent, Section.NO_TITLE | Section.EXPANDED);
		addPart(versionSection);
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event) {
		if(event.getSource() instanceof DetailsViewer){
			detailsViewer = (DetailsViewer) event.getSource();
			
			if(event.getSelection() == CdmFormFactory.EMPTY_SELECTION){
				setInput(null);
				currentViewPart = VIEW_PART.EMPTY;
				return;
			}
			
			IStructuredSelection selection = (IStructuredSelection) event.getSelection(); 
			setInput(selection.getFirstElement());
		}
	}
}
