// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public abstract class AbstractCdmEntityWizardPage<T extends IdentifiableEntity> extends WizardPage implements IConversationEnabled {

	private static final Logger logger = Logger
			.getLogger(AbstractCdmEntityWizardPage.class);
	protected CdmFormFactory formFactory;
	protected T entity;

	AbstractCdmDetailElement detailElement;
	private Composite control;
	private ConversationHolder conversation;
	
	protected AbstractCdmEntityWizardPage(CdmFormFactory formFactory, ConversationHolder conversation, T entity) {
		super("page");
		this.formFactory = formFactory;
		this.entity = entity;
		this.conversation = conversation;
		setDescription(entity.getTitleCache());
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		control = formFactory.createComposite(parent);
		
		control.setLayout(CdmFormFactory.LAYOUT(2, false));
		WizardPageRootElement rootElement = new WizardPageRootElement(formFactory, control, getConversationHolder());
		
		detailElement = createElement(rootElement);
		
		setControl(control);
	}
	
	public T getEntity() {
		return entity;
	}
	
	/**
	 * Creates the detail element for this wizard page
	 * 
	 * @param rootElement
	 */
	public abstract AbstractCdmDetailElement createElement(ICdmFormElement rootElement);
	
	private class WizardPageRootElement extends RootElement implements IConversationEnabled{

		private ConversationHolder conversation;
		
		public WizardPageRootElement(CdmFormFactory formFactory, 
				Composite layoutComposite, ConversationHolder conversation) {
			super(formFactory, layoutComposite);
			this.conversation = conversation;
		}
		
		@Override
		public void refresh() {
			super.refresh();
			this.removeElements();
			detailElement = createElement(this);
			control.layout();
		}
		
		public ConversationHolder getConversationHolder() {
			return conversation;
		}
		
		public void update(CdmDataChangeMap changeEvents) {}
	}
	
	public ConversationHolder getConversationHolder() {
		return conversation;
	}
	
	public void update(CdmDataChangeMap changeEvents) {}
}
