/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TypedListener;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.forms.widgets.ToggleHyperlink;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;

/**
 * 
 * @author n.hoffmann
 * @created Feb 22, 2010
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractFormSection<T extends VersionableEntity> extends Section implements ISelectionChangedListener, IEntityElement<T>, IConversationEnabled{
	
	private static final Logger logger = Logger
	.getLogger(AbstractFormSection.class);
		
	private ISelectionProvider selectionProvider;
	
	private T entity; 
	
	private Set<ICdmFormElement> elements = new HashSet<ICdmFormElement>();
	
	protected CdmFormFactory formFactory;
		
	private ListenerList propertyChangeListeners;

	private ICdmFormElement parentElement;
	
	private ConversationHolder conversation;

	/**
	 * @param conversation TODO
	 * @param style
	 * @param parent
	 */
	protected AbstractFormSection(CdmFormFactory formFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(parentElement.getLayoutComposite(), style);
		
		this.parentElement = parentElement;
		
		this.formFactory = formFactory;
		
		this.conversation = conversation;
		
		this.setLayoutData(CdmFormFactory.FILL());
		
		Composite client = formFactory.createComposite(this, SWT.WRAP);
		client.setBackgroundMode(SWT.INHERIT_DEFAULT);
		
		TableWrapLayout layout = CdmFormFactory.LAYOUT();
		layout.bottomMargin = 10;
		layout.rightMargin = 5;
		
		client.setLayout(layout);

		this.setClient(client);
	}

	protected AbstractFormSection(CdmFormFactory formFactory, ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
		this(formFactory, conversation, parentElement, style);
		this.selectionProvider = selectionProvider;
	}
	
	public ListenerList getPropertyChangeListeners() {
		return propertyChangeListeners;
	}
	
	public void setPropertyChangeListeners(
			ListenerList propertyChangeListeners) {
		this.propertyChangeListeners = propertyChangeListeners;
	}

	/**
	 * 
	 * @param entity
	 */
	public void setEntity(T entity){
		this.entity = entity;
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IEntityElement#getEntity()
	 */
	public T getEntity() {
		return entity;
	}
		
	public ToggleHyperlink getToggle(){
		return this.toggle;
	}

	public AbstractFormSection<T> getSection(){
		return this;
	}	
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#firePropertyChangeEvent()
	 */
	public void firePropertyChangeEvent(CdmPropertyChangeEvent event) {
		Assert.isNotNull(propertyChangeListeners, "No property change listeners.");
		for(Object listener : propertyChangeListeners.getListeners()){
			((IPropertyChangeListener) listener).propertyChange(event);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Composite#setFocus()
	 */
	@Override
	public boolean setFocus() {
		return getClient().setFocus();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#setBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setBackground(Color color) {
		if(getLayoutComposite() != null){
			getLayoutComposite().setBackground(color);
			if(getLayoutComposite().getChildren().length != 0){
				for(Control child : getLayoutComposite().getChildren()){
					child.setBackground(color);
				}
			}
		}
		super.setBackground(color);
	}
	
	public void widgetSelected(SelectionEvent e) {
		Widget widget = e.widget;
		
		if(widget instanceof Control){
			Control control = (Control) widget;
			if(checkControlAncestryForWidget(control)){
				if(getEntity() != null){
					IStructuredSelection selection = new StructuredSelection(getEntity());
					if(selectionProvider != null){
						selectionProvider.setSelection(selection);
					}
				}
			}
		}	
	}
	
	private boolean checkControlAncestryForWidget(Control control){
		if(control.equals(this)){
			return true;
		}else{
			Control parent = control.getParent();
			if(parent == null){
				return false;
			}else{
				return checkControlAncestryForWidget(parent);
			}
		}
	}
	
	/**
	 * @param b
	 */
	public void setSelected(boolean selected) {
		if(selected){
			setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_LIST_SELECTION));
		}else{
			setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		}
	}
	

	public void selectionChanged(SelectionChangedEvent event) {
		if(event.getSelection() == formFactory.EMPTY_SELECTION){
			return;
		}
		
		IStructuredSelection selection = (IStructuredSelection) event.getSelection();
		setSelected(false);
		
		Object selectedObject =  selection.getFirstElement();
		
		if(selectedObject != null && selectedObject.equals(getEntity())){
			setSelected(true);
		}
	}
	
	/**
	 * 
	 * @param listener
	 */
	public void addSelectionListener(SelectionListener listener) {
		addListener(SWT.Selection, new TypedListener(listener));
	}
	 
	/**
	 * 
	 * @param listener
	 */
	public void removeSelectionListener(SelectionListener listener) {
		removeListener(SWT.Selection, listener);
	}
	
	public Color getColor(boolean selected) {
		return selected ? SELECTED : NOT_SELECTED;
	}
	
	public void addElement(ICdmFormElement element){
		elements.add(element);
	}
	
	protected void removeElement(ICdmFormElement element){
		elements.remove(element);
	}
	
	public void removeElements() {
		for(ICdmFormElement childElement : getElements()){
			// recursion
			childElement.removeElements();
			
			// unregister selection arbitrator
			if(childElement instanceof ISelectableElement){
				ISelectableElement selectableElement = (ISelectableElement) childElement;
				if (selectableElement != null && selectableElement.getSelectionArbitrator() != null) {
					formFactory.destroySelectionArbitrator(selectableElement.getSelectionArbitrator());
				}
			}
			
			// dispose of the controls
			for(Control control : childElement.getControls()){
				// we added the layoutComposite of the parental element as the layout composite to this formElement
				// but we do not want to destroy it.
				if(control.equals(childElement.getLayoutComposite())){
					continue;
				}else{
					control.dispose();
					control = null;
				}
			}
		}
		
		elements.clear();
	}
	
	public ICdmFormElement getParentElement() {
		return parentElement;
	}
	
	public Set<ICdmFormElement> getElements() {
		return elements;
	}
	
	public Set<Control> getControls() {
		Set<Control> controls = new HashSet<Control>();
		
		for (Control control : getChildren()){
			controls.add(control);
		}
		
		return controls; 
	}
	
	@Override
	public void dispose() {
		removeElements();
		super.dispose();
	}
	
	public Composite getLayoutComposite() {
		return (Composite) getClient();
	}
	
	public boolean containsFormElement(ICdmFormElement formElement){
		if(formElement == this){
			return true;
		}else{
			for(ICdmFormElement element : getElements()){
				boolean contains = element.containsFormElement(formElement);
				if(contains == true){
					return true;
				}
			}
			return false;
		}
	}


	public CdmFormFactory getFormFactory() {
		return formFactory;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormElement#refresh()
	 */
	public void refresh() {
		// empty default implementation
		
	}
	
	public ConversationHolder getConversationHolder() {
		return conversation;
	}
	
	public void update(CdmDataChangeMap changeEvents) {}
}
