/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import java.net.URI;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TypedEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.IFormColors;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.internal.forms.widgets.FormFonts;
import org.joda.time.Partial;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.AnnotationType;
import eu.etaxonomy.cdm.model.common.Credit;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.TimePeriod;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.Modifier;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.Scope;
import eu.etaxonomy.cdm.model.description.Sex;
import eu.etaxonomy.cdm.model.description.Stage;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.location.NamedAreaType;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.media.RightsTerm;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.NameTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.occurrence.FieldObservation;
import eu.etaxonomy.cdm.model.occurrence.PreservationMethod;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.strategy.parser.ParserProblem;
import eu.etaxonomy.taxeditor.forms.selection.AbstractSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.AgentSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.DerivedUnitBaseSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.FieldObservationSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.InstitutionSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.NameSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.NamedAreaSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.NomenclaturalAuthorTeamSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.PersonSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.ReferenceSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.TaxonSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.TeamSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.AbstractEnumComboElement;
import eu.etaxonomy.taxeditor.forms.term.AbstractTermComboElement;
import eu.etaxonomy.taxeditor.forms.term.AnnotationTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.ExtensionTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.LanguageComboElement;
import eu.etaxonomy.taxeditor.forms.term.MarkerTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.NameRelationshipTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.NameTypeDesignationStatusComboElement;
import eu.etaxonomy.taxeditor.forms.term.NamedAreaTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.NomenclaturalCodeComboElement;
import eu.etaxonomy.taxeditor.forms.term.NomenclaturalStatusTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.PresenceAbsenceTermComboElement;
import eu.etaxonomy.taxeditor.forms.term.PreservationMethodComboElement;
import eu.etaxonomy.taxeditor.forms.term.RankComboElement;
import eu.etaxonomy.taxeditor.forms.term.ReferenceTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.RightsTypeComboElement;
import eu.etaxonomy.taxeditor.forms.term.SexComboElement;
import eu.etaxonomy.taxeditor.forms.term.SpecimenTypeDesignationStatusComboElement;
import eu.etaxonomy.taxeditor.forms.term.StageComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.section.EmptyElement;
import eu.etaxonomy.taxeditor.section.agent.PersonDetailElement;
import eu.etaxonomy.taxeditor.section.agent.PersonDetailSection;
import eu.etaxonomy.taxeditor.section.agent.TeamDetailElement;
import eu.etaxonomy.taxeditor.section.agent.TeamDetailSection;
import eu.etaxonomy.taxeditor.section.agent.TeamMemberElement;
import eu.etaxonomy.taxeditor.section.agent.TeamMemberSection;
import eu.etaxonomy.taxeditor.section.agent.TeamOrPersonBaseDetailSection;
import eu.etaxonomy.taxeditor.section.description.DescriptionDetailElement;
import eu.etaxonomy.taxeditor.section.description.DescriptionDetailSection;
import eu.etaxonomy.taxeditor.section.description.DescriptionElementDetailElement;
import eu.etaxonomy.taxeditor.section.description.DescriptionElementDetailSection;
import eu.etaxonomy.taxeditor.section.description.DescriptionElementMediaSection;
import eu.etaxonomy.taxeditor.section.description.DescriptionElementSourceElement;
import eu.etaxonomy.taxeditor.section.description.DescriptionElementSourceSection;
import eu.etaxonomy.taxeditor.section.description.DescriptionSourceElement;
import eu.etaxonomy.taxeditor.section.description.DescriptionSourceSection;
import eu.etaxonomy.taxeditor.section.description.ModifierElement;
import eu.etaxonomy.taxeditor.section.description.ModifierSection;
import eu.etaxonomy.taxeditor.section.description.ScopeElement;
import eu.etaxonomy.taxeditor.section.description.ScopeSection;
import eu.etaxonomy.taxeditor.section.description.detail.AbstractDetailedDescriptionDetailElement;
import eu.etaxonomy.taxeditor.section.description.detail.CommonNameDetailElement;
import eu.etaxonomy.taxeditor.section.description.detail.DistributionDetailElement;
import eu.etaxonomy.taxeditor.section.description.detail.IndividualsAssociationDetailElement;
import eu.etaxonomy.taxeditor.section.description.detail.TaxonInteractionDetailElement;
import eu.etaxonomy.taxeditor.section.description.detail.TextDataDetailElement;
import eu.etaxonomy.taxeditor.section.name.AuthorshipDetailElement;
import eu.etaxonomy.taxeditor.section.name.AuthorshipDetailSection;
import eu.etaxonomy.taxeditor.section.name.NameDetailElement;
import eu.etaxonomy.taxeditor.section.name.NameDetailSection;
import eu.etaxonomy.taxeditor.section.name.NameRelationshipDetailElement;
import eu.etaxonomy.taxeditor.section.name.NameRelationshipDetailSection;
import eu.etaxonomy.taxeditor.section.name.NameTypeDesignationElement;
import eu.etaxonomy.taxeditor.section.name.NomenclaturalStatusElement;
import eu.etaxonomy.taxeditor.section.name.NomenclaturalStatusSection;
import eu.etaxonomy.taxeditor.section.name.NonViralNameDetailElement;
import eu.etaxonomy.taxeditor.section.name.NonViralNameDetailSection;
import eu.etaxonomy.taxeditor.section.name.ProtologueElement;
import eu.etaxonomy.taxeditor.section.name.ProtologueSection;
import eu.etaxonomy.taxeditor.section.name.SpecimenTypeDesignationElement;
import eu.etaxonomy.taxeditor.section.name.TypeDesignationSection;
import eu.etaxonomy.taxeditor.section.occurrence.DerivedUnitBaseDetailElement;
import eu.etaxonomy.taxeditor.section.occurrence.DerivedUnitBaseDetailSection;
import eu.etaxonomy.taxeditor.section.occurrence.DerivedUnitFacadeDetailElement;
import eu.etaxonomy.taxeditor.section.occurrence.DerivedUnitFacadeDetailSection;
import eu.etaxonomy.taxeditor.section.occurrence.FieldObservationDetailElement;
import eu.etaxonomy.taxeditor.section.occurrence.FieldObservationDetailSection;
import eu.etaxonomy.taxeditor.section.occurrence.GatheringEventDetailElement;
import eu.etaxonomy.taxeditor.section.occurrence.GatheringEventDetailSection;
import eu.etaxonomy.taxeditor.section.reference.NomenclaturalReferenceDetailElement;
import eu.etaxonomy.taxeditor.section.reference.NomenclaturalReferenceDetailSection;
import eu.etaxonomy.taxeditor.section.reference.ReferenceBaseDetailElement;
import eu.etaxonomy.taxeditor.section.reference.ReferenceBaseDetailSection;
import eu.etaxonomy.taxeditor.section.supplemental.AnnotationElement;
import eu.etaxonomy.taxeditor.section.supplemental.AnnotationSection;
import eu.etaxonomy.taxeditor.section.supplemental.CreditElement;
import eu.etaxonomy.taxeditor.section.supplemental.CreditSection;
import eu.etaxonomy.taxeditor.section.supplemental.ExtensionElement;
import eu.etaxonomy.taxeditor.section.supplemental.ExtensionSection;
import eu.etaxonomy.taxeditor.section.supplemental.HeadlineSection;
import eu.etaxonomy.taxeditor.section.supplemental.IdentifiableSourceElement;
import eu.etaxonomy.taxeditor.section.supplemental.ImageFileElement;
import eu.etaxonomy.taxeditor.section.supplemental.MarkerElement;
import eu.etaxonomy.taxeditor.section.supplemental.MarkerSection;
import eu.etaxonomy.taxeditor.section.supplemental.MediaElement;
import eu.etaxonomy.taxeditor.section.supplemental.MediaRepresentationElement;
import eu.etaxonomy.taxeditor.section.supplemental.MediaRepresentationPartSection;
import eu.etaxonomy.taxeditor.section.supplemental.MediaRepresentationSection;
import eu.etaxonomy.taxeditor.section.supplemental.MediaSection;
import eu.etaxonomy.taxeditor.section.supplemental.RightsElement;
import eu.etaxonomy.taxeditor.section.supplemental.RightsSection;
import eu.etaxonomy.taxeditor.section.supplemental.SourceSection;
import eu.etaxonomy.taxeditor.section.supplemental.VersionElement;
import eu.etaxonomy.taxeditor.section.supplemental.VersionSection;
import eu.etaxonomy.taxeditor.section.taxon.ParsingMessageElement;
import eu.etaxonomy.taxeditor.section.taxon.ParsingMessagesSection;
import eu.etaxonomy.taxeditor.section.taxon.TaxonBaseDetailElement;
import eu.etaxonomy.taxeditor.section.taxon.TaxonBaseDetailSection;

/**
 * 
 * @author n.hoffmann
 * @created Feb 24, 2010
 * @version 1.0
 */
public class CdmFormFactory extends FormToolkit{

	private BoldFontHolder2 boldFontHolder2;
	private MouseListener selectionMouseHandler;
	private FocusListener selectionFocusHandler;
	
	private ListenerList selectionListenerList = new ListenerList();
	
	private ListenerList propertyChangeListeners = new ListenerList();
	
	private int orientation = Window.getDefaultOrientation();
	private ISelectionProvider selectionProvider;
	
	/**
	 * 
	 * @return
	 */
	public static final TableWrapLayout LAYOUT(){
		return LAYOUT(1, false);
	}
	
	/**
	 * 
	 * @param columns
	 * @param equalWidth
	 * @return
	 */
	public static final TableWrapLayout LAYOUT(int columns, boolean equalWidth){
		TableWrapLayout layout = new TableWrapLayout();
		layout.topMargin = 0;
		layout.rightMargin = 0;
		layout.bottomMargin = 0;
		layout.leftMargin = 0;
		layout.numColumns = columns;
		layout.makeColumnsEqualWidth = equalWidth;
		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		return layout;
	}
	
	/**
	 * 
	 * @return
	 */
	public static final TableWrapData FILL(){
		TableWrapData layoutData = new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB);
//		layoutData.grabHorizontal = true;
//		layoutData.grabVertical = true;
		return layoutData;
	}
	
	public static final TableWrapData LEFT(){
		return new TableWrapData(TableWrapData.LEFT, TableWrapData.TOP);
	}
	
	public static final TableWrapData RIGHT(){
		return new TableWrapData(TableWrapData.RIGHT, TableWrapData.TOP);
	}
	
	/**
	 * 
	 * @return
	 */
	public static final TableWrapData FILL_HORIZONTALLY(){
		return FILL_HORIZONTALLY(1, 1);
	}
	
	/**
	 * 
	 * @param horizontalSpan
	 * @param verticalSpan
	 * @return
	 */
	public static final TableWrapData FILL_HORIZONTALLY(int horizontalSpan, int verticalSpan){
		TableWrapData layoutData = new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.TOP);
		layoutData.colspan = horizontalSpan;
		layoutData.rowspan = verticalSpan;
		return layoutData;
	}
	
	public static ISelection EMPTY_SELECTION = new ISelection() {
		public boolean isEmpty() {
			return true;
		}
	};
	
	private class SelectionMouseHandler extends MouseAdapter{
		@Override
		public void mouseDown(MouseEvent e) {
			notifySelectionListeners(e);
		}
	}
	
	private class SelectionFocusHandler extends FocusAdapter{
		@Override
		public void focusGained(FocusEvent e) {
			notifySelectionListeners(e);
		}
	}
	
	private void notifySelectionListeners(TypedEvent e){
		Event event = new Event();
		event.widget = e.widget;
		SelectionEvent selectionEvent = new SelectionEvent(event);
		
		for(Object listener : selectionListenerList.getListeners()){
			((SelectionListener) listener).widgetSelected(selectionEvent);
		}
	}
	
	/**
	 * 
	 * @param display
	 * @param selectionProvider
	 */
	public CdmFormFactory(Display display, ISelectionProvider selectionProvider) {
		super(display);
		this.selectionProvider = selectionProvider;
		init();
	}
	
	/**
	 * 
	 * @param display
	 */
	public CdmFormFactory(Display display){
		super(display);
		init();
	}
	
	/**
	 * 
	 */
	private void init() {
		boldFontHolder2 = new BoldFontHolder2();
		selectionMouseHandler = new SelectionMouseHandler();
		selectionFocusHandler = new SelectionFocusHandler();
	}
	
	/**
	 * Creates an instance initialized with the correct selectionProvider
	 * 
	 * Make sure to remove the instance when the entityComposite disposes via
	 * destroySelectionArbitrator(..)
	 * @return
	 */
	public SelectionArbitrator createSelectionArbitrator(IEntityElement entityElement){
		SelectionArbitrator selectionArbitrator = new SelectionArbitrator(entityElement);
		selectionArbitrator.addSelectionProvider(selectionProvider);
		selectionProvider.addSelectionChangedListener(selectionArbitrator);
		addSelectionListener(selectionArbitrator);
		return selectionArbitrator;
	}
	
	public void destroySelectionArbitrator(SelectionArbitrator selectionArbitrator){
		removeSelectionListener(selectionArbitrator);
		selectionProvider.removeSelectionChangedListener(selectionArbitrator);
	}

	@Override
	public void adapt(Composite composite) {
		composite.addMouseListener(selectionMouseHandler);
		super.adapt(composite);
	}
	
	public void adapt(AbstractCdmFormElement formElement){		
		formElement.setPropertyChangeListeners(propertyChangeListeners);
	}
	
	@Override
	public void adapt(Control control, boolean trackFocus, boolean trackKeyboard) {
		if(trackFocus){
			control.addFocusListener(selectionFocusHandler);
		}
		super.adapt(control, trackFocus, trackKeyboard);
	}
	
	public void destroyElement(ICdmFormElement formElement){
		// return if element was not initialized
		if(formElement == null){
			return;
		}
		// destroy selection arbitrator, if any
		if(formElement instanceof ISelectableElement){
			destroySelectionArbitrator(((ISelectableElement) formElement).getSelectionArbitrator());
		}
		// remove this element form its parents list of elements
//		ICdmFormElement parentElement = formElement.getParentElement();
//		if(parentElement != null){
//			parentElement.removeElement(formElement);
//		}
		// call destroy on child elements recursively
		for(ICdmFormElement childElement : formElement.getElements()){
			destroyElement(childElement);
		}
		// dispose of the controls
		for(Control control : formElement.getControls()){
			// we added the layoutComposite of the parental element as the layout composite to this formElement
			// but we do not want to destroy it.
			if(control.equals(formElement.getLayoutComposite())){
				continue;
			}else{
				control.dispose();
				control = null;
			}
		}
	}
	
	
	public Label createEmptyCell(Composite parent){
		return this.createLabel(parent, null);
	}
	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param textHeight
	 * @param style
	 * @return
	 */
	public TextWithLabelElement createMultilineTextWithLabel(ICdmFormElement parentElement, String labelString, int textHeight, int style){
		TextWithLabelElement element = new TextWithLabelElement(this, parentElement, labelString, "", textHeight, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param initialText
	 * @param style
	 * @return
	 */
	public TextWithLabelElement createTextWithLabelElement(ICdmFormElement parentElement, String labelString, String initialText, int style){
		TextWithLabelElement element = new TextWithLabelElement(this, parentElement, labelString, initialText, null, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public IntegerTextWithLabelElement createIntegerTextWithLabelElement(ICdmFormElement parentElement, String labelString, Integer initialInteger, int style){
		IntegerTextWithLabelElement element = new IntegerTextWithLabelElement(this, parentElement, labelString, initialInteger, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public LanguageStringWithLabelElement createLanguageStringWithLabelElement(ICdmFormElement parentElement, String labelString, LanguageString languageString, int style){
		LanguageStringWithLabelElement element = new LanguageStringWithLabelElement(this, parentElement, labelString, languageString, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public LanguageStringWithLabelElement createLanguageStringWithLabelElement(ICdmFormElement parentElement, String labelString, LanguageString languageString, int height, int style){
		LanguageStringWithLabelElement element = new LanguageStringWithLabelElement(this, parentElement, labelString, languageString, height, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	/**
	 * 
	 * @param parentElement
	 * @param labelString
	 * @param initialText
	 * @param style
	 * @return
	 */
	public TextWithLabelElement createTextElement(ICdmFormElement parentElement, String initialText, int style){
		TextWithLabelElement element = new TextWithLabelElement(this, parentElement, null, initialText, null, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public KeyValueViewerElement createKeyValueViewerElement(ICdmFormElement parentElement, String keyHeading, String valueHeading, Map<Object, Object> map){
		KeyValueViewerElement element = new KeyValueViewerElement(this, parentElement, keyHeading, valueHeading, map);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	
	public static enum TermComboType{
		EXTENSIONTYPE,
		LANGUAGE,
		MARKERTYPE,
		NOMENCLATURALSTATUSTYPE,
		RANK,
		RIGHTSTYPE, 
		NAMERELATIONSHIP, 
		NAMEDAREATYPE, 
		PRESENCEABSENCESTATUS, 
		ANNOTATIONTYPE,
		NAMETYPEDESIGNATIONSTATUS,
		SPECIMENTYPEDESIGNATIONSTATUS,
		SEX,
		STAGE,
		PRESERVATION_METHOD
	}
	
	public AbstractTermComboElement createTermComboElement(TermComboType termComboType, ICdmFormElement parentElement, DefinedTermBase selection, int style){
		AbstractTermComboElement element = null;
		
		switch(termComboType){
		case EXTENSIONTYPE:
			element = new ExtensionTypeComboElement(this, parentElement, null, (ExtensionType) selection, style);
			break;
		case LANGUAGE:
			element = new LanguageComboElement(this, parentElement, "Language", (Language) selection, style);
			break;
		case MARKERTYPE:
			element = new MarkerTypeComboElement(this, parentElement, null, (MarkerType) selection, style);
			break;
		case NOMENCLATURALSTATUSTYPE:
			element = new NomenclaturalStatusTypeComboElement(this, parentElement, "Nomenclatural Status Type", (NomenclaturalStatusType) selection, style);
			break;
		case RANK:
			element = new RankComboElement(this, parentElement, "Rank", (Rank) selection, style);
			break;
		case RIGHTSTYPE:
			element = new RightsTypeComboElement(this, parentElement, "Rights Type", (RightsTerm) selection, style);
			break;
		case NAMERELATIONSHIP:
			element = new NameRelationshipTypeComboElement(this, parentElement, "Name Relationship Type", (NameRelationshipType) selection, style);
			break;
		case NAMEDAREATYPE:
			element = new NamedAreaTypeComboElement(this, parentElement, "Named Area Type", (NamedAreaType) selection, style);
			break;
		case ANNOTATIONTYPE:
			element = new AnnotationTypeComboElement(this, parentElement, "Annotation Type", (AnnotationType) selection, style);
			break;
		case PRESENCEABSENCESTATUS:
			element = new PresenceAbsenceTermComboElement(this, parentElement, "Distribution Status", (PresenceAbsenceTermBase) selection, style);
			break;
		case NAMETYPEDESIGNATIONSTATUS:
			element = new NameTypeDesignationStatusComboElement(this, parentElement, "Designation Status", (NameTypeDesignationStatus) selection, style);
			break;
		case SPECIMENTYPEDESIGNATIONSTATUS:
			element = new SpecimenTypeDesignationStatusComboElement(this, parentElement, "Designation Status", (SpecimenTypeDesignationStatus) selection, style);
			break;
		case SEX:
			element = new SexComboElement(this, parentElement, "Sex", (Sex) selection, style);
			break;
		case STAGE:
			element = new StageComboElement(this, parentElement, "Life Stage", (Stage) selection, style);
			break;
		case PRESERVATION_METHOD:
			element = new PreservationMethodComboElement(this, parentElement, "Preservation Method", (PreservationMethod) selection, style);
		}
		
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public static enum EnumComboType{
		REFERENCETYPE, 
		NOMENCLATURALCODE
	}
	
	public AbstractEnumComboElement createEnumComboElement(EnumComboType enumComboType, ICdmFormElement parentElement, int style){
		AbstractEnumComboElement element = null;

		switch(enumComboType){
		case REFERENCETYPE:
			element = new ReferenceTypeComboElement(this, parentElement, "Reference Type", null, style);
			break;
		case NOMENCLATURALCODE:
			element = new NomenclaturalCodeComboElement(this, parentElement, "Nomenclatural Code", null, style);
			break;
		}
		adapt(element);
		parentElement.addElement(element);
		return element;
	}	
	
	/**
	 * 
	 * @param parent
	 * @param imageUri
	 * @param style
	 * @return
	 */
	public ImageElement createImageElement(ICdmFormElement parentElement, URI imageUri, int style){
		ImageElement element = new ImageElement(this, parentElement, imageUri, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	/**
	 * 
	 * @param parent
	 * @param labelString
	 * @param initialText
	 * @param style
	 * @return
	 */
	public TextActionElement createTextActionElement(ICdmFormElement parentElement, String labelString, String buttonLabel, String initialText, int style){
		TextActionElement element = new TextActionElement(this, parentElement, labelString, buttonLabel, initialText, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	/**
	 * 	
	 * @author n.hoffmann
	 * @created Mar 5, 2010
	 * @version 1.0
	 */
	public static enum SelectionType{
		AGENT,
		INSTITUTION,
		NAME,
		REFERENCE,
		TAXON,
		TEAM, 
		AUTHOR_TEAM,
		PERSON, 
		DERIVED_UNIT, 
		FIELD_OBSERVATION
	}	
		
	/**
	 * @param parent
	 * @param string
	 * @param b
	 * @param taxonBasePropertySection
	 * @return
	 */
	public CheckboxElement createCheckbox(ICdmFormElement parentElement, String label, boolean initialState, int style) {
		CheckboxElement element = new CheckboxElement(this, parentElement, label, initialState, style | orientation);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

	
	
	/**
	 * Creates a section as a part of the form.
	 * 
	 * @param parent
	 *            the section parent
	 * @param sectionStyle
	 *            the section style
	 * @return the section widget
	 */
	public Section adapt(AbstractFormSection section) {
		section.setMenu(section.getLayoutComposite().getMenu());
		adapt(section, true, true);
		
		// handle focus and property change events for cdm use
		section.addFocusListener(selectionFocusHandler);
		section.setPropertyChangeListeners(propertyChangeListeners);
		
		if (section.getToggle() != null) {
			section.getToggle().setHoverDecorationColor(getColors()
					.getColor(IFormColors.TB_TOGGLE_HOVER));
			section.getToggle().setDecorationColor(getColors()
					.getColor(IFormColors.TB_TOGGLE));
		}
		
		section.setFont(boldFontHolder2.getBoldFont(section.getLayoutComposite().getFont()));

		if ((section.getStyle() & Section.TITLE_BAR) != 0
				|| (section.getStyle() & Section.SHORT_TITLE_BAR) != 0) {
			getColors().initializeSectionToolBarColors();
			section.setTitleBarBackground(getColors().getColor(IFormColors.TB_BG));
			section.setTitleBarBorderColor(getColors()
					.getColor(IFormColors.TB_BORDER));
		}
		// call setTitleBarForeground regardless as it also sets the label color
		section.setTitleBarForeground(getColors()
				.getColor(IFormColors.TB_TOGGLE));
		return section;
	}
	
	private class BoldFontHolder2 {
		private Font normalFont;

		private Font boldFont;

		public BoldFontHolder2() {
		}

		public Font getBoldFont(Font font) {
			createBoldFont(font);
			return boldFont;
		}

		private void createBoldFont(Font font) {
			if (normalFont == null || !normalFont.equals(font)) {
				normalFont = font;
				dispose();
			}
			if (boldFont == null) {
				boldFont = FormFonts.getInstance().getBoldFont(getColors().getDisplay(),
						normalFont);
			}
		}

		public void dispose() {
			if (boldFont != null) {
				FormFonts.getInstance().markFinished(boldFont, getColors().getDisplay());
				boldFont = null;
			}
		}
	}
	
	public ToggleableTextElement createToggleableTextField(ICdmFormElement parentElement,
			String labelString,	String initialText, boolean initialState, int style) {
		ToggleableTextElement element = new ToggleableTextElement(this, parentElement,
				labelString, initialText, initialState, style | orientation);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public TimePeriodElement createTimePeriodElement(
			ICdmFormElement parentElement, String labelString, TimePeriod timePeriod, int style) {
		TimePeriodElement element = new TimePeriodElement(this, parentElement, labelString, timePeriod, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public DateDetailSection createDateDetailSection(ICdmFormElement parentElement, int style){
		DateDetailSection section = new DateDetailSection(this, parentElement, style);
		parentElement.addElement(section);
		adapt(section);
		return section;
	}
	
	public PartialElement createPartialElement(ICdmFormElement parentElement, String labelString, Partial partial, int style){
		PartialElement element = new PartialElement(this, parentElement, labelString, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

	public void addSelectionListener(SelectionListener listener){
		selectionListenerList.add(listener);
	}
	
	public void removeSelectionListener(SelectionListener listener){
		selectionListenerList.remove(listener);
	}
	
	public void addPropertyChangeListener(IPropertyChangeListener listener) {
		propertyChangeListeners.add(listener);
	}
	
	public void removePropertyChangeListener(IPropertyChangeListener listener){
		propertyChangeListeners.remove(listener);
	}



	public Label createHorizontalSeparator(ICdmFormElement parentElement, int style) {
		Label separator = this.createSeparator(parentElement.getLayoutComposite(), SWT.HORIZONTAL | style);
		separator.setLayoutData(FILL_HORIZONTALLY());
		return separator;
	}
	
	public VersionElement createVersionElement(ICdmFormElement parentElement, VersionableEntity entity,
			int style) {
		VersionElement element = new VersionElement(this, parentElement, entity, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

	public VersionSection createVersionSection(ICdmFormElement parentElement, int style) {
		VersionSection section = new VersionSection(this, parentElement, style);
		parentElement.addElement(section);
		adapt(section);
		return section;
	}
	
	public EmptyElement createEmptyElement(ICdmFormElement parentElement) {
		EmptyElement element = new EmptyElement(this, parentElement, null, SWT.NULL);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public HeadlineSection createHeadlineSection(ICdmFormElement parentElement) {
		HeadlineSection section = new HeadlineSection(this, parentElement, SWT.NULL);
		parentElement.addElement(section);
		adapt(section);
		return section;
	}


	public ParsingMessageElement createParsingMessageElement(
			ICdmFormElement parentElement, ParserProblem parserProblem, int style) {
		ParsingMessageElement element = new ParsingMessageElement(this, parentElement, parserProblem, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

	
	// SIMPLIFY
	
	/**
	 * 
	 */
	public static enum DetailType{
		NONVIRALNAME,
		SCIENTIFICNAME,
		REFERENCEBASE,
		NOMENCLATURALREFERENCE,
		TAXONBASE,
		AUTHORSHIP,
		TEAM,
		PERSON,
		DESCRIPTION,
		DESCRIPTIONELEMENT, 
		PARSINGMESSAGE,
		NAMERELATIONSHIP, 
		TEAMORPERSONBASE,
		MEDIA, 
		FIELD_OBSERVATION, 
		DERIVED_UNIT_FACADE, 
		GATHERING_EVENT, 
		DERIVED_UNIT
	}
	
	/**
	 * 
	 * @param detailType
	 * @param parentElement
	 * @param selectionProvider
	 * @param style
	 * @return
	 */
	public AbstractFormSection createCdmDetailSection(DetailType detailType, ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style){
		AbstractFormSection section = null;
		
		switch(detailType){
		case SCIENTIFICNAME:
			section = new NameDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case REFERENCEBASE:
			section = new ReferenceBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case NOMENCLATURALREFERENCE:
			section = new NomenclaturalReferenceDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case TAXONBASE:
			section = new TaxonBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case AUTHORSHIP:
			section = new AuthorshipDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case TEAMORPERSONBASE:
			section = new TeamOrPersonBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case TEAM:
			section = new TeamDetailSection(this, conversation, parentElement, null, style);
			break;
		case PERSON:
			section = new PersonDetailSection(this, conversation, parentElement, null, style);
			break;
		case DESCRIPTION:
			section = new DescriptionDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case DESCRIPTIONELEMENT:
			section = new DescriptionElementDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case PARSINGMESSAGE:
			section = new ParsingMessagesSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case NONVIRALNAME:
			section = new NonViralNameDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case MEDIA:
			section = new eu.etaxonomy.taxeditor.section.description.MediaDetailsSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case DERIVED_UNIT_FACADE:
			section = new DerivedUnitFacadeDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case FIELD_OBSERVATION:
			section = new FieldObservationDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case GATHERING_EVENT:
			section = new GatheringEventDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		case DERIVED_UNIT:
			section = new DerivedUnitBaseDetailSection(this, conversation, parentElement, selectionProvider, style);
			break;
		}
		parentElement.addElement(section);
		adapt(section);
		return section;
	}
	
	/**
	 * 
	 * @param detailType
	 * @param parent
	 * @param style
	 * @return
	 */
	public AbstractCdmDetailElement createCdmDetailElement(DetailType detailType, ICdmFormElement parentElement, int style){
		AbstractCdmDetailElement element = null;
		
		switch(detailType){
		case SCIENTIFICNAME:
			element = new NameDetailElement(this, parentElement, style);
			break;
		case REFERENCEBASE:
			element = new ReferenceBaseDetailElement(this, parentElement, style);
			break;
		case NOMENCLATURALREFERENCE:
			element = new NomenclaturalReferenceDetailElement(this, parentElement, style);
			break;
		case TAXONBASE:
			element = new TaxonBaseDetailElement(this, parentElement, style);
			break;
		case AUTHORSHIP:
			element = new AuthorshipDetailElement(this, parentElement, style);
			break;
		case TEAM:
			element = new TeamDetailElement(this, parentElement, style);
			break;
		case PERSON:
			element = new PersonDetailElement(this, parentElement, style);
			break;
		case DESCRIPTION:
			element = new DescriptionDetailElement(this, parentElement, style);
			break;
		case DESCRIPTIONELEMENT:
			element = new DescriptionElementDetailElement(this, parentElement, style);
			break;
		case NONVIRALNAME:
			element = new NonViralNameDetailElement(this, parentElement);
			break;
		case DERIVED_UNIT_FACADE:
			element = new DerivedUnitFacadeDetailElement(this, parentElement);
			break;
		case FIELD_OBSERVATION:
			element = new FieldObservationDetailElement(this, parentElement);
			break;
		case GATHERING_EVENT:
			element = new GatheringEventDetailElement(this, parentElement);
			break;
		case DERIVED_UNIT:
			element = new DerivedUnitBaseDetailElement(this, parentElement);
			break;
		}
		
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created Mar 5, 2010
	 * @version 1.0
	 */
	public static enum EntityDetailType{
		TEAM,
		TEAMMEMBER,
		ANNOTATION,
		CREDIT,
		DESCRIPTIONELEMENTSOURCE,
		EXTENSION,
		MARKER,
		MEDIA, 
		DESCRIPTIONELEMENTMEDIA,
		MEDIAREPRESENTATION,
		MEDIAREPRESENTATIONPART,
		MODIFIER,
		NOMENCLATURALSTATUS,
		NAME_RELATIONSHIP,
		PROTOLOG,
		RIGHTS,
		SOURCE, 
		SCOPE, 
		DESCRIPTIONSOURCE, 
		TYPEDESIGNATION
	}
	
	/**
	 * 
	 * @param entityDetailType
	 * @param parent
	 * @param style
	 * @return
	 */
	public AbstractFormSection createEntityDetailSection(EntityDetailType entityDetailType, ConversationHolder conversation, ICdmFormElement parentElement, int style){
		AbstractFormSection section = null;
		
		switch(entityDetailType){
		case TEAM:
			section = new TeamDetailSection(this, conversation, parentElement, null, style);
			break;
		case TEAMMEMBER:
			section = new TeamMemberSection(this, conversation, parentElement, style);
			break;
		case ANNOTATION:
			section = new AnnotationSection(this, conversation, parentElement, style);
			break;
		case CREDIT:
			section = new CreditSection(this, conversation, parentElement, style);
			break;
		case DESCRIPTIONELEMENTSOURCE:
			section = new DescriptionElementSourceSection(this, conversation, parentElement, style);
			break;
		case EXTENSION:
			section = new ExtensionSection(this, conversation, parentElement, style);
			break;
		case MARKER:
			section = new MarkerSection(this, conversation, parentElement, style);
			break;
		case MEDIA:
			section = new MediaSection(this, conversation, parentElement, style);
			break;
		case DESCRIPTIONELEMENTMEDIA:
			section = new DescriptionElementMediaSection(this, conversation, parentElement, style);
			break;
		case MEDIAREPRESENTATION:
			section = new MediaRepresentationSection(this, conversation, parentElement, style);
			break;
		case MEDIAREPRESENTATIONPART:
			section = new MediaRepresentationPartSection(this, conversation, parentElement, style);
			break;
		case MODIFIER:
			section = new ModifierSection(this, conversation, parentElement, style);
			break;
		case NOMENCLATURALSTATUS:
			section = new NomenclaturalStatusSection(this, conversation, parentElement, style);
			break;
		case NAME_RELATIONSHIP:
			section = new NameRelationshipDetailSection(this, conversation, parentElement, style);
			break;
		case PROTOLOG:
			section = new ProtologueSection(this, conversation, parentElement, style);
			break;
		case RIGHTS:
			section = new RightsSection(this, conversation, parentElement, style);
			break;
		case SOURCE:
			section = new SourceSection(this, conversation, parentElement, style);
			break;
		case SCOPE:
			section = new ScopeSection(this, conversation, parentElement, style);
			break;
		case DESCRIPTIONSOURCE:
			section = new DescriptionSourceSection(this, conversation, parentElement, style);
			break;
		case TYPEDESIGNATION:
			section = new TypeDesignationSection(this, conversation, parentElement, style);
			break;
		}
		parentElement.addElement(section);
		adapt(section);
		return section;
	}
	
	/**
	 * 
	 * @param parent
	 * @param entity
	 * @param removeListener
	 * @param style
	 * @return
	 */
	public AbstractEntityCollectionElement createEntityCollectionElement(AbstractFormSection parentElement, VersionableEntity versionableEntity, SelectionListener removeListener, Color backgroundColor, int style){
		AbstractEntityCollectionElement element = null;
		
		VersionableEntity entity = (VersionableEntity) HibernateProxyHelper.deproxy(versionableEntity);
		
		if(entity instanceof Annotation){
			element = new AnnotationElement(this, parentElement, (Annotation) entity, removeListener, style);
		}
		else if(entity instanceof Person){
			element = new TeamMemberElement(this, parentElement, (Person) entity, removeListener, style);
		}
		else if(entity instanceof Credit){
			element = new CreditElement(this, parentElement, (Credit) entity, removeListener, style);
		}
		else if(entity instanceof Extension){
			element = new ExtensionElement(this, parentElement, (Extension) entity, removeListener, style);
		}
		else if(entity instanceof Marker){
			element = new MarkerElement(this, parentElement, (Marker) entity, removeListener, style);
		}
		else if(entity instanceof Media){
			element = new MediaElement(this, parentElement, (Media) entity, removeListener, style);
		}
		else if(entity instanceof MediaRepresentation){
			element = new MediaRepresentationElement(this, parentElement, (MediaRepresentation) entity, removeListener, style);
		}
		else if(entity instanceof ImageFile){
			element = new ImageFileElement(this, parentElement, (ImageFile) entity, removeListener, style);
		}
		else if(entity instanceof Modifier){
			element = new ModifierElement(this, parentElement, (Modifier) entity, removeListener, style);
		}
		else if(entity instanceof NomenclaturalStatus){
			element = new NomenclaturalStatusElement(this, parentElement, (NomenclaturalStatus) entity, removeListener, style);
		}
		else if(entity instanceof Rights){
			element = new RightsElement(this, parentElement, (Rights) entity, removeListener, style);
		}
		else if(entity instanceof DescriptionElementSource){
			element = new DescriptionElementSourceElement(this, parentElement, (DescriptionElementSource) entity, removeListener, style);
		}
		else if(entity instanceof IdentifiableSource){
			element = new IdentifiableSourceElement(this, parentElement, (IdentifiableSource) entity, removeListener, style);
		}
		else if(entity instanceof Scope){
			element = new ScopeElement(this, parentElement, (Scope) entity, removeListener, style);
		}
		else if(entity instanceof ReferenceBase){
			element = new DescriptionSourceElement(this, parentElement, (ReferenceBase) entity, removeListener, style);
		}
		else if(entity instanceof NameTypeDesignation){
			element = new NameTypeDesignationElement(this, parentElement, (NameTypeDesignation) entity, removeListener, style);
		}
		else if(entity instanceof NameRelationship){
			element = new NameRelationshipDetailElement(this, parentElement, (NameRelationship) entity, removeListener, style);
		}
		else if(entity instanceof SpecimenTypeDesignation){
			element = new SpecimenTypeDesignationElement(this, parentElement, (SpecimenTypeDesignation) entity, removeListener, style);
		}
		
		else if(entity instanceof DescriptionElementBase){
			// this is the special case for protologs, maybe we can do this differently when API improves
			DescriptionElementBase descriptionElement = (DescriptionElementBase) entity;
			if(descriptionElement.getFeature().equals(Feature.PROTOLOGUE())){
				element = new ProtologueElement(this, parentElement, descriptionElement, removeListener, style);
			}
		}
		
		Assert.isNotNull(element, "Composite should not be null at this point. Looks like the case is not handled already. Check implementation.");
		
		if(backgroundColor != null && !backgroundColor.isDisposed()){
			element.setBackgroundColor(backgroundColor);
		}
		
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	/**
	 * 
	 * @param selectionType
	 * @param parentElement
	 * @param labelString
	 * @param selection
	 * @param style
	 * @return
	 */
	public AbstractSelectionElement createSelectionElement(SelectionType selectionType,  ConversationHolder conversation, ICdmFormElement parentElement, String labelString, IdentifiableEntity selection, int style){
		AbstractSelectionElement element = null;
		
		IdentifiableEntity entity = (IdentifiableEntity) HibernateProxyHelper.deproxy(selection);
		
		switch (selectionType) {
		case AGENT:
			element = new AgentSelectionElement(this, conversation, parentElement, labelString, (TeamOrPersonBase) entity, style);
			break;
		case PERSON:
			element = new PersonSelectionElement(this, conversation, parentElement, labelString, (Person) entity, style);
			break;
		case TEAM:
			element = new TeamSelectionElement(this, conversation, parentElement, labelString, (Team) entity, style);
			break;
		case AUTHOR_TEAM:
			element = new NomenclaturalAuthorTeamSelectionElement(this, conversation, parentElement, labelString, (Team) entity, style);
			break;
		case INSTITUTION:
			element = new InstitutionSelectionElement(this, conversation, parentElement, labelString, (Institution) entity, style);
			break;
		case NAME:
			element = new NameSelectionElement(this, conversation, parentElement, labelString, (TaxonNameBase) entity, style);
			break;
		case REFERENCE:
			element = new ReferenceSelectionElement(this, conversation, parentElement, labelString, (ReferenceBase) entity, style);
			break;
		case TAXON:
			element = new TaxonSelectionElement(this, conversation, parentElement, labelString, (TaxonBase) entity, style);
			break;
		case DERIVED_UNIT:
			element = new DerivedUnitBaseSelectionElement(this, conversation, parentElement, labelString, (DerivedUnitBase) entity, style);
			break;
		case FIELD_OBSERVATION:
			element = new FieldObservationSelectionElement(this, conversation, parentElement, labelString, (FieldObservation) entity, style);
			break;
		default:
			new RuntimeException("There is no selection composite for the desired selection type.");
			break;
		}
		
		adapt(element);
		parentElement.addElement(element);
		return element;
	}
	
	public NamedAreaSelectionElement createNamedAreaSelectionElement(ICdmFormElement parentElement, ConversationHolder conversation, String labelString, NamedArea selection, int style){
		NamedAreaSelectionElement element = new NamedAreaSelectionElement(this, conversation, parentElement, labelString, selection, style);
		adapt(element);
		parentElement.addElement(element);
		return element;
	}

	public LabelElement createLabel(ICdmFormElement parentElement, String text) {
		LabelElement labelElement = new LabelElement(this, parentElement, text);
		adapt(labelElement);
		parentElement.addElement(labelElement);
		return labelElement;
	}

	public ISelectionProvider getSelectionProvider() {
		return selectionProvider;
	}

	public AbstractDetailedDescriptionDetailElement createDetailedDescriptionDetailElement(
			ICdmFormElement parentElement, DescriptionElementBase entity, int style) {
		AbstractDetailedDescriptionDetailElement detailedDescriptionElement = null;
		
		if(entity instanceof TextData){
			detailedDescriptionElement = new TextDataDetailElement(this, parentElement, (TextData) entity, style);
		}
		else if(entity instanceof CommonTaxonName){
			detailedDescriptionElement = new CommonNameDetailElement(this, parentElement, (CommonTaxonName) entity, style);
		}
		else if(entity instanceof IndividualsAssociation){
			detailedDescriptionElement = new IndividualsAssociationDetailElement(this, parentElement, (IndividualsAssociation) entity, style);
		}
		else if(entity instanceof TaxonInteraction){
			detailedDescriptionElement = new TaxonInteractionDetailElement(this, parentElement, (TaxonInteraction) entity, style);
		}
		else if(entity instanceof Distribution){
			detailedDescriptionElement = new DistributionDetailElement(this, parentElement, (Distribution) entity, style);
		}
		else{
			throw new IllegalStateException("There is no interface for the given descriptino element");
		}
		adapt(detailedDescriptionElement);
		parentElement.addElement(detailedDescriptionElement);
		return detailedDescriptionElement;
		
	}
}
