// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class CheckboxElement extends AbstractCdmFormElement implements SelectionListener, IEnableableFormElement {

	private static final Logger logger = Logger
			.getLogger(CheckboxElement.class);
	
	private Button checkbox;

	private Label label;
	
	/**
	 * 
	 * @param parent
	 * @param label
	 * @param initialState
	 * @param listener
	 * @param style
	 */
	protected CheckboxElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, boolean initialState,
			int style) {
		super(formFactory, parentElement);
		
		if(labelString != null){
			label = formFactory.createLabel(getLayoutComposite(), labelString);
			addControl(label);
		}
		
		checkbox = formFactory.createButton(getLayoutComposite(), null, SWT.CHECK | style);
		checkbox.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		
		checkbox.setSelection(initialState);
		
		checkbox.addSelectionListener(this);
		addControl(checkbox);
	}

	public void setSelection(boolean selected){
		checkbox.removeSelectionListener(this);
		checkbox.setSelection(selected);
		checkbox.addSelectionListener(this);
	}
	
	public boolean getSelection(){
		return checkbox.getSelection();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}

	public void setSelected(boolean selected) {
		checkbox.setBackground(getColor(selected));
	}

	public void setEnabled(boolean enabled) {
		checkbox.setEnabled(enabled);
	}

	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;
		
		Color color = EditorUtil.getColor(colorId);
		checkbox.setBackground(color);
	}
}
