// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.joda.time.Partial;

import eu.etaxonomy.cdm.model.common.TimePeriod;

/**
 * @author n.hoffmann
 * @created Mar 31, 2010
 * @version 1.0
 */
public class DateDetailSection extends AbstractFormSection {


	private static final Logger logger = Logger
			.getLogger(DateDetailSection.class);
	private PartialElement partialElement_start;
	private PartialElement partialElement_end;
	private TimePeriod timePeriod;
	
	protected DateDetailSection(CdmFormFactory formFactory, ICdmFormElement parentElement,
			int style) {
		super(formFactory, null, parentElement, style);
		this.setText("Date Details");
		
		partialElement_start = formFactory.createPartialElement(this, "Start", null, style);
		partialElement_end = formFactory.createPartialElement(this, "End", null, style);
		
		formFactory.addPropertyChangeListener(this);
	}

	public void setTimePeriod(TimePeriod timePeriod) {
		this.timePeriod = timePeriod;
		
		partialElement_start.setPartial(timePeriod.getStart());
		partialElement_end.setPartial(timePeriod.getEnd());
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if(event == null){
			return;
		}
		Object eventSource = event.getSource();
		if(eventSource == partialElement_start){
			timePeriod.setStart(partialElement_start.getPartial());
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
		}else if(eventSource == partialElement_end){
			Partial end = partialElement_end.getPartial();
			
			timePeriod.setEnd(end);
			
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, event));
		}
	}

	public TimePeriod getTimePeriod() {
		return timePeriod;
	}
	
	
}
