
package eu.etaxonomy.taxeditor.forms;

import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.forms.widgets.TableWrapData;

public class KeyValueViewerElement extends AbstractCdmFormElement {

	private static final Logger logger = Logger
			.getLogger(KeyValueViewerElement.class);
	
	private Map<? extends Object, ? extends Object> map;

	private TableViewer tableViewer;

	private Label label;

	public KeyValueViewerElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, String keyHeading, String valueHeading, Map<Object, Object> map) {
		super(formFactory, formElement);
		
		label = formFactory.createLabel(getLayoutComposite(), "Meta Data");
		label.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		addControl(label);
		
		tableViewer = new TableViewer(formElement.getLayoutComposite(), SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		addControl(tableViewer.getControl());
		
		createColumns(tableViewer, keyHeading, valueHeading);
		
		TableWrapData layoutData = CdmFormFactory.FILL_HORIZONTALLY();
		layoutData.heightHint = 100;
		
		tableViewer.getControl().setLayoutData(layoutData);
		
		tableViewer.setContentProvider(new KeyValueContentProvider());
		tableViewer.setLabelProvider(new KeyValueLabelProvider());
		
		if(map != null){
			setInput(map);
		}
		
	}
	
	
	public void setSelected(boolean selected) {
		// TODO Auto-generated method stub

	}
	
	// This will create the columns for the table
	private void createColumns(TableViewer viewer, String keyHeading, String valueHeading) {
		Table table = viewer.getTable();
		String[] titles = {keyHeading, valueHeading};
		int[] bounds = { 100, 100};

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}


	public void setInput(Map<? extends Object, ? extends Object> map) {
		this.map = map;
		tableViewer.setInput(map);
		tableViewer.refresh();
	}
	
	private class KeyValueContentProvider implements IStructuredContentProvider{

		public Object[] getElements(Object inputElement) {
			if(inputElement instanceof Map){
				return ((Map) inputElement).entrySet().toArray();
			}
			return null;
		}

		public void dispose() {}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
		
	}
	
	private class KeyValueLabelProvider extends LabelProvider implements ITableLabelProvider{

		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		public String getColumnText(Object element, int columnIndex) {
			if(element instanceof Entry){
			
				Entry entry = (Entry) element;
				
				switch(columnIndex){
				case 0:
					return entry.getKey().toString();
				case 1:
					return entry.getValue().toString();
				}
			}
			return element.toString();
		}
	}
}
