// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.joda.time.Partial;

import eu.etaxonomy.cdm.model.common.TimePeriod;

/**
 * @author n.hoffmann
 * @created Mar 31, 2010
 * @version 1.0
 */
public class PartialElement extends AbstractCdmFormElement implements ModifyListener {


	private static final Logger logger = Logger.getLogger(PartialElement.class);

	
	private Label label;
	private Composite container;
	private Text text_day;
	private Text text_month;
	private Text text_year;
	private Partial partial;

	public PartialElement(CdmFormFactory formFactory,
			ICdmFormElement formElement, String labelString, int style) {
		super(formFactory, formElement);
		
		label = formFactory.createLabel(getLayoutComposite(), labelString);
		addControl(label);
		
		container = formFactory.createComposite(getLayoutComposite());
		addControl(container);
		container.setLayout(CdmFormFactory.LAYOUT(3, false));
		container.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		container.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_GREEN));
		
		text_day = formFactory.createText(container, null, style);
		addControl(text_day);
		text_day.setTextLimit(2);
		
		TableWrapData layoutData = new TableWrapData();
		
		
		text_day.setLayoutData(layoutData);
		
		text_month = formFactory.createText(container, null, style);
		addControl(text_month);
		text_month.setTextLimit(2);
		
		text_month.setLayoutData(layoutData);
		
		text_year = formFactory.createText(container, null, style);
		addControl(text_year);		
		text_year.setTextLimit(4);
		
		layoutData = new TableWrapData();
		
		text_year.setLayoutData(layoutData);
		
		addListeners();
	}
	
	public void setPartial(Partial partial){
		this.partial = partial;
		
		removeListeners();
		
		if(partial != null){
			setInteger(text_day, TimePeriod.getPartialValue(partial, TimePeriod.DAY_TYPE));
			setInteger(text_month, TimePeriod.getPartialValue(partial, TimePeriod.MONTH_TYPE));
			setInteger(text_year, TimePeriod.getPartialValue(partial, TimePeriod.YEAR_TYPE));
		}
		addListeners();
	}
	
	private void addListeners() {
		text_day.addModifyListener(this);
		text_month.addModifyListener(this);
		text_year.addModifyListener(this);
	}

	private void removeListeners() {
		text_day.removeModifyListener(this);
		text_month.removeModifyListener(this);
		text_year.removeModifyListener(this);
	}

	public Partial getPartial(){
		return partial;
	}
	
	private void setInteger(Text text, Integer integer) {
		text.setText(integer == null ? "" : integer + "");
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ISelectable#setSelected(boolean)
	 */
	public void setSelected(boolean selected) {
		// TODO Auto-generated method stub

	}

	public void modifyText(ModifyEvent e) {
		Widget widget = e.widget;
		
		if(widget == text_day){
			partial = TimePeriod.setPartialField(partial, new Integer(text_day.getText()), TimePeriod.DAY_TYPE);
		}else if(widget == text_month){
			partial = TimePeriod.setPartialField(partial, new Integer(text_month.getText()), TimePeriod.MONTH_TYPE);
		}else if(widget == text_year){
			String yearString = text_year.getText();
			
			partial = TimePeriod.setPartialField(partial, new Integer(yearString), TimePeriod.YEAR_TYPE);
		}
		
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}
}
