// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;


/**
 * @author n.hoffmann
 * @created Feb 25, 2010
 * @version 1.0
 */
public class SelectionArbitrator implements SelectionListener, ISelectionChangedListener{
	private static final Logger logger = Logger
			.getLogger(SelectionArbitrator.class);
	private IEntityElement entityElement;

	private Set<ISelectionProvider> selectionProviders = new HashSet<ISelectionProvider>();
	
	public SelectionArbitrator(){}
	
	public SelectionArbitrator(IEntityElement selectableComposite){
		this.entityElement = selectableComposite;
	}
	
	public void widgetSelected(SelectionEvent e) {
		Widget widget = e.widget;
		
		if(widget instanceof Control){
			Control control = (Control) widget;
			if(isWidgetPartOfFormElement(control, entityElement)){
				if(entityElement.getEntity() != null){
					IStructuredSelection selection = new StructuredSelection(entityElement.getEntity());
					setSelection(selection);
				}
			}
		}	
	}
	
	private void setSelection(ISelection selection){
		for(ISelectionProvider selectionProvider : selectionProviders){
			selectionProvider.setSelection(selection);
		}
	}
	
	private boolean isWidgetPartOfFormElement(Control widget, ICdmFormElement element){
		// check if widget is the controls
		if(widget.equals(element.getLayoutComposite())){
			return true;
		}
		
		// check if widget is controlled by the given element
		for(Control control : element.getControls()){
			if(control.equals(widget)) return true;
		}
		
		// descend into child elements
		for(ICdmFormElement childElement : element.getElements()){
			isWidgetPartOfFormElement(widget, childElement);
		}
		
		return false;
	}
	
	public void addSelectionProvider(ISelectionProvider selectionProvider){
		selectionProviders.add(selectionProvider);
	}
	
	public void removeSelectionProvider(ISelectionProvider selectionProvider){
		selectionProviders.remove(selectionProvider);
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}

	public IEntityElement getEntityElement() {
		return entityElement;
	}

	public void setEntityComposite(IEntityElement entityElement) {
		this.entityElement = entityElement;
	}

	public void selectionChanged(SelectionChangedEvent event) {
//		if(!(event.getSource() instanceof CdmDetailsViewer)){
//			return;
//		}
		
		try{
		entityElement.setSelected(false);
		}catch(SWTException e){
			logger.error("Widgets of element are disposed: " + entityElement);
//			throw new RuntimeException("Widgets of element are disposed: " + entityElement);
		}
		ISelection selection = event.getSelection();		
		if(selection != null && selection instanceof StructuredSelection){
			Object selectedElement = ((StructuredSelection) selection).getFirstElement();
			if(entityElement.getEntity() != null && entityElement.getEntity().equals(selectedElement)){
				entityElement.setSelected(true);
			}
		}
	}
}
