/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;

/**
 * @author nho
 *
 */
public class TextActionElement extends AbstractCdmFormElement implements ModifyListener{

	private Label label;
	private Text text;
	private Button button;
	private Composite box;

	/**
	 * @param parent
	 * @param style
	 */
	protected TextActionElement(CdmFormFactory toolkit, ICdmFormElement parentElement, String labelString, String buttonLabel, String initialText, int style) {
		super(toolkit, parentElement);
		
		label = toolkit.createLabel(getLayoutComposite(), labelString, SWT.WRAP);
		addControl(label);
		
		box = formFactory.createComposite(getLayoutComposite());
		box.setLayout(CdmFormFactory.LAYOUT(2, false));
		box.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		addControl(box);
		
		text = toolkit.createText(box, "", style);
		addControl(text);
		text.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		
		setText(CdmUtils.Nz(initialText));
		
		button = toolkit.createButton(box, buttonLabel, SWT.PUSH);
		addControl(button);
	}

	/**
	 * @param text the text to set
	 */
	public void setText(String string) {
		text.setText(CdmUtils.Nz(string));
	}

	/**
	 * @return the text
	 */
	public String getText() {
		return text.getText();
	}

	public void addSelectionListener(SelectionListener listener){
		button.addSelectionListener(listener);
	}

	public void removeSelectionListener(SelectionListener listener){
		button.removeSelectionListener(listener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		propertyChange(null);
	}

	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
	}
	
	public void addModifyListener(ModifyListener listener){
		text.addModifyListener(listener);
	}
	
	public void removeModifyListener(ModifyListener listener){
		text.removeModifyListener(listener);
	}
}
