/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.TableWrapData;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 *
 */
public class TextWithLabelElement extends AbstractCdmFormElement implements ModifyListener, IEnableableFormElement{
	protected Text text;
	private Label label;
	
	public static final int MAX_HEIGHT = 0;
	public static final int SINGLE = -1;
	
	/**
	 * 
	 * @param formFactory
	 * @param parentElement
	 * @param labelString
	 * @param initialText
	 * @param textHeight
	 * @param style
	 */
	protected TextWithLabelElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, String initialText, Integer textHeight, int style) 
	{
		super(formFactory, parentElement);
		
		if(labelString != null && ! labelString.equals("")){
			label = formFactory.createLabel(getLayoutComposite(), CdmUtils.Nz(labelString), SWT.NULL);
			addControl(label);
			label.setLayoutData(CdmFormFactory.LEFT());
		}
		
		int scrollStyle = textHeight == null ? SWT.NULL : (SWT.V_SCROLL | SWT.MULTI);
		text = formFactory.createText(getLayoutComposite(), "", SWT.BORDER | SWT.WRAP | scrollStyle);
		addControl(text);
				
		if(textHeight == null){
			// Don't accept carriage returns as input when in single line mode
			text.addKeyListener( new KeyAdapter(){
				@Override
				public void keyPressed(KeyEvent e) {
				if(e.character == SWT.CR)
					e.doit = false;
				}
			});
		}
		
		TableWrapData layoutData = CdmFormFactory.FILL();
		if (textHeight != null && textHeight > 0){
			((TableWrapData) layoutData).heightHint = textHeight;
		}	
		
		text.setLayoutData(layoutData);
		
		text.addModifyListener(this);
		
		setText(initialText);
	}
	
	/**
	 * Get the text of this composites text composite
	 * 
	 * @return
	 */
	public String getText() {
		return text.getText();
	}

	/**
	 * Set the text of this composites text composite
	 * 
	 * @param string
	 */
	public void setText(String string) {
		Listener[] listeners = text.getListeners(SWT.Modify);
		
		for(Listener listener :  listeners){
			text.removeListener(SWT.Modify, listener);
		}
		
		text.setText(CdmUtils.Nz(string));
		
		for(Listener listener :  listeners){
			text.addListener(SWT.Modify, listener);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
	}

	public void setEnabled(boolean enabled) {
		text.setEnabled(enabled);
		String symbolicName = enabled ? Resources.COLOR_FONT_DEFAULT : Resources.COLOR_TEXT_DISABLED;
		text.setForeground(EditorUtil.getColor(symbolicName));
	}

	public void setIrrelevant(boolean irrelevant) {
 		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;
		
		Color color = EditorUtil.getColor(colorId);
		text.setBackground(color);
	}
}
