// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.common.TimePeriod;

/**
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 */
public class TimePeriodElement extends AbstractCdmFormElement{


	private static final Logger logger = Logger
			.getLogger(TimePeriodElement.class);
	private Label label;
	
	private TextWithLabelElement text_freeText;
	private TimePeriod timePeriod;
	private DateDetailSection section_dateDetails;
	
	/**
	 * @param parent
	 * @param listener
	 * @param style
	 */
	public TimePeriodElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, TimePeriod timePeriod, final int style) {
		super(formFactory, parentElement);
		
		formFactory.addPropertyChangeListener(this);
		
		text_freeText = formFactory.createTextWithLabelElement(this, labelString, null, style);
		addControl(formFactory.createLabel(getLayoutComposite(), ""));
		
		section_dateDetails = formFactory.createDateDetailSection(this, Section.TWISTIE);
		addControl(section_dateDetails);
		
		setEntity(timePeriod);
		
	}

	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
	}

	public void setEntity(TimePeriod timePeriod) {

		this.timePeriod = timePeriod;
		if(timePeriod != null){
			text_freeText.setText(timePeriod.toString());
			section_dateDetails.setTimePeriod(timePeriod);
		}
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if(event == null){
			return;
		}
		Object eventSource = event.getSource();
		if(eventSource == text_freeText){
			String freeText = text_freeText.getText();
			
			timePeriod = TimePeriod.parseString(freeText);
		}else if(eventSource == section_dateDetails){
			timePeriod = section_dateDetails.getTimePeriod();
		}
	}
	
}
