// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * When the button is pressed, this textfield may be edited.
 * 
 * @author n.hoffmann
 * @created Nov 18, 2009
 * @version 1.0
 */
public class ToggleableTextElement extends AbstractCdmFormElement implements SelectionListener, ModifyListener, IEnableableFormElement {


	private static final Logger logger = Logger
			.getLogger(ToggleableTextElement.class);
	private Text text_cache;
	private Button button_toggle;
	
	private boolean state;
	private Set<SelectionListener> selectionListener = new HashSet<SelectionListener>();
	private Label label;
	private Composite container;
	
	/**
	 * @param parent
	 * @param listener
	 * @param style
	 */
	protected ToggleableTextElement(CdmFormFactory formFactory, ICdmFormElement parentElement,
			String labelString, String initialText, boolean initialState, int style) {
		super(formFactory, parentElement);
		
		label = formFactory.createLabel(getLayoutComposite(), labelString, style);
		addControl(label);
		
		container = formFactory.createComposite(getLayoutComposite(), SWT.WRAP);		
		container.setLayout(CdmFormFactory.LAYOUT(2, false));
		container.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		addControl(container);
		
		text_cache = formFactory.createText(container, initialText, SWT.WRAP | SWT.MULTI);
		addControl(text_cache);
		text_cache.addModifyListener(this);
		text_cache.setLayoutData(CdmFormFactory.FILL());

		// Don't accept carriage returns as input
		text_cache.addKeyListener( new KeyAdapter(){
			@Override
			public void keyPressed(KeyEvent e) {
			if(e.character == SWT.CR)
				e.doit = false;
			}
		});
		
		button_toggle = formFactory.createButton(container, "Edit", SWT.TOGGLE);
		addControl(button_toggle);
		button_toggle.addSelectionListener(this);
		
		setState(initialState);
	}

	public void setText(String text){
		if(text != null){
			// store current caret position
			int caretPosition = text_cache.getCaretPosition();
			
			text_cache.removeModifyListener(this);
			text_cache.setText(text);
			text_cache.addModifyListener(this);

			// restore caret position
			text_cache.setSelection(caretPosition);
		}
	}

	public String getText(){
		return text_cache.getText();
	}
	
	/**
	 * @param initialState
	 */
	public void setState(boolean state) {
		this.state = state;
		setEnabled(state);
	}
	
	public void setEnabled(boolean enabled) {
		text_cache.setEnabled(enabled);
		String symbolicName = enabled ? Resources.COLOR_FONT_DEFAULT : Resources.COLOR_TEXT_DISABLED;
		text_cache.setForeground(EditorUtil.getColor(symbolicName));
		button_toggle.setSelection(enabled);
	}

	public boolean getState(){
		return state;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		setState(button_toggle.getSelection());
		for(SelectionListener listener : selectionListener){
			listener.widgetSelected(e);
		}
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}
	
	public void addSelectionListener(SelectionListener listener){
		selectionListener.add(listener);
	}
	
	public void removeSelectionListener(SelectionListener listener){
		selectionListener.remove(listener);
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}

	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
		container.setBackground(getColor(selected));
	}

	public void modifyText(ModifyEvent e) {
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;
		
		Color color = EditorUtil.getColor(colorId);
		text_cache.setBackground(color);
	}
}
