/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.selection;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.forms.IEntityElement;
import eu.etaxonomy.taxeditor.forms.ISelectableElement;
import eu.etaxonomy.taxeditor.forms.SelectionArbitrator;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * 
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractSelectionElement<T extends IdentifiableEntity> extends AbstractCdmFormElement implements SelectionListener, IEnableableFormElement, ISelectableElement, IEntityElement<T>, IConversationEnabled{

	private static final Logger logger = Logger
				.getLogger(AbstractSelectionElement.class);
	
	protected T entity;
	
	protected Label label;
	protected Label text;
	protected Button button_selection;
	
	private SelectionArbitrator selectionArbitrator;

	protected Button button_edit;

	private String labelString;

	private Composite selectableComposite;

	private Button button_remove;

	private boolean isEditable;

	private boolean isDeletable;

	private EditFromSelectionWizard wizard;
	
	private ConversationHolder conversation;

	/**
	 * 
	 * @param formFactory
	 * @param conversation TODO
	 * @param parentElement
	 * @param labelString
	 * @param entity
	 * @param isEditable
	 * @param isSelectable
	 * @param isDeletable
	 * @param style
	 */
	public AbstractSelectionElement(
			CdmFormFactory formFactory, 
			ConversationHolder conversation, 
			ICdmFormElement parentElement, 
			String labelString, 
			T entity, 
			boolean isEditable, 
			boolean isSelectable, 
			boolean isDeletable, int style) {
		super(formFactory, parentElement);
		
		this.isEditable = isEditable;
		this.isDeletable = isDeletable;
		
		this.labelString = labelString;
		
		this.conversation = conversation;
		
		if(isSelectable && formFactory.getSelectionProvider() != null){
			selectionArbitrator = formFactory.createSelectionArbitrator(this);
		}
		
		createControls(getLayoutComposite(), SWT.NULL);		
		
		setEntity(entity);
	}
	
	private void createControls(Composite parent, int style){
		
		label = formFactory.createLabel(getLayoutComposite(), labelString, SWT.NULL);
		
		addControl(label);
		
		selectableComposite = formFactory.createComposite(getLayoutComposite());
		
		int columns = 2;
		if(isEditable) columns += 1;
		if(isDeletable) columns += 1;		
		
		selectableComposite.setLayout(CdmFormFactory.LAYOUT(columns, false));
		selectableComposite.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		
		addControl(selectableComposite);
		
		text = formFactory.createLabel(selectableComposite, null, SWT.WRAP);
		addControl(text);
		
		text.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		text.setBackground(EditorUtil.getColor(Resources.COLOR_TEXT_DISABLED_BACKGROUND));
		
		button_selection = formFactory.createButton(selectableComposite, null, SWT.PUSH);
		button_selection.setImage(ImageResources.getImage(ImageResources.BROWSE_ICON));
		button_selection.setToolTipText("Browse existing entities");
		addControl(button_selection);
		button_selection.addSelectionListener(this);
		
		if(isEditable){
	 		button_edit = formFactory.createButton(selectableComposite, null, SWT.PUSH);
	 		button_edit.setImage(ImageResources.getImage(ImageResources.EDIT_ICON));
	 		button_edit.setToolTipText("Edit entity");
			addControl(button_edit);
		}
		
		if(isDeletable){
			button_remove = formFactory.createButton(selectableComposite, null, SWT.PUSH);
			button_remove.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));
			button_remove.setToolTipText("Remove entity");
			addControl(button_remove);
			button_remove.addSelectionListener(new DeleteListener(this));
		}
	}
	
	/**
	 * Return the selected object
	 * 
	 * @return
	 */
	public T getSelection(){
		return entity;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IEnableableFormElement#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		button_selection.setEnabled(enabled);
		if(isEditable){
			button_edit.setEnabled(enabled && entity != null);
		}
	}
	
	/**
	 * 
	 * @param selection
	 */
	protected void setSelectionInternal(T selection){
		if(selection != null && !selection.equals(this.entity)){
			setEntity(selection);
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
		}
	}
	
	/**
	 * 
	 * @param selection
	 */
	public void setEntity(T selection){
		this.entity = selection;	
		if(isEditable){
//			clearEditButtonSelectionListener();
//			addEditButtonSelectionListener(new BulkEditorOpeningSelectionListener(selection));
			createEditWizard();
		}
		updateElement();
	}
	
	private void createEditWizard(){
		if(entity != null){
			wizard = new EditFromSelectionWizard(this);
			button_edit.addSelectionListener(wizard);
		}
	}

	/**
	 * Updates this elements view
	 */
	protected void updateElement(){
		text.setText(CdmUtils.Nz(getTitle())); // title can be null
		if(isEditable){
			button_edit.setEnabled(entity != null);
		}
	}
	
	protected void updateFromWizard(){
		updateElement();
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}
	
	protected String getTitle(){
		return (entity != null) ? entity.getTitleCache() : "";
	}
	
	/**
	 * 
	 * @param selected
	 */
	public void setSelected(boolean selected) {
		Color color = getColor(selected);
		getLayoutComposite().setBackground(color);
		selectableComposite.setBackground(color);
		label.setBackground(color);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IEntityElement#getEntity()
	 */
	public T getEntity() {
		return entity;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.ISelectableElement#getSelectionArbitrator()
	 */
	public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}
	
	/**
	 * Convenient access to current shell
	 * @return
	 */
	protected Shell getShell(){
		return getLayoutComposite().getShell();
	}

	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_TEXT_DISABLED_BACKGROUND;
		
		Color color = EditorUtil.getColor(colorId);
		text.setBackground(color);
	}
	
	private class DeleteListener extends SelectionAdapter{
		
		private AbstractSelectionElement selectionElement;

		public DeleteListener(AbstractSelectionElement selectionElement){
			this.selectionElement = selectionElement;
		}
		
		@Override
		public void widgetSelected(SelectionEvent e) {
			setEntity(null);
			firePropertyChangeEvent(new CdmPropertyChangeEvent(selectionElement, null));
		}
	}
	
	// not used
	public void widgetDefaultSelected(SelectionEvent e) {}
	
	public ConversationHolder getConversationHolder() {
		return conversation;
	}
	
	public void update(CdmDataChangeMap changeEvents) {}
}
