// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.selection;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.forms.AbstractCdmEntityWizardPage;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.section.agent.PersonWizardPage;
import eu.etaxonomy.taxeditor.section.agent.TeamWizardPage;
import eu.etaxonomy.taxeditor.section.name.NonViralNameWizardPage;
import eu.etaxonomy.taxeditor.section.occurrence.DerivedUnitFacadeWizardPage;
import eu.etaxonomy.taxeditor.section.reference.ReferenceBaseWizardPage;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public class EditFromSelectionWizard extends Wizard implements SelectionListener {
	private static final Logger logger = Logger
			.getLogger(EditFromSelectionWizard.class);
	
	private AbstractSelectionElement selectionElement;


	AbstractCdmEntityWizardPage page;
	
	public EditFromSelectionWizard(AbstractSelectionElement selectionElement){
		this.selectionElement = selectionElement;		
		this.setWindowTitle("Edit Entity");
	}
	
	@Override
	public void addPages() {
			
		CdmFormFactory formFactory = selectionElement.getFormFactory();
		IdentifiableEntity entity = (IdentifiableEntity) HibernateProxyHelper.deproxy(selectionElement.getEntity());
		
		if(entity instanceof ReferenceBase){
			page = new ReferenceBaseWizardPage(formFactory, selectionElement.getConversationHolder(), (ReferenceBase) entity);
		}
		else if(entity instanceof Team){
			page = new TeamWizardPage(formFactory, selectionElement.getConversationHolder(), (Team) entity);
		}
		else if(entity instanceof Person){
			page = new PersonWizardPage(formFactory, selectionElement.getConversationHolder(), (Person) entity);
		}
		else if(entity instanceof NonViralName){
			page = new NonViralNameWizardPage(formFactory, selectionElement.getConversationHolder(), (NonViralName) entity);
		}
		else if (entity instanceof DerivedUnitBase){
			page = new DerivedUnitFacadeWizardPage(formFactory, selectionElement.getConversationHolder(), (DerivedUnitBase) entity);
		}
		else{
			logger.warn("No detail element for current selection");
			throw new RuntimeException("Could not open wizard for selected entity");
		}
		
		addPage(page);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		return true;
	}

	public void widgetSelected(SelectionEvent e) {
		WizardDialog dialog = new WizardDialog(selectionElement.getShell(), this);
		if(dialog.open() == IStatus.OK){
			selectionElement.updateFromWizard();
		}
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}
}
