// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.term;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 * @created Mar 16, 2010
 * @version 1.0
 */
public abstract class AbstractEnumComboElement<T extends Enum> extends AbstractCdmFormElement implements SelectionListener, IEnableableFormElement{


	private static final Logger logger = Logger
			.getLogger(AbstractEnumComboElement.class);
	
	private static final int VISIBLE_ITEMS = 10;
	
	protected T selection;
	
	protected List<T> elementTypeList = new ArrayList<T>();
	
	private Label label;
	protected Combo combo;

	public AbstractEnumComboElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, String labelString) {
		super(formFactory, parentElement);
		label = formFactory.createLabel(getLayoutComposite(), labelString);
		addControl(label);
		
		// create combo
	    combo = new Combo(getLayoutComposite(), SWT.BORDER | SWT.READ_ONLY);
		addControl(combo);
	    combo.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
	    combo.setVisibleItemCount(VISIBLE_ITEMS);
	    
	    populateTypes();
	    
	    combo.addSelectionListener(this);
	}
	

	protected abstract void populateTypes();
	
	
	/**
	 * @param selection the selection to set
	 */
	public void setSelection(T selection) {
		this.selection = selection;
		combo.select(elementTypeList.indexOf(selection));
	}
	
	public void addSelectionListener(SelectionListener listener){
		combo.addSelectionListener(listener);
	}
	
	public void removeSelectionListener(SelectionListener listener){
		combo.removeSelectionListener(listener);
	}

	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
	}
	
	/**
	 * @return the selection
	 */
	public T getSelection() {
		return selection;
	}

	public void setEnabled(boolean enabled) {
		combo.setEnabled(enabled);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		selection = elementTypeList.get(combo.getSelectionIndex());
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}
	
	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;
		
		Color color = EditorUtil.getColor(colorId);
		combo.setBackground(color);
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}
}
