/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.term;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * 
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractTermComboElement<T extends DefinedTermBase> extends AbstractCdmFormElement implements SelectionListener, DisposeListener, IEnableableFormElement{

	private static final Logger logger = Logger.getLogger(AbstractTermComboElement.class);
	
	T selection;
	
	private static String EMPTY_ELEMENT_LABEL = "";
	T emptyElement;
	
	private Label label;
	private Combo combo;

	private ArrayList<T> terms;

	private Comparator<T> termComparator;

	private Composite msgComposite;
	
	/**
	 * @param parentElement
	 * @param style
	 */
	public AbstractTermComboElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, T selection, int visibleItems, int style) {
		super(formFactory, parentElement);
		
		if(labelString != null){
			label = formFactory.createLabel(getLayoutComposite(), labelString);
			addControl(label);
		}
		
        // create combo
        combo = new Combo(getLayoutComposite(), SWT.BORDER | SWT.READ_ONLY);
		addControl(combo);
        combo.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
        combo.setVisibleItemCount(visibleItems);
        
        populateTerms();
        
        combo.addSelectionListener(this);
        combo.addDisposeListener(this);
        PreferencesUtil.getPreferenceStore().addPropertyChangeListener(this);
        
        if(selection != null){
        	setSelection(selection);
        }
	}

	public T getSelection(){
		return selection;
	}
	
	public void setSelection(T selection){
		Listener[] listeners = combo.getListeners(SWT.Selection);
		
		for (Listener listener : listeners){
			combo.removeListener(SWT.Selection, listener);
		}
		
		int index = terms.indexOf(selection);
		if(index == -1){
			createTermNotInPreferredTerms(selection);
		}else{
			combo.select(index);
		}
		
		for (Listener listener : listeners){
			combo.addListener(SWT.Selection, listener);
		}
	}
	
	private void populateTerms(){
		List<T> preferredTerms = preferredTerms();
		
		combo.removeAll();
		
		terms = new ArrayList<T>();
		
		int i = 1;
		int index = 0;
		
		// Add an empty element for when nothing was selected already
		combo.add(EMPTY_ELEMENT_LABEL);
		terms.add(emptyElement);
		
		if(termComparator != null)
			Collections.sort(preferredTerms, termComparator);
		for (T term : preferredTerms) {
			String label = getLabel(term); 
			if (label == null) {
				label = "No label for " + term;
				logger.warn(label);
			}
			combo.add(label);
			terms.add(term);

			i++;
			if (selection != null) {
				if (selection.equals(term)) {
			 		index = i;
				}
			} else {
				// TODO isn't it better to put the last language at the top of the menu, not automatically select?
//					if (lastLanguage != null && lastLanguage.equals(language)) {
//						index = i;  
//					}
			}
		}
		
		// Check whether language is in menu
		if (msgComposite != null) {
			msgComposite.dispose();
			msgComposite = null;
		}
		if (selection != null && index == 0) {
			createTermNotInPreferredTerms(selection);
		}
		
		combo.select(index);	
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IEnableableFormElement#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		combo.setEnabled(enabled);
	}
	
	/**
	 * @return
	 */
	protected abstract List<T> preferredTerms();
	
	/**
	 * May be overridden by derived classes if the desired label string does not
	 * reside in term.getLabel();
	 * 
	 * @param term
	 * @return
	 */
	protected String getLabel(T term){
		return term.getLabel(CdmStore.getDefaultLanguage());
	}

	private void createTermNotInPreferredTerms(T term) {
		
//		PreferencesUtil.addTermToPreferredTerms(term);
//	
//		msgComposite = formFactory.createComposite(getLayoutComposite());
//		msgComposite.setLayout(LAYOUT());
//		msgComposite.setLayoutData(FILL_GRID_HORIZONTALLY(2, 1));
//		
//		String langText = term.getDescription() == null ? term.getLabel() : term.getDescription();
//		langText += " is used here, but was not in your list of preferred terms.";
//		Label label = formFactory.createLabel(msgComposite, langText, SWT.WRAP);
//
//		
//		msgComposite.moveBelow(combo);
//		
//		getLayoutComposite().layout();
		
//		setSelection(term);
	}
	
	public void addSelectionListener(SelectionListener listener){
		combo.addSelectionListener(listener);
	}
	
	public void removeSelectionListener(SelectionListener listener){
		combo.removeSelectionListener(listener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		selection = terms.get(combo.getSelectionIndex());
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		if (event != null && PreferencesUtil.PREFERRED_TERMS_CHANGE.equals(event.getProperty())) {
			populateTerms();
		}
	}
	
	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
	}
	
	public void widgetDisposed(DisposeEvent e) {
		PreferencesUtil.getPreferenceStore().removePropertyChangeListener(this);
	}
	
	// not used
	public void widgetDefaultSelected(SelectionEvent e) {}
	
	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;
		
		Color color = EditorUtil.getColor(colorId);
		combo.setBackground(color);
	}
}
