// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.forms.IEntityElement;
import eu.etaxonomy.taxeditor.forms.ISelectable;

/**
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDetailElement<T extends AnnotatableEntity> extends
		AbstractCdmFormElement implements IEntityElement<T>, ISelectable, IEnableableFormElement, IConversationEnabled{

	private static final Logger logger = Logger
	.getLogger(AbstractCdmDetailElement.class);
	
	private T entity;
		
	private boolean enabled;

	private boolean irrelevant;
		
	public AbstractCdmDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, (ICdmFormElement) formElement);
		// register as property change listener
		formFactory.addPropertyChangeListener(this);
	}
	
	/**
	 * Sets the entity and updates controls
	 * 
	 * @param entity
	 */
	public void setEntity(T entity){
		this.entity = entity;
		updateContent();
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IEntityElement#getEntity()
	 */
	public T getEntity() {
		return entity;
	}
	
	/**
	 * Updates all widgets to display the latest data
	 */
	protected void updateContent(){
		removeElements();	
		createControls(this, entity, SWT.WRAP);
		getLayoutComposite().layout();
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ISelectableComposite#setSelected(boolean)
	 */
	public void setSelected(boolean selected) {
//		for(ICdmFormElement element : getElements()){
//			if(element instanceof ISelectable){
//				((ISelectable) element).setSelected(selected);
//			}
//		}
//		getLayoutComposite().setBackground(getColor(selected));
		Composite section = getLayoutComposite().getParent();
		section.setBackground(getColor(selected));
	}
	
	/**
	 * Sets all field to the given enablement state except the objects in collection except
	 * 
	 * @see eu.etaxonomy.taxeditor.forms.IEnableableFormElement#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled, Collection<Object> except) {
		for(ICdmFormElement element : getElements()){
			if(element instanceof IEnableableFormElement){
				IEnableableFormElement enableableElement = (IEnableableFormElement) element;
				if(except != null && except.contains(enableableElement)){
//					enableableElement.setEnabled(enabled);
				}else{
					enableableElement.setEnabled(enabled);
				}
			}
		}
	}
	
	/**
	 * 
	 * @param irrelevant
	 * @param except
	 */
	public void setIrrelevant(boolean irrelevant, Collection<Object> except){
		//logger.warn("Setting " + this.getClass().getSimpleName() + " to irrelevant state: " + irrelevant);
		for(ICdmFormElement element : getElements()){
			if(element instanceof IEnableableFormElement){
				IEnableableFormElement relevantElement = (IEnableableFormElement) element;
				if(except != null && except.contains(relevantElement)){
					// do nothing
					//logger.warn("not setting elements irrelevance: " + relevantElement.getClass().getSimpleName() + " in :" + this.getClass().getSimpleName());
				}else{
					relevantElement.setIrrelevant(irrelevant);
				}
			}
		}
	}
	
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
		setEnabled(enabled, null);
	}

	public void setIrrelevant(boolean irrelevant) {
		this.irrelevant = irrelevant;
		setIrrelevant(irrelevant, null);
	}
	
	/**
	 * Create all content elements
	 * 
	 * @param formElement
	 * @param entity
	 * @param style
	 */
	protected abstract void createControls(ICdmFormElement formElement, T entity, int style);
	
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if(event == null){
			return;
		}
		
		Object eventSource = event.getSource();
		
		if(eventSource instanceof ICdmFormElement){
			ICdmFormElement eventElement = (ICdmFormElement) eventSource;
			ICdmFormElement eventElementContainer = eventElement.getParentElement();
			if (eventElementContainer == this){
				handleEvent(eventSource);
				if(getParentElement() instanceof AbstractCdmDetailSection)
					((AbstractCdmDetailSection)getParentElement()).updateTitle();
			}
		}
	}

	public abstract void handleEvent(Object eventSource);

	public boolean isIrrelevant() {
		return irrelevant;
	}
	
	public ConversationHolder getConversationHolder() {
		if(getParentElement() instanceof IConversationEnabled){
			return ((IConversationEnabled) getParentElement()).getConversationHolder();
		}
		throw new IllegalArgumentException("Parent element should be IConversationEnabled");
	}
	
	public void update(CdmDataChangeMap changeEvents) {}
}
