// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.forms.ISelectableElement;
import eu.etaxonomy.taxeditor.forms.SelectionArbitrator;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;

/**
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDetailSection<T extends AnnotatableEntity> extends AbstractFormSection<T> implements IEnableableFormElement, IExpansionListener{


	private static final Logger logger = Logger
			.getLogger(AbstractCdmDetailSection.class);
	
	protected AbstractCdmDetailElement<T> detailElement;
	
	protected SelectionArbitrator selectionArbitrator;
	
	public AbstractCdmDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
		super(formFactory, conversation, parentElement, selectionProvider, Section.CLIENT_INDENT | style);
		this.setText(getHeading());
		
		this.addExpansionListener(this);
		
		createControls(this, SWT.NULL);
	}
	
	
	protected void createControls(AbstractCdmDetailSection<T> formElement, int style) {
		TableWrapLayout layout = (TableWrapLayout) getLayoutComposite().getLayout();
		layout.topMargin = 10;
		layout.numColumns = 2;
		
		getLayoutComposite().setLayout(layout);
		detailElement = formFactory.createCdmDetailElement(getDetailType(), formElement, style);
	}
	
	protected abstract DetailType getDetailType();

	/**
	 * @return the heading for this section
	 */
	public abstract String getHeading();
	
	@Override
	public void dispose() {
		if(detailElement instanceof ISelectableElement){
			formFactory.destroySelectionArbitrator(((ISelectableElement)detailElement).getSelectionArbitrator());
		}
		super.dispose();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.AbstractEditorFormSection#setBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setBackground(Color color) {
		if(detailElement != null){
//			detailComposite.setBackground(color);
		}
		super.setBackground(color);
	}
	
	public void setEntity(T entity) {
		if(detailElement != null){
			detailElement.setEntity(entity);
		}
		super.setEntity(entity);
		setSectionTitle();
		layout();
	}
	
	protected void setSectionTitle(){
		String title = "";
		if(getEntity() != null && (getEntity() instanceof IdentifiableEntity)){
			 title = ": " + ((IdentifiableEntity) getEntity()).getTitleCache();
		}
		this.setText(getHeading() + title);
	}
	
	public void updateTitle(){
		setSectionTitle();
		layout();
	}
	
	public void setIrrelevant(boolean irrelevant) {
		if(detailElement != null){
			detailElement.setIrrelevant(irrelevant);
		}
	}
	
	public void expansionStateChanging(ExpansionEvent e) {
		logger.warn("Expansion State Changing");
	}
	
	public void expansionStateChanged(ExpansionEvent e) {
		logger.warn("Expansion State Changed");
	}
}
