// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.description;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.operations.ChangeDescriptionElementType;
import eu.etaxonomy.taxeditor.operations.ChangeDescriptionElementType.DestinationType;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 5, 2010
 * @version 1.0
 */
public class DescriptionElementDetailSection extends
		AbstractCdmDetailSection<DescriptionElementBase> {
	
	private static final Logger logger = Logger
			.getLogger(DescriptionElementDetailSection.class);

	public DescriptionElementDetailSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
		
		setTextClient(createToolbar());
	}
	
	
	private Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
		
		final String destinationString = (getEntity() instanceof TextData) ? "the specific element" : "free text";
		
		final String label = "Change to " + destinationString; 
		
		Action addAction = new Action("Change Type", Action.AS_PUSH_BUTTON) {
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				boolean confirmed = EditorUtil.confirmDialog("Confirmation", "Do you really want to change to " + destinationString + "? Current data will be lost.");
				if (confirmed) {
					ChangeDescriptionElementType operation = new ChangeDescriptionElementType(label, getEntity(), EditorUtil.getDetailsView(), EditorUtil.getActiveMultiPageTaxonEditor().getUndoContext());
					EditorUtil.executeOperation(operation);
				}
			}
		};
		// TODO enable this once a proper icon has been found 
//		addAction.setImageDescriptor(new ImageDescriptor() {
//			
//			@Override
//			public ImageData getImageData() {
//				return ImageResources.getImage(ImageResources.ADD_ICON).getImageData();
//			}
//		});
		addAction.setToolTipText(label);
		
		toolBarManager.add(addAction);
		
		ICdmFormElement parentElement = this.getParentElement();
		
		
		return toolBarManager.createControl(this);
	}
	
	@Override
	public String getHeading() {
		return "Description Element";
	}

	@Override
	protected DetailType getDetailType() {
		return DetailType.DESCRIPTIONELEMENT;
	}
	
	@Override
	protected void setSectionTitle() {
		this.setText(getHeading() + ": " + getEntity().getFeature().getLabel(CdmStore.getDefaultLanguage()));
	}
}
