// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.description;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEntityElement;
import eu.etaxonomy.taxeditor.forms.SelectionArbitrator;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.selection.NameSelectionElement;
import eu.etaxonomy.taxeditor.section.supplemental.AbstractOriginalSourceElement;

/**
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 * @param <T>
 */
public class DescriptionElementSourceElement extends AbstractOriginalSourceElement<DescriptionElementSource> implements IEntityElement<DescriptionElementSource>{
	
	private static final Logger logger = Logger
	.getLogger(DescriptionElementSourceElement.class);

	
	private SelectionArbitrator selectionArbitrator;

	private NameSelectionElement selection_name;

	
	/**
	 * @param formElement
	 * @param element
	 * @param removeListener
	 * @param propertyChangelistener
	 * @param style
	 */
	public DescriptionElementSourceElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement,
			DescriptionElementSource element, SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, style);
		selectionArbitrator = formFactory.createSelectionArbitrator(this);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.OriginalSourceComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		super.createControls(formElement, style);		
		selection_name = (NameSelectionElement) formFactory.createSelectionElement(SelectionType.NAME, getConversationHolder(), formElement, "Name in Source", null, SWT.NULL);
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.AbstractOriginalSourceElement#setElement(eu.etaxonomy.cdm.model.common.OriginalSourceBase)
	 */
	@Override
	public void setEntity(DescriptionElementSource entity) {
		super.setEntity(entity);
		text_idInSource.setText(entity.getIdInSource());
		text_idNamespace.setText(entity.getIdNamespace());

		selection_reference.setEntity(entity.getCitation());
		text_referenceDetail.setText(entity.getCitationMicroReference());
		text_originaleNameString.setText(entity.getOriginalNameString());
		selection_name.setEntity(entity.getNameUsedInSource());
	}
	

	public void setSelected(boolean selected) {
		// TODO Auto-generated method stub
		
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_idInSource){
			getEntity().setIdInSource(text_idInSource.getText());
		}
		else if(eventSource == text_idNamespace){
			getEntity().setIdNamespace(text_idNamespace.getText());
		}
		else if(eventSource == selection_reference){
			getEntity().setCitation(selection_reference.getSelection());
		}
		else if(eventSource == text_referenceDetail){
			getEntity().setCitationMicroReference(text_referenceDetail.getText());
		}
		else if(eventSource == text_originaleNameString){
			getEntity().setOriginalNameString(text_originaleNameString.getText());
		}
		else if(eventSource == selection_name){
			getEntity().setNameUsedInSource(selection_name.getSelection());
		}
	}
	
}
