// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Arrays;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.forms.ToggleableTextElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.selection.AbstractSelectionElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;

/**
 * @author n.hoffmann
 * @created Mar 4, 2010
 * @version 1.0
 */
public class AuthorshipDetailElement extends AbstractCdmDetailElement<NonViralName> implements IEnableableFormElement {

	private ToggleableTextElement toggleable_cache;

	private AbstractSelectionElement selection_exBasionymAuthorTeam;
	private AbstractSelectionElement selection_basionymAuthorTeam;
	private AbstractSelectionElement selection_exCombinationAuthorTeam;
	private AbstractSelectionElement selection_combinationAuthorTeam;
	
	public AuthorshipDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}
	
	@Override
	protected void createControls(ICdmFormElement formElement, NonViralName entity, int style) {
		toggleable_cache = formFactory.createToggleableTextField(this, "Authorship Cache", entity.getAuthorshipCache(), entity.isProtectedAuthorshipCache(), style);
		
		selection_combinationAuthorTeam = formFactory.createSelectionElement(SelectionType.AUTHOR_TEAM, getConversationHolder(), formElement, "Combination Author Team", getAuthor(entity.getCombinationAuthorTeam()), style);
		addElement(selection_combinationAuthorTeam);
		selection_exCombinationAuthorTeam = formFactory.createSelectionElement(SelectionType.AUTHOR_TEAM, getConversationHolder(), formElement, "Ex Combination Author Team",getAuthor(entity.getExCombinationAuthorTeam()), style);
		addElement(selection_exCombinationAuthorTeam);
		selection_basionymAuthorTeam = formFactory.createSelectionElement(SelectionType.AUTHOR_TEAM, getConversationHolder(), formElement, "Basionym Author Team", getAuthor(entity.getBasionymAuthorTeam()), style);
		addElement(selection_basionymAuthorTeam);
		selection_exBasionymAuthorTeam = formFactory.createSelectionElement(SelectionType.AUTHOR_TEAM, getConversationHolder(), formElement, "Ex Basionym Author Team", getAuthor(entity.getExBasionymAuthorTeam()), style);
		addElement(selection_exBasionymAuthorTeam);
	}

	@Override
	public void updateContent() {
		if(getEntity() == null){
			setEntity(NonViralName.NewInstance(null));
		}
		NonViralName nonViralName = getEntity();
		
		super.updateContent();
		toggleable_cache.setEnabled(getEntity().isProtectedAuthorshipCache());
		
		if(this.isIrrelevant()){
			setIrrelevant(isIrrelevant());
		}else{
			setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
		}
	}
	
	
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == toggleable_cache){
			getEntity().setAuthorshipCache(toggleable_cache.getText(), toggleable_cache.getState());
			if(! isIrrelevant()) setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
		}
		else if(eventSource == selection_combinationAuthorTeam){
			getEntity().setCombinationAuthorTeam((INomenclaturalAuthor) selection_combinationAuthorTeam.getSelection());
		}
		else if(eventSource == selection_exCombinationAuthorTeam){
			getEntity().setExCombinationAuthorTeam((INomenclaturalAuthor) selection_exCombinationAuthorTeam.getSelection());
		}
		else if(eventSource == selection_basionymAuthorTeam){
			getEntity().setBasionymAuthorTeam((INomenclaturalAuthor) selection_basionymAuthorTeam.getSelection());
		}
		else if(eventSource == selection_exBasionymAuthorTeam){
			getEntity().setExBasionymAuthorTeam((INomenclaturalAuthor) selection_exBasionymAuthorTeam.getSelection());
		}
		
		if(eventSource != toggleable_cache){
			toggleable_cache.setText(getEntity().getAuthorshipCache());
		}
		
		// we have to notify the parent if this is embedded in the nonviral name section
		// maybe we can handle this a little bit more elegant
		if(getParentElement() instanceof AbstractCdmDetailSection)
			firePropertyChangeEvent(new CdmPropertyChangeEvent(getParentElement(), null));
	}
	
	private TeamOrPersonBase getAuthor(Object object){
		if(object == null){
			return null;
		}
		Object deproxiedObject = HibernateProxyHelper.deproxy(object);
		if(deproxiedObject instanceof Person){
			Person person = (Person) deproxiedObject;
			Team team = Team.NewInstance();
			team.addTeamMember(person);
			return team;
		}
		else if(deproxiedObject instanceof Team){
			return (Team) deproxiedObject;
		}else{
			throw new IllegalArgumentException("Given object os of the wrong type");
		}
	}
}
