// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelectionProvider;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created 05.06.2009
 * @version 1.0
 */
public class AuthorshipDetailSection extends AbstractCdmDetailSection<NonViralName> implements ITaxonDetailSection {


	private static final Logger logger = Logger.getLogger(AuthorshipDetailSection.class);


	/**
	 * @param parentElement
	 * @param listener
	 * @param style
	 */
	public AuthorshipDetailSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
	}
	
	/**
	 * Author title is not in titleCache field
	 * 
	 * @see eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection#setSectionTitle()
	 */
	@Override
	protected void setSectionTitle() {
		if(getEntity() != null){
			this.setText(getHeading() + ": " + getEntity().getAuthorshipCache());
		}
	}

	public void setTaxon(TaxonBase entity) {
		NonViralName name = (NonViralName) HibernateProxyHelper.deproxy(entity.getName());
		setEntity(name);
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.AbstractCdmDetailSection#getHeading()
	 */
	@Override
	public String getHeading() {
		return "Authorship";
	}
	
	@Override
	protected DetailType getDetailType() {
		return DetailType.AUTHORSHIP;
	}
	
}
