// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.strategy.parser.ParserProblem;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.forms.CheckboxElement;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEnableableFormElement;
import eu.etaxonomy.taxeditor.forms.ISelectableElement;
import eu.etaxonomy.taxeditor.forms.IntegerTextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.SelectionArbitrator;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.ToggleableTextElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.term.AbstractTermComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;

/**
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public class NameDetailElement extends AbstractCdmDetailElement<NonViralName> implements ISelectableElement, IEnableableFormElement{

	private static final Logger logger = Logger
			.getLogger(NameDetailElement.class);
	
	private ToggleableTextElement toggleable_cache;
	private AbstractTermComboElement<Rank> combo_rank;
	private TextWithLabelElement text_appendedPhrase;
	private TextWithLabelElement text_uninomial;
	private TextWithLabelElement text_infragenericEpithet;
	private TextWithLabelElement text_specificEpithet;
	private TextWithLabelElement text_infraspecificEpithet;
	private SelectionArbitrator selectionArbitrator;
	private CheckboxElement checkbox_anamorphic;
	private TextWithLabelElement text_breed;
	private IntegerTextWithLabelElement text_publicationYear;
	private IntegerTextWithLabelElement text_originalPublicationYear;
	private int cursorPosition;
	private ToggleableTextElement text_fullTitleCache;
	private ToggleableTextElement text_titleCache;
	
	public NameDetailElement(CdmFormFactory cdmFormFactory, ICdmFormElement formElement,
			int style) {
		super(cdmFormFactory, formElement);
		// register as selection listener
		if(cdmFormFactory.getSelectionProvider() != null){
			selectionArbitrator = cdmFormFactory.createSelectionArbitrator(this);
		}
	}

	@Override
	protected void createControls(ICdmFormElement formElement, NonViralName nonViralName, int style) {
		
		toggleable_cache = formFactory.createToggleableTextField(this, "Name Cache", nonViralName.getNameCache(), nonViralName.isProtectedNameCache(), style);
		
		combo_rank = formFactory.createTermComboElement(TermComboType.RANK, this, nonViralName.getRank(), style);

		createGenusOrUninomialControls(this, nonViralName, style);
		createInfragenerericEpithetControls(this, nonViralName, style);
		createSpecificEpithetControls(this, nonViralName, style);
		createInfraSpecificEpithetControls(this, nonViralName, style);
		
		createSpecificNameParts(this, nonViralName, style);
		
		text_appendedPhrase = formFactory.createTextWithLabelElement(formElement, "Appended Phrase", nonViralName.getAppendedPhrase(), style);
	}

	protected void clearCheckRankWarnings() {
		if(getEntity().hasProblem(ParserProblem.CheckRank)){
			getEntity().removeParsingProblem(ParserProblem.CheckRank);
		}
	}

//
//	
//	public void setEnabled(boolean enabled){
//		setEnabled(enabled, null);
//	}
	
	@Override
	protected void updateContent() {
		if(getEntity() == null){
			setEntity(NonViralName.NewInstance(null));
		}
		
		super.updateContent();
		
		if(isIrrelevant()){
			setIrrelevant(isIrrelevant());
		}else{
			setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
		}
	}


	private void createSpecificNameParts(ICdmFormElement formElement,
			NonViralName nonViralName, int style) {
		switch(nonViralName.getNomenclaturalCode()){
		case ICBN:
			createBotanicalNameParts(formElement, nonViralName, style);
			break;
		case ICZN:
			createZoologicalNameParts(formElement, nonViralName, style);
			break;
		}
	}
	
	private void createBotanicalNameParts(ICdmFormElement formElement, NonViralName nonViralName, int style){
		BotanicalName botanicalName = (BotanicalName) nonViralName;
		checkbox_anamorphic = formFactory.createCheckbox(formElement, "Anamorphic", botanicalName.isAnamorphic(), style);
	}

	private void createZoologicalNameParts(ICdmFormElement formElement, NonViralName nonViralName, int style){
		ZoologicalName zoologicalName = (ZoologicalName) nonViralName;
		text_breed = formFactory.createTextWithLabelElement(formElement, "Breed", zoologicalName.getBreed(), style);
		text_publicationYear = formFactory.createIntegerTextWithLabelElement(formElement, "Publication Year", zoologicalName.getPublicationYear(), style);
		text_originalPublicationYear = formFactory.createIntegerTextWithLabelElement(formElement, "Orig. Publication Year", zoologicalName.getOriginalPublicationYear(), style);
	}
	
	private void createGenusOrUninomialControls(ICdmFormElement element, NonViralName nonViralName, int style){		
		String title = "Genus";
		Rank rank = nonViralName.getRank();
		if(rank != null && rank.isSupraGeneric()){
			title = "Uninomial";			
		}
		text_uninomial = formFactory.createTextWithLabelElement(element, title, nonViralName.getGenusOrUninomial(), style);
	}
	
	private void createInfragenerericEpithetControls(ICdmFormElement element, NonViralName nonViralName, int style){
		if(nonViralName.getRank() != null && nonViralName.getRank().isInfraGeneric() && !nonViralName.getRank().isSpeciesAggregate()){
			text_infragenericEpithet = formFactory.createTextWithLabelElement(element, "Infrageneric Epithet", nonViralName.getInfraGenericEpithet(), style);
		}
	}
	
	private void createSpecificEpithetControls(ICdmFormElement element, NonViralName nonViralName, int style){
		if(nonViralName.getRank() != null && (nonViralName.getRank().isSpecies()  || nonViralName.getRank().isInfraSpecific() || nonViralName.getRank().isSpeciesAggregate())){
			text_specificEpithet = formFactory.createTextWithLabelElement(element, "Specific Epithet", nonViralName.getSpecificEpithet(), SWT.NULL);
		}
	}
	
	private void createInfraSpecificEpithetControls(ICdmFormElement element, NonViralName nonViralName, int style){
		if(nonViralName.getRank() != null && nonViralName.getRank().isInfraSpecific()){
			text_infraspecificEpithet = formFactory.createTextWithLabelElement(element, "Infraspecific Epithet", nonViralName.getInfraSpecificEpithet(), SWT.NULL);
		}
	}
	
	@Override
	public void handleEvent(Object eventSource){
		if(eventSource == combo_rank){
			getEntity().setRank(combo_rank.getSelection());
			clearCheckRankWarnings();	
			EditorUtil.refreshDetailsViewer();
		}		
		else if(eventSource == text_appendedPhrase){
			getEntity().setAppendedPhrase(text_appendedPhrase.getText());
		}
		else if(eventSource == toggleable_cache){
			getEntity().setNameCache(toggleable_cache.getText(), toggleable_cache.getState());
			//setEnabled(! text_nameCache.getState(), Arrays.asList(new Object[]{text_nameCache, combo_nomenclaturalCode}));
			if(! isIrrelevant()) setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
		}
		else if(eventSource == text_infragenericEpithet){
			getEntity().setInfraGenericEpithet(text_infragenericEpithet.getText());
		}
		else if(eventSource == text_infraspecificEpithet){
			getEntity().setInfraSpecificEpithet(text_infraspecificEpithet.getText());
		}
		else if(eventSource == text_specificEpithet){
			getEntity().setSpecificEpithet(text_specificEpithet.getText());
		}
		else if(eventSource == text_uninomial){
			getEntity().setGenusOrUninomial(text_uninomial.getText());
		}		
		else if(eventSource == checkbox_anamorphic){
			((BotanicalName)getEntity()).setAnamorphic(checkbox_anamorphic.getSelection());
		}
		
		if(eventSource != toggleable_cache){
			toggleable_cache.setText(getEntity().getTitleCache());
		}
		
		// we have to notify the parent if this is embedded in the nonviral name section
		// maybe we can handle this a little bit more elegant
		if(getParentElement() instanceof AbstractCdmDetailSection)
			firePropertyChangeEvent(new CdmPropertyChangeEvent(getParentElement(), null));
	}
		
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.ISelectableElement#getSelectionArbitrator()
	 */
	public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}
}
