// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Collection;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created Mar 18, 2010
 * @version 1.0
 */
public class NameRelationshipDetailSection extends AbstractEntityCollectionSection<TaxonNameBase, NameRelationship> implements ITaxonDetailSection{

	private static final Logger logger = Logger
				.getLogger(NameRelationshipDetailSection.class);
	
	public NameRelationshipDetailSection(CdmFormFactory formFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Name Relationship", style);
	}

	@Override
	public void addElement(NameRelationship element) {
		//getEntity().addRelationship(element);
	}

	@Override
	public NameRelationship createNewElement() {
		
		NameRelationshipWizard wizard = new NameRelationshipWizard(this);
		WizardDialog dialog = new WizardDialog(EditorUtil.getShell(), wizard);
		int status = dialog.open();
		
		if(status == IStatus.OK)
			return wizard.getNameRelationship();
		
		return null;
	}

	@Override
	public Collection<NameRelationship> getCollection(TaxonNameBase entity) {
		return entity.getRelationsFromThisName();
	}

	@Override
	public String getEmptyString() {
		return "No name relationships yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Add a new name relationship from this name.";
	}

	@Override
	public void removeElement(NameRelationship element) {
		getEntity().removeNameRelationship(element);
	}	
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.ITaxonDetailSection#setTaxon(eu.etaxonomy.cdm.model.taxon.TaxonBase)
	 */
	public void setTaxon(TaxonBase taxon) {	
		NonViralName name = (NonViralName) HibernateProxyHelper.deproxy(taxon.getName());
		setEntity(name);
	}
}
