// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.RootElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.selection.NameSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.NameRelationshipTypeComboElement;

/**
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public class NameRelationshipWizardPage extends WizardPage implements IPropertyChangeListener {


	private static final Logger logger = Logger
			.getLogger(NameRelationshipWizardPage.class);


	private CdmFormFactory formFactory;


	private NameSelectionElement selection_relatedTo;


	private NameRelationshipTypeComboElement combo_relationshipType;


	private NameRelationshipDetailSection callingSection;


	private TaxonNameBase entity;


	private NameRelationshipType type;


	private TaxonNameBase toName;

	
	protected NameRelationshipWizardPage(NameRelationshipDetailSection callingSection) {
		super("NameRelationshipWizardPage");
		setTitle("New Name Relationship");
		setDescription(callingSection.getEntity().getTitleCache());
		this.callingSection = callingSection;
		this.entity = callingSection.getEntity();
		this.formFactory = callingSection.getFormFactory();
		
		formFactory.addPropertyChangeListener(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		this.setPageComplete(false);
		Composite control = formFactory.createComposite(parent); 
		
		control.setLayout(CdmFormFactory.LAYOUT(2, false));
		
		RootElement rootElement = new RootElement(formFactory, control);
		
		combo_relationshipType = (NameRelationshipTypeComboElement) formFactory.createTermComboElement(TermComboType.NAMERELATIONSHIP, rootElement, null, SWT.NULL);
		
		selection_relatedTo = (NameSelectionElement) formFactory.createSelectionElement(SelectionType.NAME, ((NameRelationshipWizard)getWizard()).getConversationHolder(), rootElement, "Related to", null, SWT.NULL);
		
		setControl(control);	
	}
	
	public NameRelationship getNameRelationship(){
		return entity.addRelationshipToName(toName, type, null, null, null);
	}
	
	public void propertyChange(PropertyChangeEvent event) {
		if(event == null){
			return;
		}
		Object eventSource = event.getSource();
		if(eventSource == combo_relationshipType){
			type = combo_relationshipType.getSelection();
		}
		else if(eventSource == selection_relatedTo){
			toName = selection_relatedTo.getEntity();
		}
		
		setPageComplete(type != null && toName != null);
	}
	
	@Override
	public void dispose() {
		formFactory.removeSelectionListener(selection_relatedTo);
		super.dispose();
	}
}
