// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import org.apache.log4j.Logger;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CheckboxElement;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.selection.NameSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.NameTypeDesignationStatusComboElement;

/**
 * @author n.hoffmann
 * @created May 17, 2010
 * @version 1.0
 */
public class NameTypeDesignationElement extends
		AbstractTypeDesignationElement<NameTypeDesignation> {

	private static final Logger logger = Logger
			.getLogger(NameTypeDesignationElement.class);
	private CheckboxElement checkbox_rejectedType;
	private CheckboxElement checkbox_conservedType;
	private NameSelectionElement selection_typeName;
	private NameTypeDesignationStatusComboElement combo_typeStatus;
	
	public NameTypeDesignationElement(CdmFormFactory formFactory,
			AbstractFormSection section, NameTypeDesignation entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, style);
	}
	
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		
		checkbox_rejectedType = formFactory.createCheckbox(formElement, "Rejected Type", false, style);
		checkbox_conservedType = formFactory.createCheckbox(formElement, "Conserved Type", false, style);
		selection_typeName = (NameSelectionElement) formFactory.createSelectionElement(SelectionType.NAME, getConversationHolder(), formElement, "Name", null, style);
		combo_typeStatus = (NameTypeDesignationStatusComboElement) formFactory.createTermComboElement(TermComboType.NAMETYPEDESIGNATIONSTATUS, formElement, null, style);
		
		super.createControls(formElement, style);
	}
	
	@Override
	public void setEntity(NameTypeDesignation entity) {
		super.setEntity(entity);
		checkbox_conservedType.setSelection(entity.isConservedType());
		checkbox_rejectedType.setSelection(entity.isRejectedType());
		
		selection_typeName.setEntity(entity.getTypeName());

		combo_typeStatus.setSelection(entity.getTypeStatus());
		
		checkbox_notDesignated.setSelection(entity.isNotDesignated());
	}
	
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == checkbox_conservedType){
			getEntity().setConservedType(checkbox_conservedType.getSelection());
		}
		else if(eventSource == checkbox_rejectedType){
			getEntity().setRejectedType(checkbox_rejectedType.getSelection());
		}
		else if(eventSource == selection_typeName){
			getEntity().setTypeName(selection_typeName.getEntity());
		}
		else if(eventSource == combo_typeStatus){
			getEntity().setTypeStatus(combo_typeStatus.getSelection());
		}
		else if(eventSource == checkbox_notDesignated){
			getEntity().setNotDesignated(checkbox_notDesignated.getSelection());
		}
		else if(eventSource == selection_reference){
			getEntity().setCitation(selection_reference.getSelection());
		}
		else if(eventSource == text_referenceDetail){
			getEntity().setCitationMicroReference(text_referenceDetail.getText());
		}
		else if(eventSource == text_originaleNameString){
			getEntity().setOriginalNameString(text_originaleNameString.getText());
		}
	}
}
