// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import org.apache.log4j.Logger;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEntityElement;
import eu.etaxonomy.taxeditor.forms.ISelectableElement;
import eu.etaxonomy.taxeditor.forms.SelectionArbitrator;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.term.AbstractTermComboElement;
import eu.etaxonomy.taxeditor.section.supplemental.AbstractReferencedEntityElement;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class NomenclaturalStatusElement extends AbstractReferencedEntityElement<NomenclaturalStatus> implements IEntityElement<NomenclaturalStatus>, ISelectableElement{
	private static final Logger logger = Logger
				.getLogger(NomenclaturalStatusElement.class);

	private AbstractTermComboElement<NomenclaturalStatusType> nomenclaturalStatusTypeCombo;
	
	private TextWithLabelElement ruleConsideredText;
	
	private SelectionArbitrator selectionArbitrator;

	/**
	 * @param formElement
	 * @param element
	 * @param removeListener
	 * @param propertyChangelistener
	 * @param style
	 */
	public NomenclaturalStatusElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement,
			NomenclaturalStatus element, SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, style);
		selectionArbitrator = cdmFormFactory.createSelectionArbitrator(this);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		nomenclaturalStatusTypeCombo = formFactory.createTermComboElement(TermComboType.NOMENCLATURALSTATUSTYPE, this, null, style);
		ruleConsideredText = formFactory.createTextWithLabelElement(this, "Rule Considered", "", style);
		
		super.createControls(element, style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#setElement(eu.etaxonomy.cdm.model.common.VersionableEntity)
	 */
	@Override
	public void setEntity(NomenclaturalStatus entity) {
		super.setEntity(entity);
		nomenclaturalStatusTypeCombo.setSelection(entity.getType());
		ruleConsideredText.setText(entity.getRuleConsidered());
	}

	public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == nomenclaturalStatusTypeCombo){
			getEntity().setType(nomenclaturalStatusTypeCombo.getSelection());
			// fire an empty property change to force regeneration of full title cache in freetext area
			firePropertyChangeEvent(null);
		}
		else if(eventSource == ruleConsideredText){
			getEntity().setRuleConsidered(ruleConsideredText.getText());
			// fire an empty property change to force regeneration of full title cache in freetext area
			firePropertyChangeEvent(null);
		}
		else if(eventSource == selection_reference){
			getEntity().setCitation(selection_reference.getSelection());
		}
		else if(eventSource == text_referenceDetail){
			getEntity().setCitationMicroReference(text_referenceDetail.getText());
		}
		else if(eventSource == text_originaleNameString){
			getEntity().setOriginalNameString(text_originaleNameString.getText());
		}
	}
}
