// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Collection;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class NomenclaturalStatusSection extends AbstractEntityCollectionSection<TaxonNameBase, NomenclaturalStatus> implements ITaxonDetailSection{

	private static final Logger logger = Logger
			.getLogger(NomenclaturalStatusSection.class);

	
	/**
	 * @param parentElement
	 * @param entity
	 * @param title
	 * @param listener
	 * @param style
	 */
	public NomenclaturalStatusSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(cdmFormFactory, conversation, parentElement, "Nomenclatural Status", style);
	}

	@Override
	public Collection<NomenclaturalStatus> getCollection(TaxonNameBase entity) {
		if(entity != null){
			return entity.getStatus();
		}
		return null;
	}

	@Override
	public String getEmptyString() {
		return "No nomenclatural status yet.";
	}

	@Override
	public NomenclaturalStatus createNewElement() {
		return NomenclaturalStatus.NewInstance(null);
	}

	@Override
	protected String getTooltipString() {
		return "Create a new nomenclatural status";
	}

	@Override
	public void addElement(NomenclaturalStatus element) {
		getEntity().addStatus(element);
	}

	@Override
	public void removeElement(NomenclaturalStatus element) {
		getEntity().removeStatus(element);
	}

	public void setTaxon(TaxonBase taxon) {
		NonViralName nonViralName = (NonViralName) HibernateProxyHelper.deproxy(taxon.getName());
		setEntity(nonViralName);
	}

}
