// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.ToggleableTextElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.EnumComboType;
import eu.etaxonomy.taxeditor.forms.term.NomenclaturalCodeComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;

/**
 * @author n.hoffmann
 * @created May 20, 2010
 * @version 1.0
 */
public class NonViralNameDetailElement extends AbstractCdmDetailElement<NonViralName> {


	private static final Logger logger = Logger
			.getLogger(NonViralNameDetailElement.class);
	private NameDetailSection section_name;
	private AuthorshipDetailSection section_author;
	private ToggleableTextElement toggleable_cache;
	private NomenclaturalCodeComboElement combo_nomenclaturalCode;

	public NonViralNameDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls(eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement, eu.etaxonomy.cdm.model.common.AnnotatableEntity, int)
	 */
	@Override
	protected void createControls(ICdmFormElement formElement,
			NonViralName entity, int style) {
		toggleable_cache = formFactory.createToggleableTextField(formElement, "Cache", entity.getTitleCache(), entity.isProtectedTitleCache(), style);
		
		combo_nomenclaturalCode = (NomenclaturalCodeComboElement) formFactory.createEnumComboElement(EnumComboType.NOMENCLATURALCODE, formElement, style);
		combo_nomenclaturalCode.setSelection(entity.getNomenclaturalCode());
		
		section_name = (NameDetailSection) formFactory.createCdmDetailSection(DetailType.SCIENTIFICNAME, getConversationHolder(), formElement, null, Section.TWISTIE | Section.EXPANDED);
		section_name.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
		addControl(section_name);
		addElement(section_name);
		section_author = (AuthorshipDetailSection) formFactory.createCdmDetailSection(DetailType.AUTHORSHIP, getConversationHolder(), formElement, null, Section.TWISTIE | Section.EXPANDED);
		section_author.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
		addControl(section_author);
		addElement(section_author);
	}

	@Override
	protected void updateContent() {
		super.updateContent();
		
		// disable nomenclatural code, because changing of nom.code is not implemented on library side
		combo_nomenclaturalCode.setEnabled(false);
		
		setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
	}
	
	@Override
	public void setEntity(NonViralName entity) {
		super.setEntity(entity);
		section_name.setEntity(entity);
		section_author.setEntity(entity);
	}
	
	@Override
	public void removeElements() {
		super.removeElements();
		if(section_name != null){
			removeControl(section_name);
			section_name.dispose();
			section_name = null;
		}
		if(section_author != null){
			removeControl(section_author);
			section_author.dispose();
			section_author = null;
		}
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == toggleable_cache){
			getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
			boolean irrelevant = toggleable_cache.getState();
			List<Object> except = Arrays.asList(new Object[]{toggleable_cache});
			setIrrelevant(irrelevant, except);
		}
		else if(eventSource == section_name || eventSource == section_author){
			if(getParentElement() instanceof AbstractCdmDetailSection)
				((AbstractCdmDetailSection) getParentElement()).updateTitle();
			if(! toggleable_cache.getState()){
				toggleable_cache.setText(getEntity().getTitleCache());
			}
		}
	}
}
