// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEntityElement;
import eu.etaxonomy.taxeditor.forms.ISelectableElement;
import eu.etaxonomy.taxeditor.forms.SelectionArbitrator;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class ProtologueElement extends AbstractEntityCollectionElement<DescriptionElementBase> implements IEntityElement<DescriptionElementBase>, ISelectableElement{

	private SelectionArbitrator selectionArbitrator;


	/**
	 * @param parent
	 * @param element
	 * @param removeListener
	 * @param propertyChangelistener
	 * @param style
	 */
	public ProtologueElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement, DescriptionElementBase element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, null, style);
		selectionArbitrator = cdmFormFactory.createSelectionArbitrator(this);
	}

	private static final Logger logger = Logger
			.getLogger(ProtologueElement.class);

	private TextWithLabelElement protologueUriText;
	private MediaRepresentationPart mediaRepresentationPart;
	private Media media;
	private MediaRepresentation mediaRepresentation;

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		protologueUriText = formFactory.createTextWithLabelElement(this, "Protologue URI", "", style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#setElement(eu.etaxonomy.cdm.model.common.VersionableEntity)
	 */
	@Override
	public void setEntity(DescriptionElementBase element) {
		this.entity = element;
		
		if(element.getMedia().size() == 0){
			element.addMedia(Media.NewInstance());
		}
		media = element.getMedia().iterator().next();
		
		
		if(media.getRepresentations().size() == 0){
			media.addRepresentation(MediaRepresentation.NewInstance());
		}		
		mediaRepresentation = media.getRepresentations().iterator().next();
		
		if(mediaRepresentation.getParts().size() == 0){
			mediaRepresentation.addRepresentationPart(MediaRepresentationPart.NewInstance(null, null));
		}
		
		mediaRepresentationPart = mediaRepresentation.getParts().iterator().next();
				
		protologueUriText.setText(mediaRepresentationPart.getUri());
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.ISelectableElement#getSelectionArbitrator()
	 */
	public SelectionArbitrator getSelectionArbitrator() {
		return selectionArbitrator;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == protologueUriText){
			mediaRepresentationPart.setUri(protologueUriText.getText());
		}	
	}
}
