// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Collection;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonNameDescription;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class ProtologueSection extends AbstractEntityCollectionSection<TaxonNameDescription, DescriptionElementBase> implements ITaxonDetailSection{
	
	private static final Logger logger = Logger
	.getLogger(ProtologueSection.class);

	/**
	 * 
	 * @param cdmFormFactory
	 * @param parentElement
	 * @param style
	 */
	public ProtologueSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Protologues", style);
	}

	@Override
	public DescriptionElementBase createNewElement() {
		return TextData.NewInstance(Feature.PROTOLOGUE());
	}


	@Override
	public Collection<DescriptionElementBase> getCollection(
			TaxonNameDescription entity) {
		if(entity != null){
			return entity.getElements();
		}
		return null;
	}


	@Override
	public String getEmptyString() {
		return "No protologues yet.";
	}


	@Override
	protected String getTooltipString() {
		return "Create a new protologue";
	}

	@Override
	public void addElement(DescriptionElementBase element) {
		getEntity().addElement(element);
	}

	@Override
	public void removeElement(DescriptionElementBase element) {
		getEntity().removeElement(element);
	}

	public void setTaxon(TaxonBase taxon) {
		
		NonViralName name = (NonViralName) HibernateProxyHelper.deproxy(taxon.getName());
		
		// FIXME does not handle multiple name descriptions at the moment
		if(name.getDescriptions().size() == 0){
			name.addDescription(TaxonNameDescription.NewInstance(name));
		}
		setEntity((TaxonNameDescription) name.getDescriptions().iterator().next());
	}
}
