// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import org.apache.log4j.Logger;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.occurrence.Specimen;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.selection.DerivedUnitBaseSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.SpecimenTypeDesignationStatusComboElement;

/**
 * @author n.hoffmann
 * @created May 17, 2010
 * @version 1.0
 */
public class SpecimenTypeDesignationElement extends
		AbstractTypeDesignationElement<SpecimenTypeDesignation> {


	private static final Logger logger = Logger
			.getLogger(SpecimenTypeDesignationElement.class);
	private SpecimenTypeDesignationStatusComboElement combo_typeStatus;
	private DerivedUnitBaseSelectionElement selection_typeSpecimen;
	private TextWithLabelElement text_specimenTypeText;
	
	public SpecimenTypeDesignationElement(CdmFormFactory formFactory,
			AbstractFormSection section, SpecimenTypeDesignation entity,
			SelectionListener removeListener, int style) {
		super(formFactory, section, entity, removeListener, style);
	}
	
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		// TODO until we do not have a specimen editor we will provide a simple text interface to enter specimen 
		// information
		
		text_specimenTypeText = formFactory.createTextWithLabelElement(formElement, "Unit", null, style);
		// TODO testing the specimen selection dialog
//		selection_typeSpecimen = (DerivedUnitBaseSelectionElement) formFactory.createSelectionElement(SelectionType.DERIVED_UNIT, formElement, "Unit", null, style);
		combo_typeStatus = (SpecimenTypeDesignationStatusComboElement) formFactory.createTermComboElement(TermComboType.SPECIMENTYPEDESIGNATIONSTATUS, formElement, null, style);
		
		super.createControls(formElement, style);
	}
	
	@Override
	public void setEntity(SpecimenTypeDesignation entity) {
		super.setEntity(entity);
		// TODO remove this once specimen are completely implemented
		if(entity.getTypeSpecimen() != null && text_specimenTypeText != null){			
			text_specimenTypeText.setText(entity.getTypeSpecimen().getTitleCache());
		}
		
		SpecimenTypeDesignationStatus typeStatus = (SpecimenTypeDesignationStatus) HibernateProxyHelper.deproxy(entity.getTypeStatus());
		combo_typeStatus.setSelection(typeStatus);
		checkbox_notDesignated.setSelection(entity.isNotDesignated());
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == combo_typeStatus){
			getEntity().setTypeStatus(combo_typeStatus.getSelection());
		}
		else if(eventSource == text_specimenTypeText){
			// TODO this is a simple workaround to enter type specimen as text strings
			if(getEntity().getTypeSpecimen() == null){
				Specimen typeSpecimen = Specimen.NewInstance();
				getEntity().setTypeSpecimen(typeSpecimen);
			}
			getEntity().getTypeSpecimen().setTitleCache(text_specimenTypeText.getText());
		}		
		else if(eventSource == checkbox_notDesignated){
			getEntity().setNotDesignated(checkbox_notDesignated.getSelection());
		}
		else if(eventSource == selection_reference){
			getEntity().setCitation(selection_reference.getSelection());
		}
		else if(eventSource == text_referenceDetail){
			getEntity().setCitationMicroReference(text_referenceDetail.getText());
		}
		else if(eventSource == text_originaleNameString){
			getEntity().setOriginalNameString(text_originaleNameString.getText());
		}
	}
}
