// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.name;

import java.util.Collection;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created May 17, 2010
 * @version 1.0
 */
public class TypeDesignationSection extends AbstractEntityCollectionSection<TaxonNameBase, TypeDesignationBase> implements ITaxonDetailSection {

	private static final Logger logger = Logger
			.getLogger(TypeDesignationSection.class);

	public TypeDesignationSection(CdmFormFactory formFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, int style) {
		super(formFactory, conversation, parentElement, "Type Designations", style);
	}
	
	
	
	@Override
	public void addElement(TypeDesignationBase element) {
		getEntity().addTypeDesignation(element, false);
	}

	@Override
	public TypeDesignationBase createNewElement() {
		if(isSpecimenType()){
			return SpecimenTypeDesignation.NewInstance();
		}else{
			return NameTypeDesignation.NewInstance();
		}
	}

	@Override
	public Collection<TypeDesignationBase> getCollection(TaxonNameBase entity) {
		return entity.getTypeDesignations();
	}

	@Override
	public String getEmptyString() {
		return "No type designations yet.";
	}

	@Override
	protected String getTooltipString() {
		return "Add a type designation";
	}

	@Override
	public void removeElement(TypeDesignationBase element) {
		getEntity().removeTypeDesignation(element);
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.ITaxonDetailSection#setTaxon(eu.etaxonomy.cdm.model.taxon.TaxonBase)
	 */
	public void setTaxon(TaxonBase taxon) {	
		NonViralName name = (NonViralName) HibernateProxyHelper.deproxy(taxon.getName());
		setEntity(name);
	}
	
	private boolean isSpecimenType(){
		Rank rank = getEntity().getRank();
		return rank.isSpecies() || rank.isInfraSpecific();
	}
}
