// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.occurrence;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.MethodNotSupportedByDerivedUnitTypeException;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.selection.NameSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.PreservationMethodComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jun 24, 2010
 * @version 1.0
 */
public class DerivedUnitBaseDetailElement extends AbstractCdmDetailElement<DerivedUnitBase> implements IFacadeHolder{
	

	private static final Logger logger = Logger
			.getLogger(DerivedUnitBaseDetailElement.class);

	private DerivedUnitFacade facade;

	private TextWithLabelElement text_derivedUnitDefinitions;

	private TextWithLabelElement text_accessionNumber;

	private TextWithLabelElement text_catalogNumber;

	private NameSelectionElement selection_storedUnder;

	private TextWithLabelElement text_collectorsNumber;

	private TextWithLabelElement text_exsicatum;

	private PreservationMethodComboElement combo_preservationMethod;
	
	public DerivedUnitBaseDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
		// TODO Auto-generated constructor stub
	}
	
	@Override
	protected void createControls(ICdmFormElement formElement,
			DerivedUnitBase entity, int style) {
		text_derivedUnitDefinitions = formFactory.createTextWithLabelElement(formElement, "Specimen Definition", facade.getDerivedUnitDefinition(CdmStore.getDefaultLanguage()), style);
		text_accessionNumber = formFactory.createTextWithLabelElement(formElement, "Accession Number", facade.getAccessionNumber(), style);
		text_catalogNumber = formFactory.createTextWithLabelElement(formElement, "Catalog Number", facade.getCatalogNumber(), style);
		try {
			combo_preservationMethod = (PreservationMethodComboElement) formFactory.createTermComboElement(TermComboType.PRESERVATION_METHOD, formElement, facade.getPreservationMethod(), style);
		} catch (MethodNotSupportedByDerivedUnitTypeException e) {
			logger.error("MethodNotSupportedByDerivedUnitTypeException when trying to get preservation method for derive unit", e);
			//throw new RuntimeException("MethodNotSupportedByDerivedUnitTypeException when trying to get preservation method for derive unit");
		}
		selection_storedUnder = (NameSelectionElement) formFactory.createSelectionElement(SelectionType.NAME, getConversationHolder(), formElement, "Stored Under", facade.getStoredUnder(), style);
		text_collectorsNumber = formFactory.createTextWithLabelElement(formElement, "Collectors Number", facade.getCollectorsNumber(), style);
		text_exsicatum = formFactory.createTextWithLabelElement(formElement, "Exsicatum", facade.getExsiccatum(), style);		
		// source
//		selection_collection = formFactory.createSelectionElement(SelectionType.COLLECTION, getConversationHolder(), formElement, "Collection", facade.getCollection(), style);
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_derivedUnitDefinitions){
			facade.addDerivedUnitDefinition(text_derivedUnitDefinitions.getText(), CdmStore.getDefaultLanguage());
		}
//		else if(eventSource == selection_determinationEvent){
//			facade.addDetermination(selection_determinationEvent.getEntity());
//		}
		else if(eventSource == text_accessionNumber){
			facade.setAccessionNumber(text_accessionNumber.getText());
		}
		else if(eventSource == text_catalogNumber){
			facade.setCatalogNumber(text_catalogNumber.getText());
		}
		else if(eventSource == combo_preservationMethod){
			try {
				facade.setPreservationMethod(combo_preservationMethod.getSelection());
			} catch (MethodNotSupportedByDerivedUnitTypeException e) {
				logger.error("MethodNotSupportedByDerivedUnitTypeException when trying to set preservation method for derive unit", e);
				throw new RuntimeException("MethodNotSupportedByDerivedUnitTypeException when trying to set preservation method for derive unit");
			}
		}
		else if(eventSource == selection_storedUnder){
			facade.setStoredUnder(selection_storedUnder.getEntity());
		}
		else if(eventSource == text_collectorsNumber){
			facade.setCollectorsNumber(text_collectorsNumber.getText());
		}
//		else if(eventSource == text_exsicatum){
//			facade.setExsiccatum();
//		}
//		else if(source){
//			
//		}
//		else if(eventSource == selection_collection){
//			facade.setCollection(selection_collection.getSelection());
//		}
	}

	public DerivedUnitFacade getFacade() {
		return facade;
	}

	public void setFacade(DerivedUnitFacade facade) {
		this.facade = facade;
		updateContent();
	}


}
