// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.occurrence;

import org.apache.log4j.Logger;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeConfigurator;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;

/**
 * @author n.hoffmann
 * @created Jun 17, 2010
 * @version 1.0
 */
public class DerivedUnitFacadeDetailElement extends AbstractCdmDetailElement<DerivedUnitBase> {
	
	private DerivedUnitFacade facade;

	private DerivedUnitFacadeConfigurator configurator;

	private GatheringEventDetailSection section_gatheringEvent;
	private DerivedUnitBaseDetailSection section_derivedUnitBase;
	private FieldObservationDetailSection section_fieldObservation;


	private static final Logger logger = Logger
			.getLogger(DerivedUnitFacadeDetailElement.class);

	public DerivedUnitFacadeDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
		configurator = DerivedUnitFacadeConfigurator.NewInstance();
		configurator.setMoveDerivedUnitMediaToGallery(true);
		configurator.setMoveFieldObjectMediaToGallery(true);
	}
	
	@Override
	public void setEntity(DerivedUnitBase entity) {
		super.setEntity(entity);
		try {
			facade = DerivedUnitFacade.NewInstance(entity, configurator);
			
			section_derivedUnitBase.setFacade(facade);
			section_gatheringEvent.setFacade(facade);
			section_fieldObservation.setFacade(facade);
			
		} catch (DerivedUnitFacadeNotSupportedException e) {
			EditorUtil.errorDialog("Error", "SpecimenFacadeNotSupportedException while inititating SpecimenFacade");
			logger.error("SpecimenFacadeNotSupportedException while inititating SpecimenFacade", e);
		}
	}
	
	@Override
	public void removeElements() {
		super.removeElements();
		if(section_derivedUnitBase != null){
			removeControl(section_derivedUnitBase);
			section_derivedUnitBase.dispose();
			section_derivedUnitBase = null;
		}
		if(section_gatheringEvent != null){
			removeControl(section_gatheringEvent);
			section_gatheringEvent.dispose();
			section_gatheringEvent = null;
		}
		if(section_fieldObservation != null){
			removeControl(section_fieldObservation);
			section_fieldObservation.dispose();
			section_fieldObservation = null;
		}
	}
	
	@Override
	public DerivedUnitBase getEntity() {
		throw new IllegalArgumentException("Should not be called");
	}
	
	public DerivedUnitFacade getFacade(){
		return facade;
	}
	
	@Override
	protected void createControls(ICdmFormElement formElement,
			DerivedUnitBase entity, int style) {
		
		section_gatheringEvent = (GatheringEventDetailSection) formFactory.createCdmDetailSection(DetailType.GATHERING_EVENT, getConversationHolder(), formElement, null, Section.TWISTIE | Section.EXPANDED);
		section_gatheringEvent.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
		addControl(section_gatheringEvent);
		addElement(section_gatheringEvent);
		
		section_fieldObservation = (FieldObservationDetailSection) formFactory.createCdmDetailSection(DetailType.FIELD_OBSERVATION, getConversationHolder(), formElement, null, Section.TWISTIE | Section.EXPANDED);
		section_fieldObservation.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
		addControl(section_fieldObservation);
		addElement(section_fieldObservation);
		
		section_derivedUnitBase = (DerivedUnitBaseDetailSection) formFactory.createCdmDetailSection(DetailType.DERIVED_UNIT, getConversationHolder(), formElement, null, Section.TWISTIE | Section.EXPANDED);
		section_derivedUnitBase.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
		addControl(section_derivedUnitBase);
		addElement(section_derivedUnitBase);
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == section_derivedUnitBase || eventSource == section_gatheringEvent || eventSource == section_fieldObservation){
			if(getParentElement() instanceof AbstractCdmDetailSection)
				((AbstractCdmDetailSection) getParentElement()).updateTitle();
		}
	}
}
