// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.occurrence;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.occurrence.FieldObservation;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IntegerTextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.term.SexComboElement;
import eu.etaxonomy.taxeditor.forms.term.StageComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jun 17, 2010
 * @version 1.0
 */
public class FieldObservationDetailElement extends AbstractCdmDetailElement<FieldObservation> implements IFacadeHolder {
	

	private static final Logger logger = Logger
			.getLogger(FieldObservationDetailElement.class);

	private DerivedUnitFacade facade;

	private TextWithLabelElement text_ecology;

	private TextWithLabelElement text_plantDescription;

	private TextWithLabelElement text_fieldObjectDefinition;

	private TextWithLabelElement text_fieldNumber;

	private TextWithLabelElement text_fieldNotes;

	private IntegerTextWithLabelElement number_individualCount;

	private SexComboElement combo_sex;

	private StageComboElement combo_lifeStage;
	
	public FieldObservationDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}
	
	@Override
	protected void createControls(ICdmFormElement formElement,
			FieldObservation entity, int style) {
		text_ecology = formFactory.createTextWithLabelElement(formElement, "Ecology", facade.getEcology(CdmStore.getDefaultLanguage()), style);
		text_plantDescription = formFactory.createTextWithLabelElement(formElement, "Plant Description", facade.getPlantDescription(CdmStore.getDefaultLanguage()), style);
		text_fieldObjectDefinition = formFactory.createTextWithLabelElement(formElement, "Field Object Definition", facade.getFieldObjectDefinition(CdmStore.getDefaultLanguage()), style);
		text_fieldNumber = formFactory.createTextWithLabelElement(formElement, "Field Number", facade.getFieldNumber(), style);
		text_fieldNotes = formFactory.createTextWithLabelElement(formElement, "Field Notes", facade.getFieldNotes(), style);
		
		number_individualCount = formFactory.createIntegerTextWithLabelElement(formElement, "Individual Count", facade.getIndividualCount(), style);
		combo_lifeStage = (StageComboElement) formFactory.createTermComboElement(TermComboType.STAGE, formElement, facade.getLifeStage(), style);
		combo_sex = (SexComboElement) formFactory.createTermComboElement(TermComboType.SEX, formElement, facade.getSex(), style);
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_ecology){
			facade.setEcology(text_ecology.getText());
		}
		else if(eventSource == text_plantDescription){
			facade.setPlantDescription(text_plantDescription.getText());
		}
		else if(eventSource == text_fieldObjectDefinition){
			facade.addFieldObjectDefinition(text_fieldObjectDefinition.getText(), CdmStore.getDefaultLanguage());
		}
		else if(eventSource == text_fieldNumber){
			facade.setFieldNumber(text_fieldNumber.getText());
		}
		else if(eventSource == text_fieldNotes){
			facade.setFieldNotes(text_fieldNotes.getText());
		}
		else if(eventSource == number_individualCount){
			facade.setIndividualCount(number_individualCount.getInteger());
		}
		else if(eventSource == combo_lifeStage){
			facade.setLifeStage(combo_lifeStage.getSelection());
		}
		else if(eventSource == combo_sex){
			facade.setSex(combo_sex.getSelection());
		}
	}

	public DerivedUnitFacade getFacade() {
		return facade;
	}

	public void setFacade(DerivedUnitFacade facade) {
		this.facade = facade;
		updateContent();
	}
}
