// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.occurrence;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.occurrence.GatheringEvent;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IntegerTextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.TimePeriodElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.TermComboType;
import eu.etaxonomy.taxeditor.forms.selection.AgentSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.LanguageComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;

/**
 * @author n.hoffmann
 * @created Jun 24, 2010
 * @version 1.0
 */
public class GatheringEventDetailElement extends AbstractCdmDetailElement<GatheringEvent> implements IFacadeHolder{
	

	private static final Logger logger = Logger
			.getLogger(GatheringEventDetailElement.class);
	
	private DerivedUnitFacade facade;

	private IntegerTextWithLabelElement number_absoluteElevation;

	private IntegerTextWithLabelElement number_absoluteElevationError;

	private IntegerTextWithLabelElement number_absoluteElevationMinimum;

	private IntegerTextWithLabelElement number_absoluteElevationMaximum;

	private AgentSelectionElement selection_collector;

	private TextWithLabelElement text_collectingMethod;

	private IntegerTextWithLabelElement number_distanceToGround;

	private IntegerTextWithLabelElement number_distanceToWaterSurface;

	private TextWithLabelElement text_gatheringEventDescription;

	private TimePeriodElement composite_gatheringPeriod;

	private TextWithLabelElement text_localityText;

	private LanguageComboElement combo_localityLanguage;
	


	public GatheringEventDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected void createControls(ICdmFormElement formElement,
			GatheringEvent entity, int style) {
		//collectingArea
		number_absoluteElevation = formFactory.createIntegerTextWithLabelElement(formElement, "Absolute Elevation", facade.getAbsoluteElevation(), style);
		number_absoluteElevationError = formFactory.createIntegerTextWithLabelElement(formElement, "Elevation Error", facade.getAbsoluteElevationError(), style);
		number_absoluteElevationMinimum = formFactory.createIntegerTextWithLabelElement(formElement, "Evelation Minimum", facade.getAbsoluteElevationMinimum(), style); 
		number_absoluteElevationMaximum = formFactory.createIntegerTextWithLabelElement(formElement, "Elevation Maximum", facade.getAbsoluteElevationMinimum(), style);
		
		selection_collector = (AgentSelectionElement) formFactory.createSelectionElement(SelectionType.AGENT, getConversationHolder(), formElement, "Collector", facade.getCollector(), style);
		text_collectingMethod = formFactory.createTextWithLabelElement(formElement, "Collecting Method", facade.getCollectingMethod(), style);
		number_distanceToGround = formFactory.createIntegerTextWithLabelElement(formElement, "Distance To Ground", facade.getDistanceToGround(), style);
		number_distanceToWaterSurface = formFactory.createIntegerTextWithLabelElement(formElement, "Distance To Water Surface", facade.getDistanceToWaterSurface(), style);
		// exactLocation
		text_gatheringEventDescription = formFactory.createTextWithLabelElement(formElement, "Gathering Event Description", facade.getGatheringEventDescription(), style);
		composite_gatheringPeriod = formFactory.createTimePeriodElement(formElement, "Gathering Period", facade.getGatheringPeriod(), style);
		text_localityText = formFactory.createTextWithLabelElement(formElement, "Locality", facade.getLocalityText(), style);
		combo_localityLanguage = (LanguageComboElement) formFactory.createTermComboElement(TermComboType.LANGUAGE, formElement, facade.getLocalityLanguage(), style);
		
		
	}

	@Override
	public void handleEvent(Object eventSource) {
	//	if(eventSource == xxx_collectingArea){
	//		
	//	}else		
		if(eventSource == number_absoluteElevation){
			facade.setAbsoluteElevation(number_absoluteElevation.getInteger());
		}
		else if(eventSource == number_absoluteElevationError){
			facade.setAbsoluteElevationError(number_absoluteElevationError.getInteger());
		}
		else if(eventSource == number_absoluteElevationMinimum){
			facade.setAbsoluteElevationRange(number_absoluteElevationMinimum.getInteger(), number_absoluteElevationMaximum.getInteger());
		}
		else if(eventSource == number_absoluteElevationMaximum){
			facade.setAbsoluteElevationRange(number_absoluteElevationMinimum.getInteger(), number_absoluteElevationMaximum.getInteger());
		}
		else if(eventSource == selection_collector){
			facade.setCollector(selection_collector.getEntity());
		}
		else if(eventSource == text_collectingMethod){
			facade.setCollectingMethod(text_collectingMethod.getText());
		}
		else if(eventSource == number_distanceToGround){
			facade.setDistanceToGround(number_distanceToGround.getInteger());
		}
		else if(eventSource == number_distanceToWaterSurface){
			facade.setDistanceToWaterSurface(number_distanceToWaterSurface.getInteger());
		}
	//	else if(eventSource == xxx_exactLocation){
	//		facade.setExactLocation(xxx_ecactLocation.getPoint());
	//	}
		else if(eventSource == text_gatheringEventDescription){
			facade.setGatheringEventDescription(text_gatheringEventDescription.getText());
		}
		else if(eventSource == composite_gatheringPeriod){
	//		facade.setGatheringPeriod(composite_gatheringPeriod.getTimePeriod);
		}
		else if(eventSource == text_localityText || eventSource == combo_localityLanguage){
			facade.setLocality(LanguageString.NewInstance(text_localityText.getText(), combo_localityLanguage.getSelection()));		
		}	
	}

	public DerivedUnitFacade getFacade() {
		return facade;
	}

	public void setFacade(DerivedUnitFacade facade) {
		this.facade = facade;
		updateContent();
	}
}
