// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.reference;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.TimePeriodElement;
import eu.etaxonomy.taxeditor.forms.ToggleableTextElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.EnumComboType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.selection.AbstractSelectionElement;
import eu.etaxonomy.taxeditor.forms.selection.ReferenceSelectionElement;
import eu.etaxonomy.taxeditor.forms.term.ReferenceTypeComboElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;

/**
 * @author n.hoffmann
 * @created Mar 2, 2010
 * @version 1.0
 */
public class ReferenceBaseDetailElement extends AbstractCdmDetailElement<ReferenceBase> {
	
	private static final Logger logger = Logger
			.getLogger(ReferenceBaseDetailElement.class);
	
	// property holder
	private ToggleableTextElement toggleable_cache;
	private TextWithLabelElement text_edition;
	private TextWithLabelElement text_editor;
	private TextWithLabelElement text_isbn;
	private TextWithLabelElement text_issn;
	private TextWithLabelElement text_organisation;
	private TextWithLabelElement text_pages;
	private TextWithLabelElement text_placePublished;
	private TextWithLabelElement text_publisher;
	private TextWithLabelElement text_referenceAbstract;
	private TextWithLabelElement text_series;
	private TextWithLabelElement text_seriesPart;
	private TextWithLabelElement text_title;
	private TextWithLabelElement text_volume;
	private TextWithLabelElement text_uri;
	private AbstractSelectionElement<AgentBase> selection_authorTeam;
	private TimePeriodElement composite_timePeriod;
	private AbstractSelectionElement<ReferenceBase> selection_inReference;
	private AbstractSelectionElement<Institution> selection_institution;
	private ReferenceTypeComboElement combo_referenceType;

	private ReferenceSelectionElement selection_inSeries;

	
	public ReferenceBaseDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.AbstractCdmDetailComposite#createControl(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	protected void createControls(ICdmFormElement formElement, ReferenceBase entity, int style) {
		toggleable_cache = formFactory.createToggleableTextField(formElement, "Reference Cache", entity.getTitleCache(), entity.isProtectedTitleCache(), style);
		
		combo_referenceType = (ReferenceTypeComboElement) formFactory.createEnumComboElement(EnumComboType.REFERENCETYPE, formElement, style);

		combo_referenceType.setSelection(entity.getType());
		
		// title
		text_title = formFactory.createTextWithLabelElement(formElement, "Title", entity.getTitle(), style);
		
		// author team
		selection_authorTeam = formFactory.createSelectionElement(SelectionType.TEAM, getConversationHolder(), formElement, "Author Team", entity.getAuthorTeam(), style);
		
		ReferenceType referenceType = entity.getType();
		
		if(referenceType != null && referenceType.equals(ReferenceType.Generic)){
			createGenericControls(this, entity, SWT.NULL);
		}else if(referenceType != null){
			if(referenceType.isPrintedUnit()){
				createPrintedUnitControls(this, referenceType, entity, SWT.NULL);		
			}	
			if(referenceType.isPublication()){
				createPublicationControls(this, referenceType, entity, SWT.NULL);
			}
			if(referenceType.isVolumeReference()){
				createVolumeControls(this, entity, SWT.NULL);
			}	
			if(referenceType.isSection()){
				createSectionControls(this, entity, SWT.NULL);
			}		
		}
		
		// date published
		composite_timePeriod = formFactory.createTimePeriodElement(formElement, "Date Published", entity.getDatePublished(), style);
		
		createUriAndAbstract(this, entity, SWT.NULL);
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.cdmdetail.AbstractCdmDetailComposite#updateContent()
	 */
	@Override
	protected void updateContent() {
		if(getEntity() == null){
			setEntity(Generic.NewInstance());
		}
		
		super.updateContent();
		
		// enable elements according to state of cache protection
//		setEnabled(getEntity().isProtectedTitleCache(), Arrays.asList(new Object[]{text_cache}));
		toggleable_cache.setEnabled(getEntity().isProtectedTitleCache());
		setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
	}
	
	/**
	 * Creates additional controls to edit a generic reference 
	 * 
	 * @param element
	 * @param reference 
	 * @param listener
	 * @param style
	 */
	private void createGenericControls(ICdmFormElement element, ReferenceBase reference, int style){
		//editor
		text_editor = formFactory.createTextWithLabelElement(element, "Editor", reference.getEditor(), style);
		
		// series
		text_series = formFactory.createTextWithLabelElement(element, "Series", reference.getSeries(), style);
		
		// volume 
		text_volume = formFactory.createTextWithLabelElement(element, "Volume", reference.getVolume(), style);
		
		// pages
		text_pages = formFactory.createTextWithLabelElement(element, "Pages", reference.getPages(), style);
	}
	
	
	/**
	 * Creates addition controls to edit a publication
	 * 
	 * @param element
	 * @param reference 
	 * @param listener
	 * @param style
	 */
	private void createPublicationControls(ICdmFormElement element, ReferenceType referenceType, ReferenceBase reference, int style){

		// placePublished
		text_placePublished = formFactory.createTextWithLabelElement(element, "Place Published", reference.getPlacePublished(), style);
		
		// publisher
		text_publisher = formFactory.createTextWithLabelElement(element, "Publisher", reference.getPublisher(), style);
		
		if(referenceType.equals(ReferenceType.Journal)){
			//issn
			text_issn = formFactory.createTextWithLabelElement(element, "ISSN", reference.getIssn(), style);
		}
		
		if(referenceType.equals(ReferenceType.Report)){
			// institution
			selection_institution = formFactory.createSelectionElement(SelectionType.INSTITUTION, getConversationHolder(), element, "Institution", reference.getInstitution(), style);
		}
		
		if(referenceType.equals(ReferenceType.Thesis)){
			// school
			selection_institution = formFactory.createSelectionElement(SelectionType.INSTITUTION, getConversationHolder(), element, "School", reference.getSchool(), style);
		}
		
		if(referenceType.equals(ReferenceType.Proceedings)){
			//organization
			text_organisation = formFactory.createTextWithLabelElement(element, "Organisation", reference.getOrganization(), style);
		}
	}
	
	
	/**
	 * 
	 * @param element
	 * @param reference 
	 * @param style
	 */
	private void createSectionControls(ICdmFormElement element, ReferenceBase reference, int style){
		
		// in reference
		if(reference.getType() == ReferenceType.BookSection || reference.getType() == ReferenceType.InProceedings){
			String label = "";
			if(reference.getType() == ReferenceType.BookSection){
				label = "Book";
			}else if(reference.getType() == ReferenceType.InProceedings){
				label = "Print Series";
			}
			
			selection_inReference = formFactory.createSelectionElement(SelectionType.REFERENCE, getConversationHolder(), element, label, reference.getInReference(), style);
		}
		
		// pages
		text_pages = formFactory.createTextWithLabelElement(element, "Pages", reference.getPages(), style);
	}
	
	/**
	 * 
	 * @param element
	 * @param reference 
	 * @param style
	 */
	private void createVolumeControls(ICdmFormElement element, ReferenceBase reference, int style){
		if(reference.getType() == ReferenceType.Article){
			selection_inReference = formFactory.createSelectionElement(SelectionType.REFERENCE, getConversationHolder(), element, "Journal", reference.getInReference(), style);
		}
		
		text_volume = formFactory.createTextWithLabelElement(element, "Volume", reference.getVolume(), style);
		
		text_series = formFactory.createTextWithLabelElement(element, "Series", reference.getVolume(), style);
	}
	
	
	/**
	 * 
	 * @param element
	 * @param referenceType
	 * @param reference 
	 * @param style
	 */
	private void createPrintedUnitControls(ICdmFormElement element, ReferenceType referenceType, ReferenceBase reference, int style){
		// in series
		selection_inSeries = (ReferenceSelectionElement) formFactory.createSelectionElement(SelectionType.REFERENCE, getConversationHolder(), element, "In Series", (ReferenceBase) reference.getInSeries(), style);
		
		//editor
		text_editor = formFactory.createTextWithLabelElement(element, "Editor", reference.getEditor(), style);
				
		// series part
		text_seriesPart = formFactory.createTextWithLabelElement(element, "Series Part", reference.getSeriesPart(), style);
		
		if(referenceType.equals(ReferenceType.Book)){
			// edition
			text_edition = formFactory.createTextWithLabelElement(element, "Edition", reference.getEdition(), style);
			
			//isbn
			text_isbn = formFactory.createTextWithLabelElement(element, "ISBN", reference.getIsbn(), style);
		}
	}
	
	private void createUriAndAbstract(ICdmFormElement element,
			ReferenceBase reference, int style) {
		// uri
		text_uri = formFactory.createTextWithLabelElement(element, "URI", reference.getUri(), style);
		// abstract
		text_referenceAbstract = formFactory.createMultilineTextWithLabel(element, "Reference Abstract", 100, style);
		text_referenceAbstract.setText(reference.getReferenceAbstract());
	}
		
	
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == combo_referenceType){
			getEntity().setType(combo_referenceType.getSelection());
			getParentElement().refresh();
		}
		else if(eventSource == composite_timePeriod){
			// TODO
		}
		// selections
		else if(eventSource == selection_authorTeam){
			getEntity().setAuthorTeam((TeamOrPersonBase)selection_authorTeam.getSelection());
		}
		else if(eventSource == selection_inReference){
			getEntity().setInReference(selection_inReference.getSelection());
		}
		else if(eventSource == selection_institution){
			getEntity().setInstitution(selection_institution.getSelection());
		}
		
		// textfields
		else if(eventSource == toggleable_cache){
			getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
//			setEnabled(! text_cache.getState(), Arrays.asList(new Object[]{text_cache}));
			setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
		}
		else if(eventSource == text_edition){
			getEntity().setEdition(text_edition.getText());
		}
		else if(eventSource == text_editor){
			getEntity().setEditor(text_editor.getText());
		}
		else if(eventSource == text_isbn){
			getEntity().setIsbn(text_isbn.getText());
		}
		else if(eventSource == text_issn){
			getEntity().setIssn(text_issn.getText());
		}
		else if(eventSource == text_organisation){
			getEntity().setOrganization(text_organisation.getText());
		}
		else if(eventSource == text_pages){
			getEntity().setPages(text_pages.getText());
		}
		else if(eventSource == text_placePublished){
			getEntity().setPlacePublished(text_placePublished.getText());
		}
		else if(eventSource == text_publisher){
			getEntity().setPublisher(text_publisher.getText());
		}
		else if(eventSource == text_referenceAbstract){
			getEntity().setReferenceAbstract(text_referenceAbstract.getText());
		}
		else if(eventSource == text_series){
			getEntity().setSeries(text_series.getText());
		}
		else if(eventSource == text_seriesPart){
			getEntity().setSeriesPart(text_seriesPart.getText());
		}
		else if(eventSource == text_title){
			getEntity().setTitle(text_title.getText());
		}
		else if(eventSource == text_uri){
			getEntity().setUri(text_uri.getText());
		}
		else if(eventSource == text_volume){
			getEntity().setVolume(text_volume.getText());
		}
		
		else if(eventSource != toggleable_cache){
			toggleable_cache.setText(getEntity().getTitleCache());
		}
	}
}
