// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.supplemental;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.common.ReferencedEntityBase;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.selection.ReferenceSelectionElement;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.section.BulkEditorOpeningSelectionListener;

/**
 * @author n.hoffmann
 * @created Mar 25, 2010
 * @version 1.0
 */
public abstract class AbstractReferencedEntityElement<T extends ReferencedEntityBase> extends
		AbstractEntityCollectionElement<T> {


	private static final Logger logger = Logger
			.getLogger(AbstractReferencedEntityElement.class);
	
	
	protected ReferenceSelectionElement selection_reference;
	protected TextWithLabelElement text_referenceDetail;


	protected TextWithLabelElement text_originaleNameString;


	private boolean hasOriginalNameString = false;

	public AbstractReferencedEntityElement(CdmFormFactory formFactory,
			AbstractFormSection section, T entity,
			SelectionListener removeListener,
			int style) {
		super(formFactory, section, entity, removeListener, null, style);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.AbstractEntityCollectionElement#createControls(eu.etaxonomy.taxeditor.forms.ICdmFormElement, int)
	 */
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		selection_reference = (ReferenceSelectionElement) formFactory.createSelectionElement(SelectionType.REFERENCE, getConversationHolder(), formElement, "Reference", null, style);
		text_referenceDetail = formFactory.createTextWithLabelElement(formElement, "Reference Detail", null, SWT.NULL);
		if(hasOriginalNameString) text_originaleNameString = formFactory.createTextWithLabelElement(formElement, "Original Name", null, SWT.NULL);
	}
	
	public void setEntity(T entity) {
		this.entity = entity;
		
		selection_reference.setEntity(entity.getCitation());
		text_referenceDetail.setText(entity.getCitationMicroReference());
		if(hasOriginalNameString) text_originaleNameString.setText(entity.getOriginalNameString());
//		// remove old listeners
//		clearEditButtonSelectionListener();
//		
//		addEditButtonSelectionListener(new BulkEditorOpeningSelectionListener(entity.getCitation()));
//		
	};
//	
//	public void addEditButtonSelectionListener(SelectionListener listener){
//		selection_reference.addEditButtonSelectionListener(listener);
//	}
//	
//	public void removeEditButtonSelectionListener(SelectionListener listener){
//		selection_reference.removeEditorButtonSelectionListener(listener);
//	}
//	
//	public void clearEditButtonSelectionListener(){
//		selection_reference.clearEditButtonSelectionListener();
//	}

	public void setHasOriginalNameString(boolean hasOriginalNameString) {
		this.hasOriginalNameString = hasOriginalNameString;
	}

	public boolean hasOriginalNameString() {
		return hasOriginalNameString;
	}
}
