// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.supplemental;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.common.mediaMetaData.ImageMetaData;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.ImageElement;
import eu.etaxonomy.taxeditor.forms.IntegerTextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.KeyValueViewerElement;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 26, 2010
 * @version 1.0
 */
public class ImageFileElement extends AbstractMediaRepresentationPartElement<ImageFile> {


	private static final Logger logger = Logger
			.getLogger(ImageFileElement.class);
	private IntegerTextWithLabelElement text_height;
	private IntegerTextWithLabelElement text_width;
	private ImageElement element_image;
	private KeyValueViewerElement element_keyValue;
	
	public ImageFileElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section, ImageFile element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, section, element, removeListener, style);
	}
	
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		super.createControls(formElement, style);
		
		text_height = formFactory.createIntegerTextWithLabelElement(formElement, "Height", null, style);
		text_height.setEnabled(false);
		text_width = formFactory.createIntegerTextWithLabelElement(formElement, "Width", null, style);
		text_width.setEnabled(false);
		element_image = formFactory.createImageElement(formElement, null, style);
		
		element_keyValue = formFactory.createKeyValueViewerElement(formElement, "Key", "Value", null);
	}
	
	@Override
	public void setEntity(ImageFile entity) {
		this.entity = entity;
		text_uri.setText(entity.getUri());
		if(entity.getSize() != null){
			text_size.setText(FileUtils.byteCountToDisplaySize(entity.getSize()));
		}
		text_height.setInteger(entity.getHeight());
		text_width.setInteger(entity.getWidth());
		
		showImage(entity.getUri());
	}
	
	private void showImage(String uriString){
		if(uriString == null){
			return;
		}
		try {
			URI imageUri = new URI(uriString);
			
			Integer size = CdmStore.getMediaService().getImageSize(imageUri, 10000);
			
			if(size != null){
				text_size.setText(FileUtils.byteCountToDisplaySize(size));
				getEntity().setSize(size);
			}
			
			
			ImageMetaData imageInfo;
			try {
				
				imageInfo = CdmStore.getMediaService().getImageInfo(imageUri, 10000);
					
				text_height.setInteger(imageInfo.getHeight());
				getEntity().setHeight(imageInfo.getHeight());
					
				text_width.setInteger(imageInfo.getWidth());
				getEntity().setWidth(imageInfo.getWidth());
				element_keyValue.setInput(imageInfo.getMetaData());
				
				element_image.setImageUri(imageUri);
				
				
			} catch (IOException e) {
				logger.error("IOError reading image metadata", e);
				throw new RuntimeException("IOError reading image metadata");
			}
						
			
		} catch (URISyntaxException e) {
			logger.warn("Image Uri is not valid");
		} catch (Exception e) {
			logger.warn("Image could not be shown: " + e.getMessage());
//			new RuntimeException(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_uri){
			getEntity().setUri(text_uri.getText());
			showImage(getEntity().getUri());
		}
	}
}
