/**
 * 
 */
package eu.etaxonomy.taxeditor.section.supplemental;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.EntityDetailType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.selection.AgentSelectionElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * 
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 */
public class MediaElement extends AbstractEntityCollectionElement<Media> {

	private MediaRepresentationSection section_mediaRepresentation;
	private AgentSelectionElement selection_artist;
	private LanguageStringWithLabelElement text_description;
	private LanguageStringWithLabelElement text_title;

	/**
	 * @param formElement
	 * @param element
	 * @param removeListener
	 * @param propertyChangelistener
	 * @param style
	 */
	public MediaElement(CdmFormFactory cdmFormFactory, AbstractFormSection formElement, Media element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, null, style);
	}
	
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		text_title = formFactory.createLanguageStringWithLabelElement(element, "Title", null, style);
		selection_artist = (AgentSelectionElement) formFactory.createSelectionElement(SelectionType.AGENT, getConversationHolder(), element, "Artist", null, style);
		text_description = formFactory.createLanguageStringWithLabelElement(element, "Description",  null, 100, style);
		section_mediaRepresentation = (MediaRepresentationSection) formFactory.createEntityDetailSection(EntityDetailType.MEDIAREPRESENTATION, getConversationHolder(), element, style);
		section_mediaRepresentation.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
	}
	
	@Override
	public void setEntity(Media element) {
		this.entity = element;
		section_mediaRepresentation.setEntity(element);
		selection_artist.setEntity((TeamOrPersonBase)HibernateProxyHelper.deproxy(element.getArtist()));
		
		text_description.setLanguageString(element.getDescription(CdmStore.getDefaultLanguage()), CdmStore.getDefaultLanguage());
		
		LanguageString title = element.getTitle(CdmStore.getDefaultLanguage());
		text_title.setLanguageString(title, CdmStore.getDefaultLanguage());
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == selection_artist){
			getEntity().setArtist(selection_artist.getSelection());
		}
		else if(eventSource == text_description){
			getEntity().addDescription(text_description.getLanguageString());
		}
		else if(eventSource == text_title){
			getEntity().addTitle(text_title.getLanguageString());
			if(getParentElement() instanceof AbstractCdmDetailSection){
				((AbstractCdmDetailSection) getParentElement()).updateTitle();
			}
		}	
	}
}
