// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.supplemental;

import org.apache.log4j.Logger;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.EntityDetailType;
import eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement;

/**
 * @author n.hoffmann
 * @created Mar 24, 2010
 * @version 1.0
 */
public class MediaRepresentationElement extends AbstractEntityCollectionElement<MediaRepresentation> {

	private static final Logger logger = Logger
			.getLogger(MediaRepresentationElement.class);
	private MediaRepresentationPartSection section_mediaRepresentationPart;
	private TextWithLabelElement text_mimeType;
	private TextWithLabelElement text_suffix;

	public MediaRepresentationElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section,
			MediaRepresentation element, SelectionListener removeListener,
			int style) {
		super(cdmFormFactory, section, element, removeListener, null, style);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.AbstractEntityCollectionElement#createControls(eu.etaxonomy.taxeditor.forms.ICdmFormElement, int)
	 */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		text_mimeType = formFactory.createTextWithLabelElement(element, "Mime Type", null, style);
		text_suffix = formFactory.createTextWithLabelElement(element, "Suffix", null, style);
		section_mediaRepresentationPart = (MediaRepresentationPartSection) formFactory.createEntityDetailSection(EntityDetailType.MEDIAREPRESENTATIONPART, getConversationHolder(), element, style);
		section_mediaRepresentationPart.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY(2, 1));
	}

	@Override
	public void setEntity(MediaRepresentation element) {
		this.entity = element;
		section_mediaRepresentationPart.setEntity(element);
		text_mimeType.setText(element.getMimeType());
		text_suffix.setText(element.getSuffix());
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_mimeType){
			getEntity().setMimeType(text_mimeType.getText());
		}
		else if(eventSource == text_suffix){
			getEntity().setSuffix(text_suffix.getText());
		}
	}
}
