// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.supplemental;

import org.apache.log4j.Logger;
import org.eclipse.swt.widgets.Label;
import org.joda.time.DateTime;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.IEntityElement;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class VersionElement extends AbstractCdmFormElement implements IEntityElement<VersionableEntity> {

	private static final Logger logger = Logger.getLogger(VersionElement.class);
	private Label label_created;
	private Label label_updated;
	
	private Label label_uuid;
	private Label label_objectId;
	private VersionableEntity entity;
	
	/**
	 * @param parent
	 * @param listener
	 * @param style
	 */
	public VersionElement(CdmFormFactory toolkit, ICdmFormElement parentElement, VersionableEntity entity, int style) {
		super(toolkit, parentElement);
		
		getLayoutComposite().setLayout(CdmFormFactory.LAYOUT());
		
		label_created = toolkit.createLabel(getLayoutComposite(), null);
		label_created.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());

		label_updated = toolkit.createLabel(getLayoutComposite(), null);
		label_updated.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		
		label_uuid = toolkit.createLabel(getLayoutComposite(), null);
		label_uuid.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
		
		label_objectId = toolkit.createLabel(getLayoutComposite(), null);
		label_objectId.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
	}
	
	public void setEntity(VersionableEntity entity){

		this.entity = entity;
		
		String createdString = "Created "
				+ dateFormat(entity.getCreated())
				+ " by " + userFormat(entity.getCreatedBy());			
		label_created.setText(createdString);

		String updatedString = "Updated "
				+ dateFormat(entity.getUpdated())
				+ " by " + userFormat(entity.getUpdatedBy());
		label_updated.setText(updatedString);
		
		boolean showDebug = PreferencesUtil.getPreferenceStore().getBoolean(PreferencesUtil.SHOW_DEBUG_INFORMATION);
		
		if(showDebug){
			String uuidString = "UUID: " + entity.getUuid();
			label_uuid.setText(uuidString);
			label_uuid.setVisible(true);
			
			String objectIdString = "Object ID: " + entity.getId();
			label_objectId.setText(objectIdString);
			label_objectId.setVisible(true);
		}else{
			label_uuid.setVisible(false);
			label_objectId.setVisible(false);
		}
		
	}
	
	private String dateFormat(DateTime dateTime){
		if(dateTime == null){
			return "";
		}
		return CdmUtils.Nz(dateTime.toString(PreferencesUtil.getDateFormatPattern()));
	}
	
	private String userFormat(User user){
		if(user == null){
			return "";
		}
		// TODO agree on what to display username or real name or even more from shibboleth
		return CdmUtils.Nz(user.getUsername());
	}

	public VersionableEntity getEntity() {
		return entity;
	}

	public void setSelected(boolean selected) {
		// will not likely be selected
	}
}
