// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.taxon;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.strategy.parser.ParserProblem;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.section.ITaxonDetailSection;

/**
 * @author n.hoffmann
 * @created Mar 3, 2010
 * @version 1.0
 */
public class ParsingMessagesSection extends AbstractCdmDetailSection<NonViralName> implements ITaxonDetailSection {

	public static final String HEADING_SUCCESS = "";
	public static final String HEADING_PROBLEM = "The name has problems.";
	
	private static final Logger logger = Logger
			.getLogger(ParsingMessagesSection.class);

	public ParsingMessagesSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
		super(cdmFormFactory, conversation, parentElement, selectionProvider, style);
	}
	
	@Override
	protected void createControls(AbstractCdmDetailSection formElement, int style) {
		// we do it differently here
	}
	
	public void setTaxon(TaxonBase taxon) {
		setEntity((NonViralName) HibernateProxyHelper.deproxy(taxon.getName()));
		destroyParsingProblemComposites();
		if(getEntity().hasProblem()){
			displayParsingProblems();
		}
	}
	
	private void destroyParsingProblemComposites() {
		setText(HEADING_SUCCESS);
		for(Control control : getLayoutComposite().getChildren()){
			control.dispose();
			control = null;
		}
	}

	private void displayParsingProblems() {
		setText(HEADING_PROBLEM);
		for(Object parserProblem : getEntity().getParsingProblems()){
			formFactory.createParsingMessageElement(this, (ParserProblem) parserProblem, SWT.NULL);
		}
		this.layout();
	}

	@Override
	public String getHeading() {
		return HEADING_SUCCESS;
	}

	@Override
	protected DetailType getDetailType() {
		return DetailType.PARSINGMESSAGE;
	}
}
