// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.section.taxon;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CheckboxElement;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.forms.TextWithLabelElement;
import eu.etaxonomy.taxeditor.forms.ToggleableTextElement;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.SelectionType;
import eu.etaxonomy.taxeditor.forms.selection.ReferenceSelectionElement;
import eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement;

/**
 * @author n.hoffmann
 * @created Feb 26, 2010
 * @version 1.0
 */
public class TaxonBaseDetailElement extends AbstractCdmDetailElement<TaxonBase>{

	private CheckboxElement checkbox_doubtful;

	private CheckboxElement checkbox_useNameCache;

	private ReferenceSelectionElement selection_secundum;

	private TextWithLabelElement text_appendedPhrase;

	private ToggleableTextElement toggleable_cache;


	private static final Logger logger = Logger
			.getLogger(TaxonBaseDetailElement.class);
	
	public TaxonBaseDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}
	
	@Override
	protected void createControls(ICdmFormElement formElement, TaxonBase entity, int style){
		toggleable_cache = formFactory.createToggleableTextField(formElement, "Title Cache", entity.getTitleCache(), entity.isProtectedTitleCache(), style);
		
		checkbox_doubtful = formFactory.createCheckbox(formElement, "Is doubtful", entity.isDoubtful(), style);
		checkbox_useNameCache = formFactory.createCheckbox(formElement, "Exclude Authorship", entity.isUseNameCache(), style);
		selection_secundum = (ReferenceSelectionElement) formFactory.createSelectionElement(SelectionType.REFERENCE, getConversationHolder(), formElement, "Secundum", entity.getSec(), style);
		text_appendedPhrase = formFactory.createTextWithLabelElement(formElement, "Appended Phrase", entity.getAppendedPhrase(), SWT.NULL);
	}
	
	@Override
	protected void updateContent() {
		super.updateContent();
		toggleable_cache.setEnabled(getEntity().isProtectedTitleCache());
		setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
	}
	
	
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource.equals(toggleable_cache)){
			getEntity().setTitleCache(toggleable_cache.getText(), toggleable_cache.getState());
//			setEnabled(! toggleable_titleCache.getState(), Arrays.asList(new Object[]{toggleable_titleCache}));
			setIrrelevant(toggleable_cache.getState(), Arrays.asList(new Object[]{toggleable_cache}));
		}
		if(eventSource.equals(checkbox_doubtful)){
			getEntity().setDoubtful(checkbox_doubtful.getSelection());
		}
		if(eventSource.equals(checkbox_useNameCache)){
			getEntity().setUseNameCache(checkbox_useNameCache.getSelection());
		}
		if(eventSource.equals(selection_secundum)){
			getEntity().setSec(selection_secundum.getSelection());
		}
		if(eventSource.equals(text_appendedPhrase)){
			getEntity().setAppendedPhrase(text_appendedPhrase.getText());
		}
		
		if(eventSource != toggleable_cache){
			toggleable_cache.setText(getEntity().getTitleCache());
		}
	}
}
