/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation;

import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.NotEnabledException;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.UndoContext;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.navigator.CommonViewer;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.navigation.internal.TaxeditorNavigationPlugin;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.section.BulkEditorOpeningSelectionListener;

/**
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public class NavigationUtil extends AbstractUtility{
	private static final Logger logger = Logger.getLogger(NavigationUtil.class);
	private static IUndoContext defaultUndoContext;
	
	/**
	 * FIXME Introduces a dependency on eu.etaxonomy.taxeditor.editor. This is highly undesirable.
	 * 
	 * It is not clear at the moment by what mechanics we are going to open the 
	 * editor. I would prefer some sort of resource model that has the editor linked 
	 * to the resource type of taxonbase. Another reason to switch to CommonNavigatorFamework
	 * 
	 * @param uuid
	 */
//	public static void openEditor(TaxonNode selection) {
//		try {
//			UUID entityUuid = selection.getUuid();			
//			EditorUtil.openTaxonNode(entityUuid);
//		} catch (PartInitException e) {
//			logger.error("Error opening the editor", e);
//		}
//	}
	
	public static void executeEditHandler(){

		String commandId = "eu.etaxonomy.taxeditor.navigation.command.editSelection";
		
		IHandlerService handlerService = (IHandlerService) NavigationUtil.getService(IHandlerService.class); 
		try {
			handlerService.executeCommand(commandId, null);
		} catch (ExecutionException e) {
			logger.error("ExecutionException when trying to execute EditHandler.", e);
		} catch (NotDefinedException e) {
			logger.error("NotDefinedException when trying to execute EditHandler.", e);
		} catch (NotEnabledException e) {
			logger.error("NotEnabledException when trying to execute EditHandler.", e);
		} catch (NotHandledException e) {
			logger.error("NotHandledException when trying to execute EditHandler.", e);
		}
	}
	
	public static void openEditor(CdmBase selectedObject){
		UUID entityUuid = selectedObject.getUuid();	
		try {	
			if(selectedObject instanceof TaxonNode){
				EditorUtil.openTaxonNode(entityUuid);
			}else if(selectedObject instanceof TaxonBase){
				EditorUtil.openTaxonBase(entityUuid);
			}else if(selectedObject instanceof TaxonNameBase){
				// TODO open bulk editor
				warningDialog("Not implemented yet", "You tried to open a name. This is not hanlded by the software yet.");
			}else{
				warningDialog("Unsupported Type", "No editor exists for the current selection: " + selectedObject);
			}
		} catch (PartInitException e) {
			logger.error("Error opening the editor", e);
		}
	}
	
	public static Shell getShell() {
		return getActiveWindow().getShell();
	}

	public static IWorkbenchWindow getActiveWindow() {
		return TaxeditorNavigationPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow();
	}
	
	public static void openEmpty(UUID parentNodeUuid) {
		try {
			EditorUtil.openEmpty(parentNodeUuid);
		} catch (PartInitException e) {
			logger.error("Error opening the editor", e);
		}
	}
	
	public static IUndoContext getWorkbenchUndoContext() {
		return TaxeditorEditorPlugin.getDefault().getWorkbench().
					getOperationSupport().getUndoContext();
	}

	public static IUndoContext getUndoContext() {
		// FIXME this has to be more specific. Every widget has to have its own undo context
//		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
		
		// Plug-ins that wish their operations to be undoable from workbench views 
		// such as the Navigator or Package Explorer should assign the workbench 
		// undo context to their operations. 
		if (defaultUndoContext == null) {
			defaultUndoContext = new UndoContext();
		}
		return defaultUndoContext;
	}

	/**
	 * Whether a taxonNode has unsaved changes.
	 * 
	 * @param taxonNode
	 * @return
	 */
	public static boolean isDirty(TaxonNode taxonNode){
		
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			
			try {
				if (reference.getEditorInput() instanceof TaxonEditorInput) {
					TaxonEditorInput editorInput = (TaxonEditorInput) reference.getEditorInput();
					if(editorInput.getTaxonNode().equals(taxonNode) && reference.isDirty()){
						return true;
					}
				}
			} catch (PartInitException e) {
				logger.error(e);
				throw new RuntimeException(e);
			}
			
		}
		return false;
	}

	/**
	 * 
	 * @param element
	 * @param parentElement
	 */
	public static void selectInNavigator(final Object element, final Object parentElement) {
		Display.getDefault().asyncExec(new Runnable(){

			public void run() {
				TaxonNavigator navigator = showNavigator();

				if (navigator != null) {
					CommonViewer viewer = navigator.getCommonViewer();
					if (viewer != null) {
						if (parentElement != null) {
							viewer.setExpandedState(parentElement, true);
						}
						viewer.setSelection(new StructuredSelection((TaxonNode) element));
					}
				}
			}
			
		});
	}

	/**
	 * @param selection
	 */
	public static void openSearch(Object selection) {
		if(selection instanceof Taxon){
			Taxon taxon = (Taxon) selection;
			
			handleOpeningOfMultipleTaxonNodes(taxon.getTaxonNodes());
			
		}else if(selection instanceof Synonym){
			Synonym synonym = (Synonym) selection;
			
			handleOpeningOfMultipleTaxa(synonym.getAcceptedTaxa());
			
		}else{
			warningDialog("Not implemented yet", "You chose to open a name that has no connection to a taxon. The Editor does not support editing of such a content type at the moment.");
			logger.warn("Could not open editor for selection. Selection was of type: " + selection.getClass().getSimpleName());
		}
		
	}

	private static void handleOpeningOfMultipleTaxa(Set<Taxon> acceptedTaxa) {
		if(acceptedTaxa.size() == 1){
			openEditor(acceptedTaxa.iterator().next());
		}else if(acceptedTaxa.size() > 1){
			// FIXME implement a dialog that shows all possible taxa and let the user choose which he wants to open.
			warningDialog("Not implemented yet", "The accepted taxon is in multiple taxonomic trees. We currently do not know which one you want to open." +
					" This case is not handled yet by the software.");
		}else if(acceptedTaxa.size() == 0){
			// this is an undesired state
			warningDialog("Incorrect state", "The accepted taxon is not in a taxonomic view. This should not have happened.");
		}
	}

	/**
	 * @param taxonNodes
	 */
	private static void handleOpeningOfMultipleTaxonNodes(
			Set<TaxonNode> taxonNodes) {
		
		if(taxonNodes.size() == 1){
			openEditor(taxonNodes.iterator().next());
		}else if(taxonNodes.size() > 1){
			// FIXME implement a dialog that shows all possible taxa and let the user choose which he wants to open.
			warningDialog("Not implemented yet", "The accepted taxon is in multiple taxonomic trees. We currently do not know which one you want to open." +
					" This case is not handled yet by the software.");
		}else if(taxonNodes.size() == 0){
			// this is an undesired state
			warningDialog("Incorrect state", "The accepted taxon is not in a taxonomic view. This should not have happened.");
		}
	}

	/**
	 * @return the TaxonNavigator instance if present
	 */
	public static TaxonNavigator showNavigator() {
		return (TaxonNavigator) showView(TaxonNavigator.ID);
	}

	public static TaxonNavigator getNavigator(boolean restore) {
		return (TaxonNavigator) getView(TaxonNavigator.ID, restore);
	}

	public static Set<IEditorPart> getOpenEditors() {
		return EditorUtil.getOpenEditors();
	}
}