package eu.etaxonomy.taxeditor.navigation.handler;
// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/


import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.wizard.ClassificationWizard;

/**
 * @author n.hoffmann
 * @created May 12, 2010
 * @version 1.0
 */
public class EditHandler extends AbstractHandler implements IHandler{
	private static final Logger logger = Logger
			.getLogger(EditHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
	
		ISelection selection = HandlerUtil.getCurrentSelection(event);//navigator.getSelection();
		
		if(selection instanceof StructuredSelection){
			final StructuredSelection structuredSelection = (StructuredSelection) selection;
			
			if(structuredSelection.size() == 1 && structuredSelection.getFirstElement() instanceof TaxonomicTree){
				
				WizardDialog dialog = new WizardDialog(HandlerUtil.getActiveShell(event), new ClassificationWizard((TaxonomicTree) structuredSelection.getFirstElement()));
				dialog.open();
				
			}
			else{

				Job job = new Job("Opening editor") {
					
					@Override
					protected IStatus run(IProgressMonitor monitor) {
						for(final Object selectedObject : structuredSelection.toArray()){
							
							if(selectedObject instanceof CdmBase){
								// let the openEditor() method handle everything from now on
								Display.getDefault().asyncExec(new Runnable(){

									public void run() {
										NavigationUtil.openEditor((CdmBase) selectedObject);
									}
									
								});
							}else{
								throw new IllegalArgumentException("selectedObject is not of type CdmBase");
							}
						}
						return Status.OK_STATUS;
					}
				};
				
				job.schedule();
			}
		}
		
		return null;
	}
}
