// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.name.IterableSynonymyList;

/**
 * @author p.ciardelli
 * @created 02.06.2009
 * @version 1.0
 */
public class SynonymContentProvider implements ITreeContentProvider {
	private static final Logger logger = Logger
			.getLogger(SynonymContentProvider.class);

	private static final Object[] NO_CHILDREN = new Object[0];
	
	public Object[] getChildren(Object parentElement) {
		Object[] children = null;
		if (parentElement instanceof Synonym) {
			children = NO_CHILDREN;
		} else if (parentElement instanceof TaxonNode) {
			List<TaxonBase> list = new ArrayList<TaxonBase>();
			
			Taxon taxon = ((TaxonNode) parentElement).getTaxon();
			
			for (TaxonBase taxonBase : new IterableSynonymyList(taxon)) {
				if (taxonBase instanceof Synonym) {
					list.add(taxonBase);
				}
			}
			children = list.toArray();
		}
		return children != null ? children : NO_CHILDREN;
	}

	public Object getParent(Object element) {
		return null;
	}

	public boolean hasChildren(Object element) {
		return this.getChildren(element).length > 0;
	}

	public Object[] getElements(Object inputElement) {
		return this.getChildren(inputElement);
	}

	public void dispose() {
	}

	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
	}
}
