// $$Id$$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.search;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 * @created 04.05.2009
 * @version 1.0
 */
public class SearchResultLabelProvider extends ColumnLabelProvider implements ILabelProvider {
	
	public String getText(Object element) {
		if(element instanceof TaxonBase<?>){
			return ((TaxonBase<?>) element).getTitleCache();
		}
		return element.toString();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ColumnLabelProvider#getBackground(java.lang.Object)
	 */
	@Override
	public Color getBackground(Object element) {
		return super.getBackground(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ColumnLabelProvider#getFont(java.lang.Object)
	 */
	@Override
	public Font getFont(Object element) {
		if(element instanceof Taxon){
			return NavigationUtil.getFont(Resources.FONT_SEARCH_VIEW_ACCEPTED);
		}else if(element instanceof Synonym){
			return NavigationUtil.getFont(Resources.FONT_SEARCH_VIEW_SYNONYM);
		}else{
			return NavigationUtil.getFont(Resources.FONT_SEARCH_VIEW_OTHER);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ColumnLabelProvider#getForeground(java.lang.Object)
	 */
	@Override
	public Color getForeground(Object element) {
		if(element instanceof Taxon){
			return NavigationUtil.getColorRegistry().get("eu.etaxonomy.taxeditor.preferences.searchResult.fontColorAccepted");
		}else if(element instanceof Synonym){
			return NavigationUtil.getColorRegistry().get("eu.etaxonomy.taxeditor.preferences.searchResult.fontColorSynonym");
		}else{
			return NavigationUtil.getColorRegistry().get("eu.etaxonomy.taxeditor.preferences.searchResult.fontColorDefault");
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ColumnLabelProvider#getImage(java.lang.Object)
	 */
	@Override
	public Image getImage(Object element) {
		if(element instanceof Taxon){
			Taxon taxon = (Taxon) element;
			if(taxon.isMisappliedName()){
				ImageResources.getImage(ImageResources.MISAPPLIED_NAME_ICON);
			}else{
				return ImageResources.getImage(ImageResources.BLACK_SQUARE_ICON);
			}
		}else if(element instanceof Synonym){
//			Synonym synonym = (Synonym) element;
			return ImageResources.getImage(ImageResources.HETEROTYPIC_SYN_ICON);
		}
		return super.getImage(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.CellLabelProvider#getToolTipText(java.lang.Object)
	 * 
	 * does not seem t work right now
	 */
	@Override
	public String getToolTipText(Object element) {
		if(element instanceof TaxonBase){
			if(element instanceof Synonym){
				return "Synonym: " + ((TaxonBase<?>) element).getName().getTitleCache();
			}else{
				return "Taxon: " + ((TaxonBase<?>) element).getName().getTitleCache();
			}
		}
		return "Name: " + element.toString();
	}
	
	
	
}
