// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.SaveTaxonomicTreeOperation;

/**
 * @author n.hoffmann
 * @created 23.06.2009
 * @version 1.0
 */
public class ClassificationWizard extends Wizard implements INewWizard {
	private static final Logger logger = Logger
			.getLogger(ClassificationWizard.class);

	private ClassificationWizardPage page;
		
	private TaxonomicTree classification;
	
	public ClassificationWizard(){
		setWindowTitle("New Classification");
	}
	
	public ClassificationWizard(TaxonomicTree classification){
		setWindowTitle("New Classification");
		this.classification = classification;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		page = new ClassificationWizardPage(classification);
		
		addPage(page);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		TaxonNavigator taxonNavigator;
		taxonNavigator = (TaxonNavigator) NavigationUtil.showView(TaxonNavigator.ID);
		
		AbstractPostOperation operation = new SaveTaxonomicTreeOperation("Save Classification", 
				NavigationUtil.getUndoContext(), page.getClassification(),
				taxonNavigator, taxonNavigator);
		NavigationUtil.executeOperation(operation);

		NavigationUtil.showNavigator().getCommonViewer().refresh();
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		
	}
}
