// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.wizard;


import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredReferenceSelectionDialog;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 23.06.2009
 * @version 1.0
 */
public class ClassificationWizardPage extends WizardPage implements ModifyListener {
	private static final Logger logger = Logger
			.getLogger(ClassificationWizardPage.class);

	private Text text_treeLabel;

	private Text text_reference;

	private Button button_browseReference;

	private UUID referenceUuid;

	private Text text_microReference;

	private TaxonomicTree classification;
	
	protected ClassificationWizardPage(TaxonomicTree classification){
		super("Classification");
		this.setTitle("Classification");
		this.setDescription(classification == null ? "Create a new Classification." : "Edit Classification.");
		
		this.classification = classification;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		
		setPageComplete(false);
		
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		composite.setLayout(gridLayout);
		
		CLabel label_treeLabel = new CLabel(composite, SWT.NULL);
		label_treeLabel.setText("Label:");
		
		text_treeLabel = new Text(composite, SWT.BORDER);
		text_treeLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		text_treeLabel.addModifyListener(this);

		// FIXME this is a rudimentary implementation. Add missing fields.

		CLabel label_reference = new CLabel(composite, SWT.NULL);
		label_reference.setText("Reference:");
		
		text_reference = new Text(composite, SWT.BORDER);
		text_reference.setEnabled(false);
		text_reference.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		button_browseReference = new Button(composite, SWT.PUSH);
		button_browseReference.setText("Browse...");
		button_browseReference.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				logger.warn("Open reference search dialog");
				
				ReferenceBase reference = FilteredReferenceSelectionDialog.selectReference(NavigationUtil.getShell(), null, null);
				
				if (reference == null) {
					return;
				}
				
				referenceUuid = reference.getUuid();
				 
				classification.setReference(reference);
				
				text_reference.setText(reference.getTitleCache());
			}
		});
		
		CLabel label_microReference = new CLabel(composite, SWT.NULL);
		label_microReference.setText("Reference Detail");
		
		text_microReference = new Text(composite, SWT.BORDER);
		text_microReference.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 2, 1));
		
		
		initialize();
		
		setControl(composite);
	}

	private void initialize() {
		if(classification != null){
			text_treeLabel.setText(classification.getTitleCache());
			if(classification.getReference() != null){
				text_reference.setText(classification.getReference().getTitleCache());
			}
			text_microReference.setText(classification.getMicroReference());
		}else{
			classification = TaxonomicTree.NewInstance(null, null, CdmStore.getDefaultLanguage());
		}
	}


	/**
	 * @return the text_treeLabel
	 */
	public String getTreeLabel() {
		return text_treeLabel.getText();
	}

	/**
	 * @return
	 */
	public String getMicroReference() {
		return text_microReference.getText();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		if(getTreeLabel().trim().length() > 0){
			setPageComplete(true);
		}
		
		classification.setName(LanguageString.NewInstance(text_treeLabel.getText(), CdmStore.getDefaultLanguage()));
		classification.setMicroReference(text_microReference.getText());
	}

	/**
	 * @return
	 */
	public UUID getReferenceUuid() {
		return referenceUuid;
	}


	public TaxonomicTree getClassification() {
		return classification;
	}
}
