/**
 * 
 */
package eu.etaxonomy.taxeditor.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.CreateTaxonNodeOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * 
 * @author n.hoffmann
 * @created Sep 15, 2009
 * @version 1.0
 */
public class NewTaxonNodeWizard extends Wizard implements INewWizard, IPostOperationEnabled {
	private static final Logger logger = Logger.getLogger(NewTaxonNodeWizard.class);

	private NewTaxonNodeWizardPage page;

	private IStructuredSelection selection;

	private TaxonNode generatedTaxonNode;

	private boolean openEmptyEditor = false;
	
	public NewTaxonNodeWizard() {
		this.setWindowTitle("New Taxon");
		selection = getNavigatorSelection();
	}

	/**
	 * Retrieve the current selection from taxon navigator
	 * 
	 * @return
	 */
	private IStructuredSelection getNavigatorSelection() {
		StructuredSelection selection = null;
		TaxonNavigator navigator = NavigationUtil.getNavigator(false);
		
		if(navigator != null){
			selection = (StructuredSelection) navigator.getCommonViewer().getSelection();
		}
		
		return selection;
	}



	@Override
	public boolean performFinish() {
		TaxonNavigator taxonNavigator;
		taxonNavigator = (TaxonNavigator) NavigationUtil.showView(TaxonNavigator.ID);
		
		if(page.getTaxonName() == null || page.getTaxonName().getFullTitleCache().length() == 0){
			openEmptyEditor = true;
		}else{			
			AbstractPostOperation operation = new CreateTaxonNodeOperation("Create Taxon", 
					NavigationUtil.getUndoContext(), page.getParentNode(), page.getTaxonName(), this, taxonNavigator);
			NavigationUtil.executeOperation(operation);
	
//			NavigationUtil.getNavigator().getCommonViewer().refresh();
		}
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		page = new NewTaxonNodeWizardPage(selection);
		
		addPage(page);
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		// workbench is not used at the moment
		this.selection = selection;
	}

	public boolean openInEditor(){
		return page.openInEditor();
	}
	
	public boolean openEmpty(){
		return openInEditor() && openEmptyEditor;
	}

	public TaxonNode getTaxonNode(){
		return generatedTaxonNode;
	}
	
	public ITreeNode getParent(){
		return page.getParentNode();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		generatedTaxonNode = (TaxonNode) objectAffectedByOperation;
		
		NavigationUtil.selectInNavigator(generatedTaxonNode, generatedTaxonNode.getParent());
		
		return true;
	}

	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}
