// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource.view;

import java.util.Map;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.CdmMetaData;
import eu.etaxonomy.cdm.model.common.CdmMetaData.MetaDataPropertyName;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 14.04.2009
 * @version 1.0
 */
public class CdmDataSourceLabelProvider extends LabelProvider implements ITableLabelProvider {

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
	 */
	public Image getColumnImage(Object element, int columnIndex) {
		// No image at the moment.
		// TODO display image in colum 0 if the datasource is connected
		
		
		if(columnIndex == 0){
			ICdmDataSource dataSource = (ICdmDataSource) element;
			
			String key = isCurrentDataSource(dataSource) ? ImageResources.IMG_DATASOURCE_CONNECTED : ImageResources.IMG_DATASOURCE_DISCONNECTED;
			
			return ImageResources.getImage(key);
		}
		
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
	 */
	public String getColumnText(Object element, int columnIndex) {
		ICdmDataSource dataSource = (ICdmDataSource) element;
		
		Map<MetaDataPropertyName, CdmMetaData> metaData = null;
		
		if(isCurrentDataSource(dataSource)){
			metaData = CdmStore.getCommonService().getCdmMetaData();
		}
		
		switch (columnIndex){
			case 1:
				return dataSource.getName();
			case 2:
				if(dataSource.getServer() != null){
					return dataSource.getServer();
				}
				// h2 databases do not have server entries
				return "local";
			case 3:
				return dataSource.getDatabaseType().getName();
			case 4:
				return dataSource.getDatabase();
			case 5:
				if (dataSource.getNomenclaturalCode() == null) {
					return "";
				}
				return dataSource.getNomenclaturalCode().name();
			case 6:
				if(metaData != null){
					CdmMetaData createDate = metaData.get(MetaDataPropertyName.DB_CREATE_DATE);
					if(createDate != null){
						return createDate.getValue(); 
					}
				}
			case 7:
				if(metaData != null){
					CdmMetaData schemaVersion = metaData.get(MetaDataPropertyName.DB_SCHEMA_VERSION);
					if(schemaVersion != null){
						return schemaVersion.getValue();
					}
				}
			case 8:
				if(metaData != null){
					CdmMetaData createNote = metaData.get(MetaDataPropertyName.DB_CREATE_NOTE);
					if(createNote != null){
						return createNote.getValue(); 
					}
				}
		}
		return "-";
	}
	
	private boolean isCurrentDataSource(ICdmDataSource dataSource){
		ICdmDataSource currentDataSource = CdmStore.getDataSource();
		
		return currentDataSource != null && currentDataSource.getName().equals(dataSource.getName());
	}
}
