// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource.view;

import java.util.List;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @created 14.04.2009
 * @version 1.0
 */
public class CdmDataSourceView extends ViewPart implements IContextListener{
	
	public static String ID = "eu.etaxonomy.taxeditor.store.datasource.datasourceView";
	
	private TableViewer tableViewer;
	
	public CdmDataSourceView(){
	}
	
	@Override
	public void createPartControl(Composite parent) {
		CdmStore.addContextListener(this);
		
		// Create top composite
		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;	
		parent.setLayout(fillLayout);
		
		List<ICdmDataSource> datasources = CdmDataSourceRepository.getAll();
		
		tableViewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
												| SWT.V_SCROLL | SWT.FULL_SELECTION);
		getSite().setSelectionProvider(tableViewer);
		
		createColumns(tableViewer);
		
		tableViewer.setContentProvider(new CdmDataSourceContentProvider());
		tableViewer.setLabelProvider(new CdmDataSourceLabelProvider());
		tableViewer.setComparator(new ViewerComparator());
		
		tableViewer.setInput(datasources);

		
		// register context menu
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, tableViewer);

		Control control = tableViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);	
		
		// connect on doubleclick
		tableViewer.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				if (event.getSelection() instanceof StructuredSelection) {
					
					Object element = ((StructuredSelection) event
							.getSelection()).getFirstElement();
					if (element instanceof ICdmDataSource) {
						// FIXME this is duplicated code from ChangeConnectionHandler. Replace this with a call to HandlerService. 
						ICdmDataSource dataSource = (ICdmDataSource) element;
						
						boolean confirmed = MessageDialog.openConfirm(StoreUtil.getShell(), "Confirm Datasource Change", "Do you really want to connect to this datasource?\n\n"
								+ dataSource.getName());
						
						if(confirmed){
							CdmDataSourceRepository.changeDataSource(dataSource);
						}
					}
				}
			}
		});
		
	}

	
	// This will create the columns for the table
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = {"Connected", "Name", "Server", "Type", "Database", "Nomenclatural Code", "Created", "CDM Version", "Notes"};
		int[] bounds = { 20, 100, 100, 100 , 100, 150, 100, 100, 100};

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}
	
	public void refresh(){
		tableViewer.setInput(CdmDataSourceRepository.getAll());
		tableViewer.refresh();	
	}

	@Override
	public void dispose() {
		CdmStore.removeContextListener(this);
		super.dispose();
	}

	@Override
	public void setFocus() {
		tableViewer.getControl().setFocus();
	}

	public boolean onContextEvent(EventType eventType) {
		tableViewer.refresh();	
		return true;
	}
}
