// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io;

import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.io.common.CdmDefaultExport;
import eu.etaxonomy.cdm.io.common.IExportConfigurator;
import eu.etaxonomy.cdm.io.common.IIoConfigurator;
import eu.etaxonomy.cdm.io.jaxb.JaxbExportConfigurator;
import eu.etaxonomy.cdm.io.sdd.SDDExportConfigurator;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @created Sep 11, 2009
 * @version 1.0
 */
public class ExportHandler extends AbstractIOHandler{
	private static final Logger logger = Logger
			.getLogger(ExportHandler.class);

	/**
	 * 
	 * @param applicationController
	 */
	private ExportHandler(CdmApplicationController applicationController){
		super(applicationController);
	}
	
	/**
	 * 
	 * @param applicationController
	 * @return
	 */
	public static ExportHandler NewInstance(CdmApplicationController applicationController){
		return new ExportHandler(applicationController);
	}
	
	/**
	 * Starts the export process
	 * 
	 * @param configurator
	 */
	public void run(final IExportConfigurator configurator){
		ProgressMonitorDialog dialog = new ProgressMonitorDialog(StoreUtil.getShell());
		
		try {
			dialog.run(false, true, new IRunnableWithProgress() {

				/*
				 * (non-Javadoc)
				 * @see org.eclipse.jface.operation.IRunnableWithProgress#run(org.eclipse.core.runtime.IProgressMonitor)
				 */
				public void run(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {
					doExport(monitor, configurator);
				}
			});
		}catch (InvocationTargetException e) {
			logger.error(e);
			throw new RuntimeException(e);
		} catch (InterruptedException e) {
			logger.error(e);
			throw new RuntimeException(e);
		}
			
	}
	
	/**
	 * 
	 * @param monitor
	 * @param configurator
	 */
	private void doExport(IProgressMonitor monitor, IExportConfigurator configurator){
		
		Assert.isNotNull(configurator, "Configuration may not be null");
		
		monitor.beginTask("Exporting database. This will take some time.", 100);
		monitor.worked(10);
		
		// terminate any open transactions
		IConversationEnabled activePart = (IConversationEnabled) StoreUtil.getActivePage().getActivePart();
		activePart.getConversationHolder().commit(false);
		monitor.worked(10);
		
		CdmDefaultExport<IExportConfigurator> exporter = new CdmDefaultExport<IExportConfigurator>();
	
		exporter.setCdmAppController(applicationController);
		monitor.worked(10);
		
		try{
			exporter.invoke(configurator);
			monitor.worked(60);
		}catch(RuntimeException e){
			StoreUtil.errorDialog("Error exporting data", "An error occured while" +
					"exporting to destination '" + configurator.getDestinationNameString() + "'.\n" +
							"Please check error log for details.");
			logger.error("Error exporting data", e);
		}
		
		// restarting transaction and committing it to trigger change listener
		// TODO verify correct behaviour
		activePart.getConversationHolder().startTransaction();
		activePart.getConversationHolder().commit();
		monitor.worked(10);
	}
	
	/**
	 * @param jaxb
	 * @return
	 */
	private IIoConfigurator getConfigurator(TYPE type) {
		Assert.isNotNull(type, "Type should not be null");
				
		switch (type){
			case Jaxb:
				return JaxbExportConfigurator.NewInstance(null, null);
			case Sdd:
				return SDDExportConfigurator.NewInstance(null, null, null);
			default:
				StoreUtil.notImplementedMessage();
				throw new IllegalArgumentException("Export not supported yet");
		}
	}
	
	/**
	 * 
	 * @return
	 */
	public final JaxbExportConfigurator JaxbConfigurator() { return (JaxbExportConfigurator) getConfigurator(TYPE.Jaxb);	}

	public final SDDExportConfigurator SddConfigurator() { return (SDDExportConfigurator) getConfigurator(TYPE.Sdd); }

}
