// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.jaxb.JaxbImportConfigurator;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 15.06.2009
 * @version 1.0
 */
public class JaxbImportWizard extends AbstractImportWizard<JaxbImportConfigurator> {
	private static final Logger logger = Logger
			.getLogger(JaxbImportWizard.class);

	private JaxbImportConfigurator configurator;
	
	private ImportFromFileDataSourceWizardPage dataSourcePage;
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.wizard.AbstractImportWizard#getConfigurator()
	 */
	@Override
	public JaxbImportConfigurator getConfigurator() {
		return configurator;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		String file = dataSourcePage.getFile();
		
		// Imports are expecting a string that can be used to instantiate a URL instance
		// TODO consolidate import and exports regarding what may be passed in as source
		// will construct a url string for now
		
		file = "file://" + file;
		
		configurator.setSource(file);
		configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);
		
		CdmStore.getImportHandler().run(configurator);
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		configurator =  CdmStore.getImportHandler().JaxbConfigurator();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.wizard.AbstractImportWizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		dataSourcePage = new ImportFromFileDataSourceWizardPage("Choose File", 
				"Please choose an XML file in the cdm JAXB format.", new String[]{"*.xml"});
		addPage(dataSourcePage);
	}
}
