// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.io.sdd.SDDImportConfigurator;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Jun 16, 2010
 * @version 1.0
 */
public class SddImportWizard extends AbstractImportWizard<SDDImportConfigurator> {
	private static final Logger logger = Logger
			.getLogger(SddImportWizard.class);

	private SDDImportConfigurator configurator;
	
	private ImportFromFileDataSourceWizardPage dataSourcePage;
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.wizard.AbstractImportWizard#getConfigurator()
	 */
	@Override
	public SDDImportConfigurator getConfigurator() {
		return configurator;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPage(org.eclipse.jface.wizard.IWizardPage)
	 */
	@Override
	public void addPage(IWizardPage page) {
		super.addPages();
		
		dataSourcePage = new ImportFromFileDataSourceWizardPage("Choose File", 
				"Please choose an XML file in the SDD format.", new String[]{"*.xml", "*.sdd"});
		addPage(dataSourcePage);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		// TODO this is a quick copy from JAXB import. It might not work at all.
		
		String file = dataSourcePage.getFile();
		
		// Imports are expecting a string that can be used to instantiate a URL instance
		// TODO consolidate import and exports regarding what may be passed in as source
		// will construct a url string for now
		
		file = "file://" + file;
		
		configurator.setSource(file);
		configurator.setDbSchemaValidation(DbSchemaValidation.CREATE);
		
		CdmStore.getImportHandler().run(configurator);
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		configurator =  CdmStore.getImportHandler().SddConfigurator();
	}
}
