// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;

/**
 * Extending <code>TextSelection</code> allows us to keep using Eclipse's text selection, while
 * <code>IStructuredSelection</code> can be used to send an object associated with the selection,
 * i.e. a property source, to the workbench selection service.
 *   
 * @author p.ciardelli
 * @created 03.07.2009
 * @version 1.0
 */
public class LineSelection extends TextSelection implements IStructuredSelection {
	@SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(LineSelection.class);

	private StructuredSelection structuredSelection;
	
	/**
	 * 
	 */
	public LineSelection(ITextSelection selection, IDocument document, Object selectedObject) {
		super(document, selection.getOffset(), selection.getLength());
		if (selectedObject != null) {
			this.structuredSelection = new StructuredSelection(selectedObject);
		} else {
			this.structuredSelection = StructuredSelection.EMPTY;
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#getFirstElement()
	 */
	public Object getFirstElement() {
		return structuredSelection.getFirstElement();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#iterator()
	 */
	public Iterator<?> iterator() {
		return structuredSelection.iterator();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#size()
	 */
	public int size() {
		return structuredSelection.size();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#toArray()
	 */
	public Object[] toArray() {
		return structuredSelection.toArray();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredSelection#toList()
	 */
	public List<?> toList() {
		return structuredSelection.toList();
	}
}
