/**
 * 
 */
package eu.etaxonomy.taxeditor.model;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.PlatformUI;

/**
 * @author n.hoffmann
 *
 */
public class TaxeditorPartService implements IPartListener2{

	public static final int PART_ACTIVATED = 10;
	public static final int PART_BROUGHT_TO_TOP = 20;
	public static final int PART_CLOSED = 30;
	public static final int PART_DEAVTICATED = 40;
	public static final int PART_HIDDEN = 50;
	public static final int PART_INPUT_CHANGED = 60;
	public static final int PART_OPENED = 70;
	public static final int PART_VISIBLE = 80;
	
	public static TaxeditorPartService instance = new TaxeditorPartService();
	
	private IPartService partService;
	
	private Map<Integer, Set<IPartChangeListener>> listenerMap = new HashMap<Integer, Set<IPartChangeListener>>();
	
	private TaxeditorPartService() {
		partService = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService();
		partService.addPartListener(this);
	}
	
	public static TaxeditorPartService getInstance(){
		return instance;
	}
	
	public void addListener(Integer eventType, IPartChangeListener listener){
		Set<IPartChangeListener> partChangeListeners = listenerMap.get(eventType);
		
		if(partChangeListeners == null){
			partChangeListeners = new HashSet<IPartChangeListener>();
			listenerMap.put(eventType, partChangeListeners);
		}
		
		partChangeListeners.add(listener);
	}
	
	public void removeListener(Integer eventType, IPartChangeListener listener){
		Set<IPartChangeListener> partChangeListeners = listenerMap.get(eventType);
		
		if(partChangeListeners == null){
			return;
		}
		
		partChangeListeners.remove(listener);
	}
	
	private void notifyListeners(Integer eventType, IWorkbenchPartReference partRef){
		Set<IPartChangeListener> partChangeListeners = listenerMap.get(eventType);
		
		if(partChangeListeners == null){
			return;
		}
		
		for(IPartChangeListener listener : partChangeListeners){
			listener.partChanged(eventType, partRef);
		}
	}
	

	public void partActivated(IWorkbenchPartReference partRef) {
		notifyListeners(PART_ACTIVATED, partRef);
	}

	public void partBroughtToTop(IWorkbenchPartReference partRef) {
		notifyListeners(PART_BROUGHT_TO_TOP, partRef);
	}

	public void partClosed(IWorkbenchPartReference partRef) {
		notifyListeners(PART_CLOSED, partRef);
	}

	public void partDeactivated(IWorkbenchPartReference partRef) {
		notifyListeners(PART_DEAVTICATED, partRef);
	}

	public void partOpened(IWorkbenchPartReference partRef) {
		notifyListeners(PART_OPENED, partRef);
	}

	public void partHidden(IWorkbenchPartReference partRef) {
		notifyListeners(PART_HIDDEN, partRef);
	}

	public void partVisible(IWorkbenchPartReference partRef) {
		notifyListeners(PART_VISIBLE, partRef);
	}

	public void partInputChanged(IWorkbenchPartReference partRef) {
		notifyListeners(PART_INPUT_CHANGED, partRef);
	}
}
