// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.operations;

import java.lang.reflect.Method;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.IndividualsAssociation;
import eu.etaxonomy.cdm.model.description.QuantitativeData;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;

/**
 * @author e.-m.lee
 * @date 25.05.2010
 *
 */
public class ChangeDescriptionElementType extends AbstractPostOperation {

	private DescriptionElementBase sourceElement;
	private DestinationType destinationType;
	private DescriptionElementBase destinationElement;

	public enum DestinationType {
		TextData,
		Distribution
	}
	
	/**
	 * @param label
	 * @param undoContext
	 */
	public ChangeDescriptionElementType(String label, DescriptionElementBase sourceType, IPostOperationEnabled postOperation, 
			IUndoContext undoContext) {
		super(label, undoContext, postOperation);
		this.sourceElement = sourceType;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		DescriptionBase inDescription = sourceElement.getInDescription();
		Feature feature = sourceElement.getFeature();
		
		if(sourceElement instanceof TextData){
			// FIXME I guess there are features that support more that one of these
			// and that case is not handled correctly at the moment
			if(feature.isSupportsDistribution()){
				destinationElement = Distribution.NewInstance();
			}
			else if(feature.isSupportsCommonTaxonName()){
				destinationElement = CommonTaxonName.NewInstance(null, null);
			}
			else if(feature.isSupportsTaxonInteraction()){
				destinationElement = TaxonInteraction.NewInstance();
			}
			else if(feature.isSupportsIndividualAssociation()){
				destinationElement = IndividualsAssociation.NewInstance();
			}
			else if(feature.isSupportsCategoricalData()){
				destinationElement = CategoricalData.NewInstance();
			}
			else if(feature.isSupportsQuantitativeData()){
				destinationElement = QuantitativeData.NewInstance();
			}
		}
		else{
			if(feature.isSupportsTextData()){
				destinationElement = TextData.NewInstance();
			}
		}
		
		destinationElement.setFeature(feature);
		inDescription.addElement(destinationElement);
		inDescription.removeElement(sourceElement);
		return postExecute(destinationElement);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

}
