// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * @author n.hoffmann
 * @created Sep 1, 2009
 * @version 1.0
 */
public class TaxonomicEditorGeneralPreferences extends PreferencePage implements
		IWorkbenchPreferencePage {
	private static final Logger logger = Logger
			.getLogger(TaxonomicEditorGeneralPreferences.class);

	boolean showUuidInSupplementalView;

	private String editMapServiceAccessPoint;

	private boolean shouldConnectAtStartup;
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		container.setLayout(gridLayout);
		
		createShowUuidInSupplementalView(container);
		createMapServiceAccessPoint(container);
		createShouldConnectAtStartUp(container);
		
		return container;
	}

	private void createMapServiceAccessPoint(Composite composite) {
		editMapServiceAccessPoint = PreferencesUtil.getMapServiceAccessPoint();
		
//		Composite composite = createComposite(parent);
		
		CLabel label = new CLabel(composite, SWT.NULL);
		label.setText("EDIT Map Service Access Point:");
		
		final Text text = new Text(composite, SWT.BORDER);
		text.setText(editMapServiceAccessPoint);
		text.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		text.addModifyListener(new ModifyListener() {
			
			public void modifyText(ModifyEvent e) {
				editMapServiceAccessPoint = text.getText();
			}
		});
	}

	private void createShowUuidInSupplementalView(Composite parent){
		showUuidInSupplementalView = getPreferenceStore().getBoolean(PreferencesUtil.SHOW_DEBUG_INFORMATION);
		
		Composite composite = createComposite(parent);
		
		final Button button_toggle = new Button(composite, SWT.CHECK);		
		
		
		button_toggle.setText("Show UUID and object ID in supplemental data view.");
		
		button_toggle.setSelection(showUuidInSupplementalView);
		
		
		button_toggle.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				showUuidInSupplementalView = button_toggle.getSelection();
			}
		});
		
	}
	
	private void createShouldConnectAtStartUp(Composite parent){
		shouldConnectAtStartup = getPreferenceStore().getBoolean(PreferencesUtil.SHOULD_CONNECT_AT_STARTUP);
		
		Composite composite = createComposite(parent);
		
		final Button button_toggle = new Button(composite, SWT.CHECK);		
		
		
		button_toggle.setText("Connect to last used datasource when launching.");
		
		button_toggle.setSelection(shouldConnectAtStartup);
		
		
		button_toggle.addSelectionListener(new SelectionAdapter(){
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
			 */
			@Override
			public void widgetSelected(SelectionEvent e) {
				shouldConnectAtStartup = button_toggle.getSelection();
			}
		});
	}
	
	private Composite createComposite(Composite parent){
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1));
		composite.setLayout(new RowLayout(SWT.HORIZONTAL));
		return composite;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		setPreferenceStore(PreferencesUtil.getPreferenceStore());
	}

	@Override
	public boolean performOk() {		
		
		getPreferenceStore().setValue(PreferencesUtil.SHOW_DEBUG_INFORMATION, showUuidInSupplementalView);
		getPreferenceStore().setValue(PreferencesUtil.EDIT_MAP_SERVICE_ACCES_POINT, editMapServiceAccessPoint);
		getPreferenceStore().setValue(PreferencesUtil.SHOULD_CONNECT_AT_STARTUP, shouldConnectAtStartup);
		
		return super.performOk();
	}
}
