// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.Representation;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public class VocabularyTermWizard<T extends DefinedTermBase> extends Wizard {
	private static final Logger logger = Logger.getLogger(VocabularyTermWizard.class);
	
	public T term = null;
	
	private VocabularyEnum vocabularyType;
	
	private boolean editMode = false;

	/**
	 * 
	 */
	public VocabularyTermWizard(VocabularyEnum vocabularyType){		
		this.vocabularyType = vocabularyType;
	}
	
	/**
	 * 
	 * @param selectedFeature
	 */
	public VocabularyTermWizard(VocabularyEnum vocabularyType, T selectedTerm){
		this(vocabularyType);		
		this.term = selectedTerm;
		editMode = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		addPage(new VocabularyTermWizardPage());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		ConversationHolder conversation = CdmStore.createConversation();
		
		if(editMode){
			VocabularyStore.updateVocabulary(getTerm());
		}else{
			VocabularyStore.saveTerm(getTerm(), getVocabulary());
		}
		
		conversation.commit(false);
		
		return true;
	}

	public VocabularyEnum getVocabulary(){
		return vocabularyType;
	}

	/**
	 * 
	 * @param term
	 */
	public void setTerm(T term) {
		this.term = term;
	}

	/**
	 * @param text
	 * @param text2
	 * @param text3
	 */
	public void setTerm(String text, String label, String abbreviatedLabel) {
		Class clazz = vocabularyType.getClazz();
		
		try {
			if(getTerm() == null){
				term = (T) clazz.newInstance();
				
				term.addRepresentation(new Representation(text, label, abbreviatedLabel, CdmStore.getDefaultLanguage()));
			}else{
				Representation representation = term.getPreferredRepresentation(CdmStore.getDefaultLanguage());
				representation.setText(text);
				representation.setLabel(label);
				representation.setAbbreviatedLabel(abbreviatedLabel);
			}
			
		} catch (InstantiationException e) {
			logger.error("InstantiationException when setting term", e);
		} catch (IllegalAccessException e) {
			logger.error("IllegalAccessException when setting term", e);
		}
		
	}
	
	/**
	 * @return the term
	 */
	public T getTerm() {
		return term;
	}

	/**
	 * @return the editMode
	 */
	public boolean isEditMode() {
		return editMode;
	}

}
