package eu.etaxonomy.taxeditor.store.internal;

import org.apache.log4j.Logger;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * The activator class controls the plug-in life cycle
 */
public class TaxeditorStorePlugin extends AbstractUIPlugin {

	public static final Logger logger = Logger.getLogger(TaxeditorStorePlugin.class);
	
	// The plug-in ID
	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.editor";

	// The shared instance
	private static TaxeditorStorePlugin plugin;
	
	/**
	 * The constructor
	 */
	public TaxeditorStorePlugin() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;

		PreferencesUtil.setDefaults();
		
		logger.trace("Plugin started: " + this.getBundle().getSymbolicName());
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
		logger.trace("Plugin stopped");
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static TaxeditorStorePlugin getDefault() {
		return plugin;
	}
	
	/**
	 * Returns an image descriptor for the image file at the given
	 * plug-in relative path
	 *
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
	
	/**
	 * Exposes this normally protected method. 
	 */
	public ImageRegistry createImageRegistry(){
		return super.createImageRegistry();
	}
	
	/************************ ERROR CODES ****************************/
	
	/**
	 * An attempt was made to save a taxon with a zero length name
	 */
	public static final int ERROR_SAVING_ZERO_NAME = 52000;
}
