// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.user.view;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 01.07.2009
 * @version 1.0
 */
public class UserManagerView extends ViewPart implements IConversationEnabled, IPostOperationEnabled, IContextListener{
	private static final Logger logger = Logger
			.getLogger(UserManagerView.class);
	
	public static String ID = "eu.etaxonomy.taxeditor.store.userManagerView";
	
	private TableViewer tableViewer;

	private ConversationHolder conversation;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		CdmStore.addContextListener(this);
		
		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;	
		parent.setLayout(fillLayout);
		
		tableViewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		
		getSite().setSelectionProvider(tableViewer);
		
		createColumns(tableViewer);
		
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, tableViewer);

		Control control = tableViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);	
		
		tableViewer.setContentProvider(new UserManagerContentProvider());
		tableViewer.setLabelProvider(new UserManagerLabelProvider());
				
		if(CdmStore.isActive()){
			tableViewer.setInput(getAllUser());
		}
		
		
		this.addListenerObject(new FocusAdapter() {
			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.FocusAdapter#focusGained(org.eclipse.swt.events.FocusEvent)
			 */
			@Override
			public void focusGained(FocusEvent e) {
				super.focusGained(e);
				conversation.bind();
				tableViewer.getControl().setFocus();
			}
		});
	}

	// This will create the columns for the table
	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = {"Active", "Name", "Enabled", "Locked", "Person", "Email"};
		int[] bounds = { 20, 200, 50, 50, 200, 100};

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		if(conversation != null){
			conversation.bind();
		}
		tableViewer.getControl().setFocus();
	}

	
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		if(CdmStore.isActive()){
			List<User> allUsers = getAllUser();
			
			tableViewer.setInput(allUsers);
			tableViewer.refresh();
		}
		
		return true;
	}
	
	/**
	 * 
	 * @return
	 */
	private List<User> getAllUser(){
		conversation = CdmStore.createConversation();
		return CdmStore.getUserService().list(null, null, null, null, null);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	public void update(CdmDataChangeMap changeEvents) {
		// data changes can only be generate here at the moment
	}

	@Override
	public void dispose() {
		CdmStore.removeContextListener(this);
		super.dispose();
	}

	public boolean onContextEvent(EventType eventType) {
		switch(eventType){
		case START:
			tableViewer.setInput(getAllUser());
			break;
		case STOP:
			tableViewer.setInput(null);
			break;
		}
		return true;
	}

	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
	
}
