// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.operations;

import junit.framework.Assert;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation;

/**
 * @author n.hoffmann
 * @created 07.04.2009
 * @version 1.0
 */
public class ChangeConceptToSynonymOperationTest extends AbstractTaxeditorOperationTest {
	private static final Logger logger = Logger
			.getLogger(ChangeConceptToSynonymOperationTest.class);
	
	private static Taxon taxon;
	
	private static Taxon concept;

	private static AbstractPostOperation operation;

	private static HomotypicalGroup homotypicalGroup;
	
	private Synonym[] synonymType = new Synonym[0];
	private Taxon[] taxonType = new Taxon[0];
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		taxon = Taxon.NewInstance(null, null);
		concept = Taxon.NewInstance(NonViralName.NewInstance(null), null);
		homotypicalGroup = HomotypicalGroup.NewInstance();
		
		concept.addTaxonRelation(taxon, TaxonRelationshipType.MISAPPLIED_NAME_FOR(), null, null);
		
		operation = new ChangeConceptToSynonymOperation("Change misapplication to synonym", null, taxon, concept, homotypicalGroup, postOperation);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecuteIProgressMonitorIAdaptable() throws ExecutionException {
		operation.execute(null, null);
		
		Assert.assertTrue(taxon.getSynonyms().size() == 1);
		Assert.assertTrue(concept.getRelationsFromThisTaxon().size() == 0);
		Assert.assertEquals(concept.getName(), taxon.getSynonyms().toArray(synonymType)[0].getName());
		
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.undo(null, null);
		
		Assert.assertTrue("There shouldn't be synonyms after undo operations", taxon.getSynonyms().size() == 0);
		Assert.assertTrue(concept.getRelationsFromThisTaxon().size() > 0);
		Assert.assertEquals(taxon, concept.getRelationsFromThisTaxon().toArray(taxonType)[0]);
	}
	
	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.redo(null, null);
		
		Assert.assertTrue(taxon.getSynonyms().size() == 1);
		Assert.assertTrue(concept.getRelationsFromThisTaxon().size() == 0);
		Assert.assertEquals(concept.getName(), taxon.getSynonyms().toArray(synonymType)[0].getName());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.AbstractPostOperation#postExecute(ICdmBase)}.
	 */
	@Test
	public void testPostExecute() {
		// TODO there is not post operation functionality for this class
		// at the moment. Implement test when there is.
		logger.warn("No post operation functionality for this class");
	}
}
