// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.operations;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.operations.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation;

/**
 * @author n.hoffmann
 * @created 07.04.2009
 * @version 1.0
 */
public class ChangeSynonymToMisapplicationOperationTest extends AbstractTaxeditorOperationTest {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToMisapplicationOperationTest.class);

	private static Synonym synonym;

	private static Taxon taxon;

	private static AbstractPostOperation operation;

	private static SynonymRelationshipType synonymRelationshipType;
	
	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		taxon = Taxon.NewInstance(null, null);
		synonym = Synonym.NewInstance(NonViralName.NewInstance(null), null);
		synonymRelationshipType = SynonymRelationshipType.SYNONYM_OF();
		taxon.addSynonym(synonym, synonymRelationshipType);
		
		operation = new ChangeSynonymToMisapplicationOperation("Change Synonym To Misapplication", null, taxon, synonym, postOperation);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testExecuteIProgressMonitorIAdaptable() throws ExecutionException {
		operation.execute(null, null);
		
		Assert.assertTrue("Synonym should be deleted from taxon", taxon.getSynonyms().size() == 0);
		Assert.assertTrue("Taxon should have misapplications", taxon.getMisappliedNames().size() > 0);
		Assert.assertEquals("The name of synonym and misapplication should be equal", synonym.getName(), taxon.getMisappliedNames().toArray(new Taxon[0])[0].getName());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testUndoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.undo(null, null);
		
		Assert.assertTrue("Taxon should have synonyms.", taxon.getSynonyms().size() > 0);
		Assert.assertTrue("Taxon should not have taxon relationship.", taxon.getTaxonRelations().size() == 0);
		Assert.assertEquals("Not the expected synonym.", synonym, taxon.getSynonyms().toArray(new Synonym[0])[0]);
		Assert.assertEquals("SynonymRelationshipType is not the expected.", synonymRelationshipType, taxon.getSynonyms().toArray(new Synonym[0])[0].getRelationType(taxon));
	}
	
	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.ChangeSynonymToMisapplicationOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException 
	 */
	@Test
	public void testRedoIProgressMonitorIAdaptable() throws ExecutionException {
		operation.redo(null, null);

		Assert.assertTrue("Synonym should be deleted from taxon", taxon.getSynonyms().size() == 0);
		Assert.assertTrue("Taxon should have misapplications", taxon.getMisappliedNames().size() > 0);
		Assert.assertEquals("The name of synonym and misapplication should be equal", synonym.getName(), taxon.getMisappliedNames().toArray(new Taxon[0])[0].getName());
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.operations.AbstractPostOperation#postExecute(ICdmBase)}.
	 */
	@Test
	public void testPostExecute() {
		// TODO there is not post operation functionality for this class
		// at the moment. Implement test when there is.
		logger.warn("No post operation functionality for this class");
	}
}
